import java.io.*;

/**
 * This class expands one line multi-CNOTs
 * using various compilation modes
 */
public class MultiCNotExpander extends ElemOpExpander{
       
	public final static int 
	    TOFFOLIS=0, 
	    CNOTS=1;//compilation modes

    /**
     * 
     */
	MultiCNotExpander(
			String 	filePrefixIn,
			String 	filePrefixOut,
			int numGitsIn,
			int compiMode)
	{
		super(filePrefixIn, filePrefixOut, numGitsIn);
		
		Throw.If(compiMode!=TOFFOLIS, 
		 	"Your choice of multiCNot compilation mode is not supported yet");
				
		try{
			openFiles();

			String lineName = "";
			while(_englishIn.hasNext()){
				lineName = _englishIn.next();
				if(lineName.equals("LOOP")){
					readWriteLOOP();
				}else if(lineName.equals("NEXT")){
					readWriteNEXT();
				}else if(lineName.equals("SIGX")){
					readWriteSIGX(compiMode);
				}else{//not LOOP or NEXT or SIGX
					String	endOfEngLine = _englishIn.nextLine();				
					echoEngAndPicLines(lineName + "\t" + endOfEngLine);
					_gm.note(lineName, endOfEngLine);
				}
			}
			closeFiles();
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
	}
    /**
	 * 
	 */
	public int getNumGitsOut(){
		//should not depend on _numBitsIn
		return _numGitsIn;		
	}
    /**
	 * 
	 */
    private void readWriteSIGX(int compiMode){
     	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated 
    		//and a GitsMonitor is attached to Expander's _gm
    	    	
    	_englishIn.next();//skip "AT"
        int targetBitPos = _englishIn.nextInt();
        
        TFControls trols = new TFControls();
        int numTrols;
        if(!_englishIn.hasNext("IF")){
        	numTrols = 0;      	
        }else{
        	trols = SEO_reader.readTFControls(_numBitsIn, _englishIn);
        	numTrols = trols.getNumControls();
        }

    	//writing        
    	
    	if(numTrols <= 2){
    		SEO_writer 	wr = new SEO_writer(
    				_englishOut, _pictureOut, new CktEmbedder(_numBitsIn));
    		wr.setGitsMonitor(_gm);
    		wr.writeControlledOneBitGate(targetBitPos, trols, new SigX());
    	}else{
    		int workSpaceSize = _numBitsIn - numTrols - 1;
    		boolean[] controlTypes = new boolean[numTrols];
    		int j, lineCountBef;
    		int numBitsBef;
    		for(j=0; j<numTrols; j++){
    			controlTypes[j] = trols.getType(numTrols - 1 - j);
    		}

    		if((workSpaceSize >= (numTrols-1))){//numTrols>=3, numBitsIn>=6
	    		numBitsBef = 2*numTrols;
	    		lineCountBef = SEO_writer.getLineCount();
	    		new MultiCNotTof1SEO_writer(
	    			_englishOut,
	    			_pictureOut,
	    			true,//doWrite
	    			getEmbThatTakesFloatedToNative(numBitsBef, targetBitPos, trols),
	    			controlTypes,
	    			_gm);
	    		_lineNumAdvance += SEO_writer.getLineCount() -lineCountBef -1;
    		}else if(workSpaceSize >= 1){//numTrols>=3, numBitsIn>=5
	     		numBitsBef = numTrols + 2;
	    		lineCountBef = SEO_writer.getLineCount();
	    		new MultiCNotTof2SEO_writer(
	    			_englishOut,
	    			_pictureOut,
	    			true,//doWrite
	    			getEmbThatTakesFloatedToNative(numBitsBef, targetBitPos, trols),
	    			controlTypes,
	    			_gm);
	    		_lineNumAdvance += SEO_writer.getLineCount() -lineCountBef -1;
   		
	    	}else{//workSpaceSize==0
	    		Throw.If(true, "need one more qubit to use as work space");
	    	}
    	}
	    	                 	  	
    }
    /**
	 * 
	 */
    private CktEmbedder getEmbThatTakesFloatedToNative(
    		int numBitsBef, 
    		int targetBitPos,
    		TFControls trols)
    {
    	//starting state has all controls floated to top (low bit pos)
    	//and target sunk to bottom (highest bit pos)
    	int numBitsAft = _numBitsIn;  	
    	int[] bitMap = new int[numBitsBef];
     	boolean[] isP0Control = Globals.F_vec(numBitsAft);
    	boolean[] isP1Control = Globals.F_vec(numBitsAft);
    	int jIn, jOut ;
    	int c, numControls = trols.getNumControls();
    	
    	jIn = 0;
    	c = numControls -1;
    	for(jOut=0; jOut<_numBitsIn; jOut++){
    		if(c>=0 && trols.getBitPos(c)==jOut){
    			bitMap[jIn] = jOut;
    			jIn++;    			
    			c--;
    		}
    	}
    	Throw.If(jIn!=numControls, 
    			"jIn at this point should be numControls");
    	c= numControls -1;
    	for(jOut=0; jOut<_numBitsIn; jOut++){
    		if(c>=0 && trols.getBitPos(c)==jOut){
    			c--;
    		}else if(jOut==targetBitPos){
    			bitMap[numBitsBef-1] = jOut;
    		}else{
    			bitMap[jIn] = jOut;
    			jIn++;
    			if(jIn==numBitsBef)break;
    		}
    	}

  	
    	
		CktEmbedder emb = new CktEmbedder(
			numBitsBef,
			numBitsAft,
			bitMap,
			isP0Control,
			isP1Control );
		return emb;
	}
	
 }
