import java.io.*;

/**
 * This class writes English and Picture files,
 * in the one line format
 * for a multi-CNOT with
 * 
 * control bits at 0, 1, 2 , ..., numControls-1, and
 * a target bit at numBits - 1 .
 * 
 * (hence numControls<=numBits-1)
 */


public class MultiCNotOneLineSEO_writer  extends SEO_writer{
	boolean[] 		_controlTypes;

	/**
	 * 
	 */	
	public MultiCNotOneLineSEO_writer(
	        String 			filePrefix, 
	        int 			numBits,
	        boolean[] 		controlTypes)
	{ 	
	 	
    	super(filePrefix, numBits);
    	
      	Throw.If(_numBits<2 , 
			"multiCNot must have at least 2 qubits (1 control, 1 target)");
 
     	Throw.If(controlTypes.length > (_numBits-1) , 
			"multiCNot must have numControls <= (numBits-1)");
		
    	_controlTypes = controlTypes;
    	
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MultiCNotOneLineSEO_writer(
		PrintWriter  	englishOut,
		PrintWriter  	pictureOut,
		Boolean 		doWrite,
		CktEmbedder 	emb,
	    boolean[] 		controlTypes)
    {
    	super(englishOut, pictureOut, emb);
    	//numBitsBef = _numBits
 
     	Throw.If(_numBits<2 , 
			"multiCNot must have at least 2 qubits (control, target)");
 
     	Throw.If(controlTypes.length > (_numBits-1) , 
			"multiCNot must have numControls <= (numBits-1)");


    	if(doWrite){write();}
    }
      	
		
	/**
	 * 
	 */
	public void write(){

		int targetBitPosition = _numBits -1;
		int numControls = _controlTypes.length;
		
		TFControls controls = new  TFControls(_numBits, numControls);
		int bitPos;
		for(int c=0; c<numControls; c++){
			bitPos = numControls-c-1;
			controls.setControl(c, bitPos , _controlTypes[bitPos]);
		}
		

		writeControlledOneBitGate(
				targetBitPosition,
				controls,
				new SigX());
	}
	
	/**
	 * 
	 */ 
   public static void main(String[] args){
	   
	   int numBits = 5;
	   boolean[] controlTypes ={true, true, false};
       
       new MultiCNotOneLineSEO_writer(
           "oneLine",// filePrefix, 
           numBits,
           controlTypes);
          
       
   }
}
