/**
 * This class returns entries of a
 * multi-CNOT matrix which has
 * 
 * control bits at 0, 1, 2 , ..., numControls-1, and
 * a target bit at numBits - 1 .
 * 
 * (hence numControls<=numBits-1)
 */

public class MultiCNotUinMat {
    private int 		_numBits;
    private boolean[]  	_controlTypes;
	/**
	 * 
	 */
    public MultiCNotUinMat(int numBits, boolean[] controlTypes) {
    	
     	Throw.If(numBits<2 , 
			"multiCNot must have at least 2 qubits (1 control, 1 target)");
 
     	Throw.If(controlTypes.length > (numBits-1) , 
			"multiCNot must have numControls <= (numBits-1)");
   	
     	_numBits = numBits;
        _controlTypes = controlTypes;
    }
	/**
	  * 
	  */
	public String toString(){
       int numRows = 1<<_numBits;
       String output = "";
       output += "numRows = "+ numRows + '\n';
       int row, col;
       for(row=0; row<numRows; row++){
    	   for(col=0; col<numRows; col++){
    		   output += getEntry(row, col) + " ";
    	   }
    	   output += '\n';
       }
       return output;
   }
	/**
	 *
	 */
	public int getEntry(int row, int col) {
		
		int j;
		int numControls =_controlTypes.length;
		for(j=0; j<(_numBits-1); j++){
			if( ((col>>j)&1) != ((row>>j)&1) ){
				return 0;
			}
		}
		
		int rowBitLast = (row>>(_numBits-1))&1;
		int colBitLast = (col>>(_numBits-1))&1;
		int prod = 1;
		int factor;
		for(j=0; j<numControls; j++){
				factor = (col>>j)&1;
			if(_controlTypes[j]==false){
				factor = 1 - factor;
			}
			prod *= factor;	
		}
		if( rowBitLast != ((colBitLast + prod)%2) ){
			return 0;
		}
		
		return 1;
	}
	/**
	* 
	*/    
	public static void main(String[] args){
		
		int numBits = 4; boolean[] controlTypes = {true, true};
		//int numBits = 6; boolean[] controlTypes = {false, false, false};
		
		System.out.println("numBits = " + numBits);
		System.out.print("controlTypes = ");
		for(int j=controlTypes.length-1; j>=0; j--){
			System.out.print(j + (controlTypes[j]==true?"T":"F") + " ");
		}
		System.out.print("\n");
		MultiCNotUinMat mat = new MultiCNotUinMat(numBits, 
				controlTypes);
        System.out.println(mat.toString());
      
   }

}
