import java.io.*;

/**
 * If Op is a 1-qubit operation different from sig_x or a multipexor, 
 * this class replaces (Op with more than one controls) by
 * a product of multiCNots and two (Op with only one control).
 * 
 * Only line types remaining after this expander is applied
 * LOOP
 * NEXT
 * SWAP (numControls=0)
 * PHAS (numControls=0)
 * P0PH, P1PH (numControls<=1)
 * SIGX (numControls arbitrary)
 * SIGY, SIGZ (numControls=0)
 * HAD2 (numControls=0)
 * ROTX, ROTY, ROTZ, ROTN (numControls<=1)
 * MP_Y (numControls arbitrary)
 * 
 */
public class MultiCRotExpander extends ElemOpExpander{
       
    /**
     * 
     */
	MultiCRotExpander(
			String 	filePrefixIn,
			String 	filePrefixOut,
			int numGitsIn)
	{
		super(filePrefixIn, filePrefixOut, numGitsIn);
		
		try{
			openFiles();

			String lineName = "";
			while(_englishIn.hasNext()){
				lineName = _englishIn.next();
				if(lineName.equals("LOOP")){
					readWriteLOOP();
				}else if(lineName.equals("NEXT")){
					readWriteNEXT();
				}else if(lineName.equals("SWAP")){
					readWriteSWAP();				
				}else if(lineName.equals("PHAS")){
					readWritePHAS();
				}else if(lineName.equals("P0PH")){
					readWriteP_PH(0);
				}else if(lineName.equals("P1PH")){
					readWriteP_PH(1);
				}else if(lineName.equals("SIGX")|| lineName.equals("MP_Y") ){
					String	endOfEngLine = _englishIn.nextLine();
					_gm.note(lineName, endOfEngLine);
					echoEngAndPicLines(lineName + "\t" + endOfEngLine);
				}else if(lineName.equals("SIGY")){
					readWriteSigYZ(OneBitRot.Y_AXIS);
				}else if(lineName.equals("SIGZ")){
					readWriteSigYZ(OneBitRot.Z_AXIS);
				}else if(lineName.equals("HAD2")){
					readWriteHAD2();
				}else if(lineName.equals("ROTX")){
					readWriteROT(OneBitRot.X_AXIS);
				}else if(lineName.equals("ROTY")){
					readWriteROT(OneBitRot.Y_AXIS);
				}else if(lineName.equals("ROTZ")){
					readWriteROT(OneBitRot.Z_AXIS);
				}else if(lineName.equals("ROTN")){
					readWriteROTN();
				}else{
					Throw.If(true, "unsupported line name");
				}
					
			}
			closeFiles();
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
	}
     /**
	 * 
	 */
	public int getNumGitsOut(){
		//should not depend on _numBitsIn
		int numGitsOut = _numGitsIn;
		if(_numGitsIn<1){numGitsOut = 1;}
		return numGitsOut;		
	}
   /**
	 * 
	 */
    private void writeGenericOneBitGate(
    	int targetBitPos, TFControls trols, OneBitGate uGate)
    {
    	//writing
    	int numTrols = trols.getNumControls();
    	
    	SEO_writer 	wr = new SEO_writer(
        	_englishOut, _pictureOut, new CktEmbedder(getNumBitsOut()));
    	wr.setGitsMonitor(_gm);
        
        if(numTrols==0){
        	wr.writeOneBitGate(targetBitPos, uGate);
        }else{//numTrols>=1   		    		
    		if(numTrols==1){
    			wr.writeControlledOneBitGate(targetBitPos, trols, uGate);  
    		}else{//numTrols>=2
    			OneBitGate  xx = new SigX();
    			int workSpaceBitPos = trols.getWorkSpaceBitPos(targetBitPos);
    			_gm.checkIsGit(workSpaceBitPos);
    			TFControls uTrol = new TFControls(getNumBitsOut(), 1);  
	        	uTrol.setControl(0, workSpaceBitPos, true);
	        	wr.writeControlledOneBitGate(workSpaceBitPos, trols, xx);
	        	wr.writeControlledOneBitGate(targetBitPos, uTrol, uGate);
	        	wr.writeControlledOneBitGate(workSpaceBitPos, trols, xx);
    			_lineNumAdvance +=2;
    		}
	    }
    }
    /**
	 * 
	 */
    private void readWriteSWAP(){
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated 
    		//and a GitsMonitor is attached to Expander's _gm
    	
        int targetBitPos1 = _englishIn.nextInt();
        int targetBitPos2 = _englishIn.nextInt();

        TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
        int numTrols = trols.getNumControls();
        
        //writing
        SEO_writer 	wr = new SEO_writer(
        	_englishOut, _pictureOut, new CktEmbedder(getNumBitsOut()));
        wr.setGitsMonitor(_gm);
        if(numTrols==0){
        	wr.writeBitSwap(targetBitPos1, targetBitPos2);
        }else{
        	
        	//<--->---@---@
        	//equals
        	//@---X---@---@
        	//X---@---@---@
        	//@---X---@---@
        	//which equals
        	//@---X
        	//X---@---@---@
        	//@---X
        	
        	OneBitGate xx = new SigX();
        	TFControls sideTrol = new TFControls(getNumBitsOut(), 1);
        	sideTrol.setControl(0, targetBitPos1/*bitPos*/, true);
        	
        	//add to trols a new true controls at targetBitPos2
        	//But remember, entries of bitPos array of TFControls must be
        	//in decreasing order
        	TFControls centerTrols = 
        		new TFControls(getNumBitsOut(), numTrols + 1);
        	int c, cNew;
        	int bitPosPrev, bitPos;
        	c = 0;
        	bitPosPrev = _numBitsIn + 5;
        	bitPos = trols.getBitPos(0);
        	for(cNew=0; cNew <= numTrols; cNew++){
        		if(bitPosPrev > targetBitPos2 && targetBitPos2>bitPos){
        			centerTrols.setControl(cNew, targetBitPos2, true);
        			bitPosPrev = targetBitPos2;
        		}else{
        			centerTrols.setControl(
        					cNew, bitPos, trols.getType(c));
        			bitPosPrev = bitPos;
        			c++;
        			if(c<numTrols){
        				bitPos = trols.getBitPos(c);
        			}else{
        				bitPos = -1;
        			}
        		}
        	}
        	
        	wr.writeControlledOneBitGate(targetBitPos2, sideTrol, xx);
        	wr.writeControlledOneBitGate(targetBitPos1, centerTrols, xx);
        	wr.writeControlledOneBitGate(targetBitPos2, sideTrol, xx);
        	_lineNumAdvance +=2;
        }
 	
    }
    /**
	 * 
	 */
    private void readWritePHAS(){
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	double angleDeg = _englishIn.nextDouble();
    	double angleRad = angleDeg*Math.PI/180;
    	_englishIn.next();//skip "AT"
    	
    	_englishIn.nextInt();//skip targetBitPos, which is never used
    	
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
		int numTrols = trols.getNumControls();
        
        //writing
        SEO_writer 	wr = new SEO_writer(
        	_englishOut, _pictureOut, new CktEmbedder(getNumBitsOut()));
        wr.setGitsMonitor(_gm);
           
        if(numTrols==0){
        	wr.writeGlobalPhaseFactor(angleRad);
        }else{//numTrols>=1
        	int uTarBitPos = trols.getBitPos(0);
        	OneBitGate 	uGate;
        	if(trols.getType(0)==true){
        		uGate = new OneBitP1PhaseFactor(angleRad);
        	}else{
        		uGate = new OneBitP0PhaseFactor(angleRad);
        	}
        	if(numTrols==1){
        		wr.writeOneBitGate(uTarBitPos, uGate);
        	}else{//numTrols>=2 
        		//red = reduced
    			TFControls redTrols = 
    				new TFControls(getNumBitsOut(), numTrols - 1);
    			int c;
	        	for(c=1; c<numTrols; c++){
	        		redTrols.setControl(
	        				c-1, trols.getBitPos(c), trols.getType(c));
	        	}
	        	writeGenericOneBitGate(uTarBitPos, redTrols, uGate);

	        }
        }
    }


    /**
	 * 
	 */
    private void readWriteP_PH(int type){
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	double angleDeg = _englishIn.nextDouble();
    	double angleRad = angleDeg*Math.PI/180;
    	_englishIn.next();//skip "AT"
    	int targetBitPos = _englishIn.nextInt();
    	
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
                    
    	OneBitGate 	uGate;
    	if(type==1){
    		uGate = new OneBitP1PhaseFactor(angleRad);
    	}else{
    		uGate = new OneBitP0PhaseFactor(angleRad);
    	}
    	
    	//writing
    	writeGenericOneBitGate(targetBitPos, trols, uGate);

    }
    /**
	 * 
	 */
    private void readWriteSigYZ(int sigmaAxis){
      	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	_englishIn.next();//skip "AT"
        int targetBitPos = _englishIn.nextInt();
        
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
        int numTrols = trols.getNumControls();

        
        //writing
        SEO_writer 	wr = new SEO_writer(
        	_englishOut, _pictureOut, new CktEmbedder(getNumBitsOut()));
        wr.setGitsMonitor(_gm);
        double piOver4 = Math.PI/4;
        if(sigmaAxis==OneBitRot.Y_AXIS){
        	if(numTrols==0){
        		wr.writeControlledOneBitGate(targetBitPos, trols, new SigY());
        	}else{
	        	wr.writeOneBitGate(targetBitPos, 
	        		new OneBitRot(piOver4, OneBitRot.Z_AXIS));
	        	wr.writeControlledOneBitGate(targetBitPos, trols, new SigX());
	        	wr.writeOneBitGate(targetBitPos, 
	        		new OneBitRot(-piOver4, OneBitRot.Z_AXIS));
	        	_lineNumAdvance +=2;
        	}
        }else if(sigmaAxis==OneBitRot.Z_AXIS){
        	if(numTrols==0){
        		wr.writeControlledOneBitGate(targetBitPos, trols, new SigZ());
        	}else{
	        	wr.writeOneBitGate(targetBitPos, 
	        		new OneBitRot(-piOver4, OneBitRot.Y_AXIS));
	        	wr.writeControlledOneBitGate(targetBitPos, trols, new SigX());
	        	wr.writeOneBitGate(targetBitPos, 
	        		new OneBitRot(piOver4, OneBitRot.Y_AXIS));
	        	_lineNumAdvance +=2;
        	}
        }
    	
    }
    /**
	 * 
	 */
    private void readWriteHAD2()
    {
      	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	_englishIn.next();//skip "AT"
        int targetBitPos = _englishIn.nextInt();
        
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
        int numTrols = trols.getNumControls();

    	//writing
    	
    	SEO_writer 	wr = new SEO_writer(
        	_englishOut, _pictureOut, new CktEmbedder(getNumBitsOut()));
    	wr.setGitsMonitor(_gm);
        
        if(numTrols==0){
        	wr.writeOneBitGate(targetBitPos, new OneBitHadamard());
        }else{//numTrols>=1
        	OneBitGate uGate = new OneBitRot(Math.PI/4, OneBitRot.Y_AXIS);
        	OneBitGate xx = new SigX();
        	
    		if(numTrols==1){
    			wr.writeControlledOneBitGate(targetBitPos, trols, xx);
    			wr.writeControlledOneBitGate(targetBitPos, trols, uGate);
    			_lineNumAdvance +=1;
    		}else{//numTrols>=2    			
    			int workSpaceBitPos = trols.getWorkSpaceBitPos(targetBitPos);		
     			_gm.checkIsGit(workSpaceBitPos);
    			TFControls uTrol = new TFControls(getNumBitsOut(), 1);  
	        	uTrol.setControl(0, workSpaceBitPos, true);
        		wr.writeControlledOneBitGate(workSpaceBitPos, trols, xx);
        		wr.writeControlledOneBitGate(targetBitPos, uTrol, xx);
        		wr.writeControlledOneBitGate(targetBitPos, uTrol, uGate);
        		wr.writeControlledOneBitGate(workSpaceBitPos, trols, xx);
    			_lineNumAdvance +=3;
    		}
	    }
    }
    /**
	 * 
	 */
    private void readWriteROT(int direction){
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	double angleDeg = _englishIn.nextDouble();
    	double angleRad = angleDeg*Math.PI/180;
    	_englishIn.next();//skip "AT"
    	int targetBitPos = _englishIn.nextInt();
    	
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
                    
    	OneBitGate 	uGate = new OneBitRot(angleRad, direction);
    	
    	//writing
    	writeGenericOneBitGate(targetBitPos, trols, uGate);

    }
    /**
	 * 
	 */
    private void readWriteROTN(){
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm

    	double angleX_deg = _englishIn.nextDouble();
    	double angleY_deg = _englishIn.nextDouble();
    	double angleZ_deg = _englishIn.nextDouble();

    	double angleX_rad = angleX_deg*Math.PI/180;
    	double angleY_rad = angleY_deg*Math.PI/180;
    	double angleZ_rad = angleZ_deg*Math.PI/180;

    	_englishIn.next();//skip "AT"
    	int targetBitPos = _englishIn.nextInt();
    	
    	TFControls trols = new TFControls(
        	SEO_reader.readTFControls(_numBitsIn, _englishIn),
        	getNumBitsOut()-_numBitsIn);
                    
    	OneBitGate 	uGate = new OneBitRot(
    			angleX_rad, angleY_rad, angleZ_rad);
    	
    	//writing
    	writeGenericOneBitGate(targetBitPos, trols, uGate);

    }

}
