import java.io.*;

/**
 * This class expands one line multiplexors
 * using various compilation modes
 */
public class MultiplexorExpander extends ElemOpExpander{

	private int _compiMode;
	private int _bitPrecision;
   
       
	public final static int 
	    EXACT_SEO=0, 
	    ORACULAR_APPROX=1,
	    SUZ_TROTTER_APPROX=2;//compilation modes
    /**
     * 
     */
	MultiplexorExpander(
			String 	filePrefixIn,
			String 	filePrefixOut,
			int numGitsIn,
			int compiMode,
			int bitPrecision)
	{
		super(filePrefixIn, filePrefixOut, numGitsIn);
		
		Throw.If(compiMode!=ORACULAR_APPROX && compiMode!=EXACT_SEO, 
		 	"Your choice of multiplexor compilation mode is not supported yet");
    	_compiMode = compiMode;
		_bitPrecision = bitPrecision;
				
		try{
			openFiles();

			String lineName = "";
			while(_englishIn.hasNext()){
				lineName = _englishIn.next();
				if(lineName.equals("LOOP")){
					readWriteLOOP();
				}else if(lineName.equals("NEXT")){
					readWriteNEXT();
				}else if(lineName.equals("MP_Y")){
					readWriteMP_Y(compiMode);
				}else{//not LOOP or NEXT or MP_Y
					String	endOfEngLine = _englishIn.nextLine();
					echoEngAndPicLines(lineName + "\t" + endOfEngLine);
					_gm.note(lineName, endOfEngLine);
				}
			}
			closeFiles();
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
	}
      /**
	 * 
	 */
	public int getNumGitsOut(){
		//should not depend on _numBitsIn
		int numGitsOut = _numGitsIn;
		if(_compiMode==ORACULAR_APPROX && _numGitsIn<_bitPrecision){
			numGitsOut = _bitPrecision;
		}
		return numGitsOut;		
	}
   /**
	 * 
	 */
    private void readWriteMP_Y(int compiMode){
    	
    	//reading
    	readMeasurePictureIn();
    		//do this first to assure numBitsIn is calculated
    		//and a GitsMonitor is attached to Expander's _gm
    	
    	_englishIn.next();//skip "AT"
        int targetBitPosition = _englishIn.nextInt();
        
        MultiControls trols = new MultiControls();
    	trols = SEO_reader.readMultiControls(getNumBitsOut(), _englishIn);
    	int numTrols = trols.getNumControls();
    	int numNonNegTrols = trols.getNumNonNegControls();

    	double[] angles = SEO_reader.readMP_Y_angles(_englishIn);
    	
    	//writing
    	//for emb below
    	int numBitsBef = numNonNegTrols + getNumGitsOut() + 1;
    	int numBitsAft = getNumBitsOut();
    	int[] bitMap = new int[numBitsBef];
    	boolean[] isP0Control = Globals.F_vec(numBitsAft);
    	boolean[] isP1Control = Globals.F_vec(numBitsAft);
    	int j;
    	for(j=0; j<numTrols; j++){
    		int trolType = trols.getType(j);
    		int bitPos =  trols.getBitPos(j);
    		if(trolType==MultiControls.P0_TYPE){
    			isP0Control[bitPos]=true;
    		}else if(trolType==MultiControls.P1_TYPE){
    			isP1Control[bitPos]=true;
    		}else if(trolType>=0){
    			bitMap[trolType]=bitPos;   			
    		}
    	}
    	bitMap[numNonNegTrols]=targetBitPosition;
    	for(j=numNonNegTrols + 1; j<numBitsBef; j++){
    		//sink to bottom of after pile
    		bitMap[j] = j + numBitsAft - numBitsBef;
    	}
    	
    	
    	CktEmbedder emb = new CktEmbedder(
    		numBitsBef,
			numBitsAft,
			bitMap,
			isP0Control,
			isP1Control );
    	int lineCountBef = SEO_writer.getLineCount();
    	switch(compiMode){
    		case(ORACULAR_APPROX):
		    	new MultiplexorOracularSEO_writer(
		    			_englishOut,
		    			_pictureOut,
		    			true,//doWrite
		    			emb,  
		    	        new DoubleArray(angles),
		    	  	    getNumGitsOut(),
		    	  	    _bitPrecision);
    			break;
    		case(EXACT_SEO):
    			new MultiplexorExactSEO_writer(
		    			_englishOut,
		    			_pictureOut,
		    			true,//doWrite
		    			emb,  
		    	        angles);
    			break;
    	}
    	_lineNumAdvance += SEO_writer.getLineCount() -lineCountBef -1;
  	
    }
	
 }
