import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

/**
 * This class defines the GUI for the main and 
 * only window (Control Panel) of Multiplexor Expander.
 */
public class MultiplexorExpanderApplet extends JApplet{
	private JLabel 		_L_image;
	private JTextField	_TF_filePrefixIn;
	private JTextField	_TF_filePrefixOut;
	private JTextField 	_TF_numGitsIn;
	private JComboBox 	_CB_compiMode;
	private JComboBox 	_CB_bitPrecision;
	private JButton		_B_doIt;
	private JTextField 	_TF_numNonGits;
	private JTextField 	_TF_numGitsOut;
	private JTextField	_TF_lenOfSEO;
//	private JTextField	_TF_error;
	/**
	 * 
	 */
	public void init(){
		java.net.URL iconURL = getClass().getResource("muxorExp-logo.gif");
		_L_image = new JLabel(new ImageIcon(iconURL));
		_TF_filePrefixIn = new JTextField();
		_TF_filePrefixOut = new JTextField();
		_TF_numGitsIn = new JTextField();
		_CB_compiMode = new JComboBox(new String[]
		    {"Exact SEO", "Oracular Approximation"});
		_CB_bitPrecision = new JComboBox(new String[]
		    {"1", "2", "3", "4", "5", "6", "7", "8"});
		_B_doIt = new JButton("Write Output Files");
		_TF_numNonGits = new JTextField();
			_TF_numNonGits.setEditable(false);
		_TF_numGitsOut = new JTextField();
			_TF_numGitsOut.setEditable(false);
		_TF_lenOfSEO = new JTextField();
			_TF_lenOfSEO.setEditable(false);
		
		JPanel panHeader = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panHeader.add(_L_image, BorderLayout.CENTER);
		panHeader.add(new JLabel(Globals.VERSION), BorderLayout.SOUTH);
		
		JPanel panIn = new JPanel(new GridLayout(5, 2));//rows, cols
		panIn.add(new JLabel("Prefix for Input Files"));
			panIn.add(_TF_filePrefixIn);
		panIn.add(new JLabel("Prefix for Output Files"));
			panIn.add(_TF_filePrefixOut);
		panIn.add(new JLabel("Number of Grounded Bits In"));
			panIn.add(_TF_numGitsIn);
		panIn.add(new JLabel("Compilation Mode"));
			panIn.add(_CB_compiMode);
		panIn.add(new JLabel("Bit Precision"));
			panIn.add(_CB_bitPrecision);
		panIn.setBorder(new TitledBorder("Inputs"));
		
		JPanel panWriteFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panWriteFlow.add(_B_doIt);

		JPanel panInPlus = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panInPlus.add(panHeader, BorderLayout.NORTH);
		panInPlus.add(panIn, BorderLayout.CENTER);
		panInPlus.add(panWriteFlow, BorderLayout.SOUTH);
		
		JPanel panOut = new JPanel(new GridLayout(3, 2));//rows, cols
		panOut.add(new JLabel("Number of Non-grounded Bits"));
			panOut.add(_TF_numNonGits);
		panOut.add(new JLabel("Number of Grounded Bits Out"));
			panOut.add(_TF_numGitsOut);
		panOut.add(new JLabel("Number of Elem. Ops. Out"));
			panOut.add(_TF_lenOfSEO);
		panOut.setBorder(new TitledBorder("Outputs"));
		
		setLayout(new BorderLayout(2, 2));//hgap, vgap
		add(panInPlus, BorderLayout.CENTER);
		add(panOut, BorderLayout.SOUTH);
		
		_B_doIt.addActionListener(new ButtonListener());
	}
	
	/**
	 *
	 */
	private class ButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {

			String  filePrefixIn;
			String	filePrefixOut;
			int numGitsIn;
			int compiMode=-1; 
			int bitPrecision;
			
			try{
				if(	_TF_filePrefixIn.getText().length()==0 ||
					_TF_filePrefixOut.getText().length()==0)
				{ throw new Exception();}
				
				filePrefixIn = _TF_filePrefixIn.getText();
				filePrefixOut = _TF_filePrefixOut.getText();
				numGitsIn = Integer.parseInt(
						_TF_numGitsIn.getText());
				switch(_CB_compiMode.getSelectedIndex()){
					case(0):
						compiMode = MultiplexorExpander.EXACT_SEO;
						break;
					case(1):
						compiMode = MultiplexorExpander.ORACULAR_APPROX;
						break;
				}
				bitPrecision =  _CB_bitPrecision.getSelectedIndex() + 1;								
				
				MultiplexorExpanderMain  m  =  new MultiplexorExpanderMain(
						filePrefixIn, 
						filePrefixOut,
						numGitsIn,
						compiMode, 
						bitPrecision);

				_TF_numNonGits.setText(String.format("%d", 
						m.getExpander().getNumNonGits()));
				_TF_numGitsOut.setText(String.format("%d", 
						m.getExpander().getNumGitsOut()));
				_TF_lenOfSEO.setText(String.format("%d", m.getLenOfSEO()));
				
			}
			catch(Exception ex){
				_TF_lenOfSEO.setText("");
				JOptionPane.showMessageDialog(null, 
						"MultiplexorExpander got BAD INPUT:\n" + ex.toString());
				return;
			}
		}		
	}
	/**
	 *
	 */
	public static void main(String[] args) {
	    
	    JFrame frame = new JFrame("Multiplexor Expander");

	    
	    MultiplexorExpanderApplet applet = new MultiplexorExpanderApplet();

	    frame.add(applet, BorderLayout.CENTER);

	    applet.init();

	    //frame.setLocationRelativeTo(null); // Center the frame
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	    
	    frame.pack();//frame.setSize(400, 250);
	    frame.setVisible(true);
	  }


}

