import java.io.*;

/**
 * This class writes English and Picture files,
 * in the one line format
 * for a multiplexor of the form 
 * [ cc, ss]
 * [-ss, cc]
 * where cc and ss are diagonal matrices satisfying 
 * cc^2 + ss^2=1
 */


public class MultiplexorOneLineSEO_writer  extends SEO_writer{
    DoubleArray  	_radAngles; //all internal angles in radians


	/**
	 * 
	 */	
	public MultiplexorOneLineSEO_writer(
	        String 			filePrefix, 
	        int 			numBits,  
	  	    DoubleArray  	radAngles)//all internal angles in radians

	{ 	
	 	
    	super(filePrefix, numBits);
    	
    	Throw.If(_numBits<2 , 
			"multiplexor must have at least 2 qubits");

		Throw.If(radAngles.length() != 1<<(_numBits-1), 
			"wrong  number of angles");		
		_radAngles = radAngles;

    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MultiplexorOneLineSEO_writer(
		PrintWriter  	englishOut,
		PrintWriter  	pictureOut,
		Boolean 		doWrite,
		CktEmbedder 	emb,  
        DoubleArray 	radAngles)
    {
    	super(englishOut, pictureOut, emb);
    	//numBitsBef = _numBits
 
    	Throw.If(_numBits<2 , 
		"multiplexor must have at least 2 qubits");
	
		Throw.If(radAngles.length() != 1<<(_numBits-1), 
			"wrong  number of angles");		
		_radAngles = radAngles;

    	if(doWrite){write();}
    }
      	
		
	/**
	 * 
	 */
	public void write(boolean wantHermitian){

		int targetBitPosition = _numBits -1;
		
		MultiControls  controls = new  MultiControls(_numBits, _numBits-1);
		for(int c=0; c<_numBits-1; c++){
			controls.setControl(c, _numBits-c-2, _numBits-c-2);
		}

		writeControlledMultiplexorGate(
				targetBitPosition,
				controls,
				wantHermitian,//withMinus
				_radAngles);
	}
	/**
	 * 
	 */
	public void write(){
		write(false);
	}
	/**
	 * 
	 */
	public void writeHermitian(){
		write(true);
    }
	
	/**
	 * 
	 */ 
   public static void main(String[] args){
	   
	   int numBits = 4;
	   int numMultiControls = numBits -1;
 	   int numAngles = 1<<numMultiControls;
	   double[]  radAngles = new double[numAngles];
	   for(int j=0; j<numAngles; j++){
		   radAngles[j] = (Math.random() -.5)*23;
	   }
       
       new MultiplexorOneLineSEO_writer(
           "oneLine",// filePrefix, 
           numBits,
           new DoubleArray(radAngles));
                
   }
}
