/**
 * This class creates a new instance of 
 * the multiplexor matrix of the form
 * [ cc, ss]
 * [-ss, cc]
 * where cc and ss are diagonal matrices satisfying 
 * cc^2 + ss^2=1
 */

public class MultiplexorUinMat {
    private int _numBits;
    private double[][] _matrix;
	/**
	 * 
	 */
    public MultiplexorUinMat(int numBits, DoubleArray angles) {
    	
        Throw.If(numBits<1, "number of bits must be >= 1");                  
        _numBits = numBits;
        
		int numRows = 1<<numBits;
		int numAngles = 1<<(numBits-1);
		Throw.If(angles.length() != numAngles, 
			"wrong  number of angles");

        
        //_matrix automatically initialized to zero
        _matrix = new double[numRows][numRows];
        
        double cc, ss;
        int k;
        for(int j=0; j<numAngles; j++){
        	cc = Math.cos(angles.get(j));
        	ss = Math.sin(angles.get(j));
        	k = j + numAngles;
        	_matrix[j][j] = cc;
        	_matrix[k][k] = cc;
        	_matrix[j][k] = ss;
        	_matrix[k][j] = -ss;
        }
        
    }
	/**
	  * 
	  */
	public String toString(){
       int numRows = _matrix.length;
       String output = "";
       output += "numRows = "+ numRows + '\n';
       int row, col;
       for(row=0; row<numRows; row++){
    	   for(col=0; col<numRows; col++){
    		   output += _matrix[row][col] + " ";
    	   }
    	   output += '\n';
       }
       return output;
   }
	/**
	 *
	 */
	public double getEntry(int row, int col) {
		return _matrix[row][col];
	}
	/**
	* 
	*/    
	public static void main(String[] args){
		int numBits = 3;
		int numAngles = 1<<(numBits-1);
		double[]  angles = new  double[numAngles];
		for(int j=0; j<numAngles; j++){
			angles[j] = (Math.random() -.5)*23;
		}

		System.out.println("numBits = " + numBits);
		MultiplexorUinMat mat = new MultiplexorUinMat(numBits, 
				new DoubleArray(angles));
        System.out.println(mat.toString());
      
   }

}
