import java.io.PrintWriter;

/**
 * CB_NetQEmb = classical Bayesian net q-Embedding
 * This class writes English and Picture Files
 * for a quantum-embedding of a Bayesian net
 * expressed as a product of multiplexors. 
 * The multiplexors are written in one line style. 
 * The class uses angles calculated by class CB_NetQEmb
 * from the Bayesian net.
 */
public class CB_NetQEmbSEO_writer  extends QEmbSEO_writer{
	CB_NetQEmb   _probQEmb;
	/**
	 * 
	 */	
	public CB_NetQEmbSEO_writer(
	        String 	filePrefix, 
	        int numBits, 
	        CB_NetQEmb  probQEmb)
	{ 	
	 	
    	super(filePrefix, numBits); 
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of bits must be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
				"angles have wrong number of column bits");
		_probQEmb = probQEmb;

    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public CB_NetQEmbSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb, 
		CB_NetQEmb  probQEmb)
    {
    	super(englishOut, pictureOut, emb);
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of bits must be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
				"angles have wrong number of column bits");
		_probQEmb = probQEmb;


    	if(doWrite){write();}
    }     	
	/**
	 * 
	 */	
	public void write(){
		
		if(!_probQEmb.usingEffiMeth()){
			writeA(_probQEmb.getAngles1());
			writeB(_probQEmb.getAngles2());
		}else{
			writeA(_probQEmb.getAngles1());
			altWriteB(_probQEmb.getAngles2());
		}
	}
	/**
	 * 
	 */	
	public void writeHermitian(){
		//This method just reverses order of gates in write()
		//and uses negative of angles in write()
		if(!_probQEmb.usingEffiMeth()){
			writeB_Hermitian(_probQEmb.getAngles2());
			writeA_Hermitian(_probQEmb.getAngles1());
		}else{
			altWriteB_Hermitian(_probQEmb.getAngles2());
			writeA_Hermitian(_probQEmb.getAngles1());
		}	
	}	
	/**
	 * 
	 */	
	public void altWriteB(DoubleArray[] angles){
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below

		//second half of the multiplexors
		for(int j=halfNumBits; j<_numBits; j++){
			int numBitsBef = _numBits + halfNumBits- j;//for emb below
			int[]  bitMap =  new int[numBitsBef];
			int k=0;
			while(k<numBitsBef){
				bitMap[k] = _numBits - 1 - k;
				k++;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = _numBits -j -1;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					2);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		false,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray).writeHermitian();			
		}		
	}
	/**
	 * 
	 */	
	public void altWriteB_Hermitian(DoubleArray[] angles)
	{
	
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below

		//must invert this:
		////second half of the multiplexors
		//for(int j=halfNumBits; j<_numBits; j++){

		//first half of the multiplexors
		for(int j=_numBits-1; j>=halfNumBits; j--){
			int numBitsBef = _numBits + halfNumBits- j;
			int[]  bitMap =  new int[numBitsBef];
			int k=0;
			while(k<numBitsBef){
				bitMap[k] = _numBits - 1 - k;
				k++;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = _numBits -j -1;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					2);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		true,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray);			
		}
	}
	/**
	 * 
	 */	    
	public RedPair pruneControls(
			CktEmbedder  	emb,
			DoubleArray		angArray,
			int				numRowBits,
			int				mType)
	{
		if(!_probQEmb.usingEffiMeth()){
			return new RedPair(emb, angArray);
		}
		
		boolean[]	isUsedAngleBit = 
			_probQEmb.getUsedAngleBits(numRowBits, mType);

		int numAngleBits = isUsedAngleBit.length;
		int j, k;
		int numBitsRed=0;		//Red = Reduced
		for(j=0; j<numAngleBits; j++){
			if(isUsedAngleBit[j])numBitsRed++;		
		}
		
		//build angArrayRed
		int[] bitPos = new int[numBitsRed];
		k=0;
		for(j=0; j<numAngleBits; j++){
			if(isUsedAngleBit[j]){
				bitPos[k] = j;
				k++;
			}
		}
		int numStatesRed = 1<<numBitsRed;
		DoubleArray angArrayRed = new DoubleArray(numStatesRed);
		int sum, st;
		for(st=0; st<numStatesRed; st++){
			sum=0;
			for(j=0; j<numBitsRed; j++){
				if(((st>>j)&1)==1){
					sum += 1<<bitPos[j];
				}
			}
			angArrayRed.set(st, angArray.get(sum));
		}
		
		//build embRed
		int numBitsBef = numBitsRed + 1;//for embRed
		int[]  bitMap =  new int[numBitsBef];
		
		k=0;
		for(j=0; j<numAngleBits; j++){
			if(isUsedAngleBit[j]){
				bitMap[k]= emb.aft(j);
				k++;
			}
		}
		bitMap[numBitsRed] = emb.aft(numAngleBits);
		CktEmbedder embRed  =  new  CktEmbedder(
				numBitsBef,
				_numBits,//numBitsAft
				bitMap,
				Globals.F_vec(_numBits),//isP0Control
				Globals.F_vec(_numBits));//isP1Control
		
		return new RedPair(embRed, angArrayRed);
	}
	/**
	 * 
	 */	    
	public static void main(String[] args)
	{
		String filePrefix = "qEmb";
		
		//String 	relPath = "2nodes/";
		String 	relPath = "3nodes/";
		//String 	relPath = "4nodeFullyConnected/";
		//String 	relPath = "Asia/";
		
		String 	statesFileName = "states.txt";
		String 	parentsFileName = "parents.txt";
		
		CB_Net  net = new CB_Net(relPath + statesFileName, 
						relPath + parentsFileName,
						relPath + "probs.txt");	
		int numNits = net.getNumNits();
		int numBits = 2*numNits;
		new CB_NetQEmbSEO_writer(relPath + filePrefix, 
				numBits, new CB_NetQEmb(net, true/*usingEffiMeth*/));
		   
		String 	filePrefixIn = relPath + filePrefix;
		String 	filePrefixOut = relPath + filePrefix + "_exact";
		int numGroundedBits = 3;//ignored
		//int compiMode = MultiplexorExpander.ORACULAR_APPROX;
		int compiMode = MultiplexorExpander.EXACT_SEO;
		int bitPrecision = 3;//ignored
	   
		//SEO_reader can't read multiplexors so 
		//first use MultiplexorExpander
		//to get exact SEO expansion,
		//and then apply SE_reader to that
		new MultiplexorExpanderMain(
			filePrefixIn,
			filePrefixOut,
			compiMode,
			numGroundedBits,
			bitPrecision);
	  	
	   	SEO_readerMu	reader = new SEO_readerMu(
	   	    relPath + filePrefix + "_exact",
	   	    numBits, 
	   	    false); // useSparse
	   
	   	//verify that 
		//<0|          |x>
		//    U_2^H U_1      = M_hyb(y|x)
		//<y|          |0>
		
		int row, col;
		int urow, ucol;
		int numNitStates = 1<<numNits;
		boolean[] rowArray, colArray;
		for(row=0; row<numNitStates; row++){
			rowArray = Globals.boolArrayFromInt(row, numNitStates);
			for(col=0; col<numNitStates; col++){
				colArray = Globals.boolArrayFromInt(col, numNitStates);
				System.out.print(Math.sqrt(
					net.getM2_entry(colArray, rowArray)*
					net.getM1_entry(rowArray, colArray)
					)+ "\t");
			}
			System.out.print("\n");
		}
		System.out.print("\n");
		
		for(row=0; row<numNitStates; row++){
			urow = row<<numNits;
			for(col=0; col<numNitStates; col++){
				ucol = col;
				System.out.print(reader.getEntry(urow, ucol) + "\t");
			}
			System.out.print("\n");
		}
	}
}
