/**
 * This class stores information associated with one 
 * node of a Bayesian net.
 */
public class Node {
	private String _name;
	private NamesList _stateNamesList;
	private int[] _parents;//parent nodes (list of their fill numbers)
	private int[] _parentDegens;//Degens=degeneracies=(number of states) 
								//for parent nodes
	private int[] _markovBlanket;

	private NodeProbMatrix _probMat; //node probability matrix

	private int _firstOwnedNit;
	private int _numOwnedNits;
	
	/**
	 * 
	 */	
	public Node(
		String 	name,
		String[] stateNames)
	{
		_name = name;
		_stateNamesList = new NamesList(stateNames);
	}
	/**
	 * 
	 */	
	public void setParents(int[] parents){
		_parents = parents;
	}
	/**
	 * 
	 */
	public void setParentDegens(int[] parentDegens){
		_parentDegens = parentDegens;
	}
	/**
	 * 
	 */
	public void setMBlanket(int[] markovBlanket){
		_markovBlanket = markovBlanket;
	}
	/**
	 * 
	 */		
	public void setNumOwnedNits(int numOwnedNits){
		_numOwnedNits = numOwnedNits;
	}
	/**
	 * 
	 */	
	public void setFirstOwnedNit(int firstOwnedNit){
		_firstOwnedNit = firstOwnedNit;
	}
	/**
	 * 
	 */	
	public void setProbMat(Object[] probMat)
	{		
		_probMat = new NodeProbMatrix(getProbMatNumCols(), probMat);		
	}
	/**
	 * 
	 */
	public void setProbMat(int numRows, int numCols)
	{		
		_probMat = new NodeProbMatrix(numRows, numCols);		
	}
	/**
	 * 
	 */
	public String getName(){
		return _name;
	}
	/**
	 * 
	 */	
	public int getNumStates(){
		return _stateNamesList.getNumNames();
	}

	public NamesList getStateNamesList(){
		return _stateNamesList;
	}
	/**
	 * 
	 */
	public int getNumParents(){
		return _parents.length;
	}
	/**
	 * 
	 */
	public int getParent(int j){
		return _parents[j];
	}
	/**
	 * 
	 */	
	public int getParentDegen(int j){
		return _parentDegens[j];
	}
	/**
	 * 
	 */
	public int getMBlanketSize(){
		return _markovBlanket.length;
	}
	/**
	 * 
	 */
	public int getMBlanketNode(int j){
		return _markovBlanket[j];
	}
	/**
	 * 
	 */
	public int getNumOwnedNits(){
		return _numOwnedNits;
	}
	/**
	 * 
	 */	
	public int getFirstOwnedNit(){
		return _firstOwnedNit;
	}
	/**
	 * 
	 */	
	public double getProb(int row, int col){
		return _probMat.getProb(row, col);
	}
	/**
	 * 
	 */
	public int getProbMatColNum(int[] colArray){
		int numParents = _parents.length;
		int sum = 0;
		int coef = 1;
		for(int j=0; j<numParents; j++){
			sum += coef*colArray[j];
			coef *= _parentDegens[j];
		}
		return sum;
	}
	/**
	 * 
	 */
	public int[] getProbMatColArray(int col){
		int numParents = _parents.length;
		int[] colArray = new int[numParents];
		int balance = col;
		for(int j=0; j<numParents; j++){
			colArray[j] = balance%_parentDegens[j];
			balance  = (balance- colArray[j])/_parentDegens[j];
		}
		return colArray;
	}
	/**
	 * 
	 */
	public int getProbMatNumCols(){
		int numParents = _parents.length;
		int ans = 1;
		for(int j=0; j<numParents;j++){
			ans *= _parentDegens[j];
		}
		return ans;
	}
	/**
	 * 
	 */
	public int[] getProbMatIntsOfRow(int row){
		return _probMat.getIntsOfRow(row);
	}
	/**
	 * 
	 */
	public double[] getProbMatDoublesOfRow(int row){
		return _probMat.getDoublesOfRow(row);
	}	
}
