import java.util.ArrayList;
import java.util.Comparator;

/**
* This class stores the probability matrix
* associated with one node of a Bayesian net.
*/
public class NodeProbMatrix{
	private int _numCols;
	private IntDoublePair[][] _probMat;
	
	/**
	 * 
	 */
	private class ColComparator implements Comparator<IntDoublePair>{
		public int compare(IntDoublePair p1, IntDoublePair p2){
			return (new Integer(p1._i)).compareTo(p2._i);
		}
	}
	/**
	 * 
	 */	
	public NodeProbMatrix(int numRows, int numCols){
		_numCols = numCols;
		_probMat = new IntDoublePair[numRows][numCols];
		int row, col;
		double prob = 1.0/numRows;
		
		for(row=0; row<numRows; row++){
			for(col=0; col<numCols; col++){
				_probMat[row][col]= new IntDoublePair(col, prob);
			}
		}		
	}
	/**
	 * 
	 */	
	public NodeProbMatrix(
		int numCols,
		Object[] probMat)
	{
		_numCols = numCols;
		int numRows = probMat.length;
		_probMat = new IntDoublePair[numRows][];
		int row, rcol, numRaggedCols;
		for(row=0; row<numRows; row++){
			numRaggedCols = ((ArrayList<IntDoublePair>)probMat[row]).size();
			_probMat[row] = new IntDoublePair[numRaggedCols];
			for(rcol=0; rcol<numRaggedCols; rcol++){
				_probMat[row][rcol] = 
					((ArrayList<IntDoublePair>)probMat[row]).get(rcol);
			}
		}
		sortEachRow();
		checkNoRepeatedCols();
		checkColsAddToOne();
	}
	/**
	 * 
	 */	
	private void sortEachRow(){
		int numRows = _probMat.length;
		for(int row=0; row<numRows; row++){
			java.util.Arrays.sort(_probMat[row], new ColComparator());
		}		
	}
	/**
	 * 
	 */	
	public double getProb(int row, int col){
		Throw.If(col>_numCols,
			"out of bounds col in NodeProbMatrix:getProb(row, col)");
		IntDoublePair target = new IntDoublePair(col, 0.0);
		ColComparator ccomp = new ColComparator();
		int loc = 
			java.util.Arrays.binarySearch(_probMat[row], target, ccomp);
		return loc<0?0.0:_probMat[row][loc]._d;
	}
	/**
	 * 
	 */	
	public int[] getIntsOfRow(int row){
		int lenRaggedRow = _probMat[row].length;
		int[] cols = new int[lenRaggedRow];
		for(int rcol=0; rcol<lenRaggedRow; rcol++){
			cols[rcol] = _probMat[row][rcol]._i;
		}
		return cols;
	}
	/**
	 * 
	 */	
	public double[] getDoublesOfRow(int row){
		int lenRaggedRow = _probMat[row].length;
		double[] probs = new double[lenRaggedRow];
		for(int rcol=0; rcol<lenRaggedRow; rcol++){
			probs[rcol] = _probMat[row][rcol]._d;
		}
		return probs;		
	}
	/**
	 * 
	 */	
	private void checkNoRepeatedCols(){
		int numRows = _probMat.length;
		int row, rcol, numRaggedCols;
		for(row=0; row<numRows; row++){
			numRaggedCols = _probMat[row].length;
			for(rcol=0; rcol<numRaggedCols-1; rcol++){
				if(_probMat[row][rcol]._i!=_probMat[row][rcol+1]._i){
					//do nothing
				}else{
					Throw.If(true, 
					 "The following column number is repeated " +
					 "in NodeProbMatrix: " + _probMat[row][rcol]._i);
				}
			}
		}	
	}
	/**
	 * 
	 */	
	private void checkColsAddToOne(){
		int numRows = _probMat.length;
		double sum;
		for(int col=0; col<_numCols; col++){
			sum = 0.0;
			for(int row=0; row<numRows; row++){
				sum += getProb(row, col);
			}
			Throw.If(Math.abs(sum-1)>10E-6,
				"Probabilities of a column of a " +
				"NodeProbMatrix don't sum to one");
		}
	}
}