%A variation of afga.m where
%(1)the first g_in used and 
%(2)the g0 used as argument in afga_step()
%are both estimated because they are not known initially.
%This program is described in the paper
%``Quantum Circuit for calculating symmetrized functions
%via Grover-like algorithm" by R.R. Tucci, arXiv?

g0_degs = 88 %g0 stands for gamma
g0est_degs = 90 %estimate of g0, assuming don't know it initially
del_lam_degs = 150
num_steps = 50 %maximum number of Grover iterations
tail_len = 20 %apply low pass filter to last tail_len instances of num_steps
num_trials = 3
plotted_trial = 0 %won't plot if this negative

assert(plotted_trial<num_trials, "plotted_trial must be less than num_trials");
    
fi = fopen ("afga_blind.txt", "w", "native");
fprintf(fi, "gamma(degs) = %10.4e\n", g0_degs);
fprintf(fi, "gamma_estimate(degs) = %10.4e\n", g0est_degs);
fprintf(fi, "del_lam(degs) = %10.4e\n", del_lam_degs);
fprintf(fi, "num_steps = %i\n", num_steps);
fprintf(fi, "tail_len = %i\n", tail_len);
fprintf(fi, "num_trials = %i\n", num_trials);
fprintf(fi, "plotted_trial = %i\n", plotted_trial);

g0 = (pi/180)*g0_degs
g0est = (pi/180)*g0est_degs
del_lam = (pi/180)*del_lam_degs;
sg0 = sin(g0);
cg0 = cos(g0);
slam = sin(del_lam);
clam = cos(del_lam);
vz = [0;0;1];
vs0 = [sg0;0;cg0];
fprintf(fi,"j\tgam_j(degs)\talp_j(degs)\tvr_x\tvr_y\tvr_z\tvs_x\tvs_y\tvs_z\n");

sx = linspace(0,num_steps-1,num_steps);
sy = linspace(0,num_steps-1,num_steps);
sz = linspace(0,num_steps-1,num_steps);

if(plotted_trial>=0)
    graphics_toolkit("fltk");%selects openGl, "gnuplot" is other option
	clf(); %clear figure queue or will draw on top of previous one
	sx_plot = linspace(0,num_steps-1,num_steps);
	sy_plot = linspace(0,num_steps-1,num_steps);
	sz_plot = linspace(0,num_steps-1,num_steps);
	time = linspace(0,num_steps-1,num_steps);
endif	

range = (num_steps-tail_len):num_steps;
for k=0:num_trials-1
	vs = vs0;
	vr = afga_rot(vz, -del_lam, vs);
	g_in = g0est;
	[g_out, alpha] = afga_step(g_in, g0est, del_lam);
	fprintf(fi,"trial %i, g0est_over_g0 = %10.4e\n", k, g0est/g0);
    for j=0:num_steps-1
	    fprintf(fi,"%i\t", j);
	    fprintf(fi,"%10.4e\t", g_in*180/pi);
	    fprintf(fi,"%10.4e\t", alpha*180/pi);
	    fprintf(fi,"%10.4e\t%10.4e\t%10.4e\t", vr(1), vr(2), vr(3));
	    fprintf(fi,"%10.4e\t%10.4e\t%10.4e\n", vs(1), vs(2), vs(3));
	    
		sx(j+1) = vs(1);
		sy(j+1) = vs(2);
		sz(j+1) = vs(3);
		
		if(k==plotted_trial)
		    sx_plot(j+1) = vs(1);
			sy_plot(j+1) = vs(2);
			sz_plot(j+1) = vs(3);
	    endif
	    
	    vs = afga_rot(vs0, -alpha, vr);
	    vr = afga_rot(vz, -del_lam, vs); 	
	    g_in = g_out;
	    [g_out, alpha] = afga_step(g_in, g0est, del_lam);
    endfor
    est_method=0;
    if(est_method==0)	
        vz_est = [(max(sx(range))+min(sx(range)))/2,(max(sy(range))+min(sy(range)))/2,(max(sz(range))+min(sz(range)))/2]';
    else
        % this estimation method doesn't work well for g0_degs= 180 - epsilon
        %vz_est = [mean(sx(range)),mean(sy(range)),mean(sz(range))]';
    endif
    vz_est = vz_est/sqrt(vz_est'*vz_est);
	g0est = acos(vz_est'*vs0)
endfor
fclose(fi);

if(plotted_trial>=0) 
    scatter(time, sx_plot, 5, "red", "filled");
    hold on;
    scatter(time, sy_plot, 5, "blue", "filled");
    hold on;
    scatter(time, sz_plot, 5, "green", "filled");
  
legend(
strcat(
sprintf("gamma(degs) = %10.4e\n", g0_degs),      
sprintf("gamma_estimate(degs) = %10.4e\n", g0est_degs),
sprintf("del_lam(degs) = %10.4e\n", del_lam_degs),
sprintf("num_steps = %i\n", num_steps),
sprintf("tail_len = %i\n", tail_len),
sprintf("num_trials = %i\n", num_trials),
sprintf("plotted_trial = %i", plotted_trial)
), 
"location", "southeast");

    xlabel('Grover Step');
    
%save plot as svg (scalable vector graphic) so 
%can touch up in Inkscape and then convert within Inkscape
%to eps (encapsulated postscript) for Latex. 
%Could also just save now directly as .eps file
    print -dsvg afga_blind.svg 
endif





