import java.io.*;
/**
 * This class writes English and Picture Files for the
 * Phase Estimation algorithm.
 */
public abstract class PhaseEstSEO_writer extends SEO_writer{	
	protected int _numAtomBits;//Atom is general unitary transformation
	protected int _numProbeBits;//from Atom^1 to Atom^pow(2,numProbeBits-1)
	protected PhaseEstEnding _ending;
	public enum PhaseEstEnding {NONE, QFT, HADS}
	//QFT= Quantum Fourier Transform
	//HADS= tensor product of single-bit Hadamards
	/**
	 *
	 */
	public PhaseEstSEO_writer(
	        String 	filePrefix,
	        int numBits)
	{
		super(filePrefix, numBits);
	}
	/**
	 *
	 */
	public PhaseEstSEO_writer(
	        String 	filePrefix,
	        int numBits,
			int numProbeBits,
			PhaseEstEnding ending)
	{
		super(filePrefix, numBits);
		_numAtomBits = _numBits - numProbeBits;
		Throw.If(_numAtomBits<1, "Atom must have at least one bit");
		_numProbeBits = numProbeBits;
		_ending = ending;
		
        write();
        
        closeFiles();        
    }
	/**
	 * 
	 */
	public PhaseEstSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		CktEmbedder emb)
    {
    	super(englishOut, pictureOut, emb);
    }
	/**
	 * 
	 */
    public PhaseEstSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		boolean doWrite,
		CktEmbedder emb,
		int numProbeBits,
		PhaseEstEnding ending)
    {
    	super(englishOut, pictureOut, emb);

		_numAtomBits = _numBits - numProbeBits;
		Throw.If(_numAtomBits<1, "Atom must have at least one bit");
		_numProbeBits = numProbeBits;
		_ending = ending;
 	
    	
    	if(doWrite){write();}
    }
    
	/**
	 * 
	 */
	protected void write(){		
		int j, numBitsBef, numBitsAft;
		
		for(j=0; j< _numProbeBits; j++){
			writeControlledOneBitGate(
				j,//  targetBitPosition
				new TFControls(_numBits,0),
				new OneBitHadamard());			
		}

		//parameters for preEmb;
		numBitsBef = _numAtomBits;
		numBitsAft = _numBits;
		int[] bitMap = new  int[numBitsBef];
		for(j=0; j<numBitsBef; j++){
			bitMap[j] = j + _numProbeBits;
		}
		boolean[] isP0Control = Globals.F_vec(numBitsAft);
		boolean[] isP1Control = Globals.F_vec(numBitsAft);
		CktEmbedder preEmb;
		CktEmbedder  savedEmb = new CktEmbedder(_emb);
		for(j=0; j< _numProbeBits; j++){
			isP1Control[_numProbeBits-1-j] = true;
			preEmb = new  CktEmbedder(
						numBitsBef,
						numBitsAft,
						bitMap,
						isP0Control,
						isP1Control );
			_emb = preEmb.getCompositeMapOfYouFollowedBy(_emb);
			writeAtomPow(j);
			//restore isP1Control to all false
			isP1Control[_numProbeBits-1-j] = false;
			_emb  = savedEmb;//restoring _emb
		}
		

		if(_ending==PhaseEstEnding.QFT){
			numBitsBef = _numProbeBits;
			numBitsAft = _numBits;
			FouSEO_writer  w = new FouSEO_writer(
				_englishOut,
				_pictureOut,
				false,// doWrite
				new CktEmbedder(
					numBitsBef,
					numBitsAft,
					Globals.identityPermutation(numBitsBef),//bitMap
					Globals.F_vec(numBitsAft),//isP0Control
					Globals.F_vec(numBitsAft)).getCompositeMapOfYouFollowedBy(_emb));//isP1Control
			w.writeHermitian();			
		}else if(_ending==PhaseEstEnding.HADS){
			for(j=0; j< _numProbeBits; j++){
				writeControlledOneBitGate(
					j,//  targetBitPosition
					new TFControls(_numBits,0),
					new OneBitHadamard());			
			}

		}
	}
	/**
	 * 
	 */
	protected void writeHermitian(){
		int j, numBitsBef, numBitsAft;
		
		if(_ending==PhaseEstEnding.QFT){
			numBitsBef = _numProbeBits;
			numBitsAft = _numBits;
			FouSEO_writer  w = new FouSEO_writer(
				_englishOut,
				_pictureOut,
				false,// doWrite
				new CktEmbedder(
					numBitsBef,
					numBitsAft,
					Globals.identityPermutation(numBitsBef),//bitMap
					Globals.F_vec(numBitsAft),//isP0Control
					Globals.F_vec(numBitsAft)).getCompositeMapOfYouFollowedBy(_emb));//isP1Control
			w.write();			
		}else if(_ending==PhaseEstEnding.HADS){
			//for(int j=0; j< _numProbeBits; j++){
			for(j=_numProbeBits-1; j>=0; j--){
				writeControlledOneBitGate(
					j,//  targetBitPosition
					new TFControls(_numBits,0),
					new OneBitHadamard());			
			}

		}
		
		//parameters for preEmb;
		numBitsBef = _numAtomBits;
		numBitsAft = _numBits;
		int[] bitMap = new  int[numBitsBef];
		for(j=0; j<numBitsBef; j++){
			bitMap[j] = j + _numProbeBits;
		}
		boolean[] isP0Control = Globals.F_vec(numBitsAft);
		boolean[] isP1Control = Globals.F_vec(numBitsAft);
		CktEmbedder preEmb;
		CktEmbedder  savedEmb = new CktEmbedder(_emb);
		//for(j=0; j< _numProbeBits; j++){
		for(j=_numProbeBits -1; j>=0; j--){
			isP1Control[_numProbeBits-1-j] = true;
			preEmb = new  CktEmbedder(
						numBitsBef,
						numBitsAft,
						bitMap,
						isP0Control,
						isP1Control );
			_emb = preEmb.getCompositeMapOfYouFollowedBy(_emb);
			writeAtomPowHermitian(j);
			isP1Control[_numProbeBits-1-j] = false;//restore isP1Control to all false
			_emb  = savedEmb;//restoring _emb
		}
		
		//for(j=0; j< _numProbeBits; j++){
		for(j=_numProbeBits -1; j>=0; j--){
			writeControlledOneBitGate(
				j,//  targetBitPosition
				new TFControls(_numBits,0),
				new OneBitHadamard());			
		}
		
	}
	/**
	 * 
	 */	 
    public void  writeAtomPow(int powerOf2){
		int loopNum = getLineCount();
		writeLOOP(loopNum, 1<<powerOf2);
		
		writeAtom();

		writeNEXT(loopNum);
	}
    /**
	 * 
	 */	 
    public void  writeAtomPowHermitian(int powerOf2){
		int loopNum = getLineCount();
		writeLOOP(loopNum, 1<<powerOf2);
		
		writeAtomHermitian();

		writeNEXT(loopNum);
	}
	/**
	 * 
	 */
    protected abstract void  writeAtom();
	/**
	 * 
	 */
    protected abstract void  writeAtomHermitian();

       
}

