import java.io.PrintWriter;

/**
 * This class writes English and Picture files
 * for MULTIPLE phase estimation steps (bunch of steps = stairs).
 */
public abstract class PhaseEstStairsSEO_writer extends SEO_writer{
	protected int 	_numAtomBits;//Atom is general unitary transformation
	protected int 	_numProbeBits;//from Atom^1 to Atom^pow(2,numProbeBits-1)
	protected QEmb   _probQEmb;
	protected int  	_numPESteps;//Phase Estimation steps in stairs
	
	/**
	 *
	 */
	public PhaseEstStairsSEO_writer(
	    String 	filePrefix,
	    int numBits,
		int numProbeBits,
		QEmb  probQEmb,
		int numPESteps)
	{
		super(filePrefix, numBits);
		_numAtomBits = _numBits - numProbeBits*numPESteps;
		_numProbeBits = numProbeBits;
		_probQEmb = probQEmb;
		_numPESteps = numPESteps;
		Throw.If(_numAtomBits <2, "Must have at leat 2 atom bits");
		
	    write();
	    
	    closeFiles();       
	}
	/**
	 * 
	 */
    public PhaseEstStairsSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		boolean doWrite,
		CktEmbedder emb,
		int numProbeBits,
		QEmb  probQEmb,
		int numPESteps)
    {
    	super(englishOut, pictureOut, emb);

		_numAtomBits = _numBits - numProbeBits*numPESteps;
		_numProbeBits = numProbeBits;
		_probQEmb = probQEmb;
		_numPESteps = numPESteps;
		Throw.If(_numAtomBits <2, "Must have at leat 2 atom bits");
	
    	
    	if(doWrite){write();}
    }
	/**
	  * 
	  */
	protected void write(){
		int numBitsBef = _numAtomBits + _numProbeBits;//for emb below
		int numBitsAft = _numBits;
		int[]  bitMap =  new int [numBitsBef];
//		boolean[] isP0Control = Globals.F_vec(numBitsAft);
//		boolean[] isP1Control = Globals.F_vec(numBitsAft);
		int j,k, boost;
		boost = numBitsAft - numBitsBef;
		for(k=_numProbeBits; k<numBitsBef; k++){//sink atoms to bottom
			bitMap[k] = k + boost;
		}
		for(j=_numPESteps-1; j>=0; j--){
			for(k=0; k<_numProbeBits; k++){
				boost = j*_numProbeBits;
				bitMap[k] = k + boost;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				bitMap,
				Globals.F_vec(_numBits),//isP0Control
				Globals.F_vec(_numBits));//isP1Control
			writePhaseEst(emb);
		}
	}
	/**
	  * 
	  */
	protected void writeHermitian(){
		int numBitsBef = _numAtomBits + _numProbeBits;//for emb below
		int numBitsAft = _numBits;
		int[]  bitMap =  new int [numBitsBef];
//		boolean[] isP0Control = Globals.F_vec(numBitsAft);
//		boolean[] isP1Control = Globals.F_vec(numBitsAft);
		int j,k, boost;
		boost = numBitsAft - numBitsBef;
		for(k=_numProbeBits; k<numBitsBef; k++){//sink atoms to bottom
			bitMap[k] = k + boost;
		}
		//for(j=_numPESteps-1; j>=0; j--){
		for(j=0; j<_numPESteps; j++){
			for(k=0; k<_numProbeBits; k++){
				boost = j*_numProbeBits;
				bitMap[k] = k + boost;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				bitMap,
				Globals.F_vec(_numBits),//isP0Control
				Globals.F_vec(_numBits));//isP1Control
			writePhaseEstHermitian(emb);
		}

	}
    /**
	 * 
	 */	
    protected abstract void writePhaseEst(CktEmbedder emb);
    /**
	 * 
	 */	
    protected abstract void writePhaseEstHermitian(CktEmbedder emb);

}


