/**
 * ProbMatQEmb = Probability matrix q-Embedding
 * This class outputs the multiplexor angles
 * used to construct a quantum embedding.
 * Its main input is a probability matrix.
 */
public class ProbMatQEmb extends QEmb{
//	protected	int				_numColBits;
	protected DoubleArray[]  	_radAngles;//in radians
	/**
	 * 
	 */
	public ProbMatQEmb(int numColBits, double[][] probMat){
		int numCols  = 1<<numColBits;
		Throw.If(numColBits<1, "must have at least one column bit");
		Throw.If(probMat.length != probMat[0].length,
			"probability matrix is not square");
		Throw.If(probMat.length != numCols, 
			"probability matrix has wrong number of columns");
		_numColBits  =  numColBits;
		
		_radAngles = buildAngleArraysFromProbMat(probMat);
	}
	/**
	 * 
	 */
	public double getAngle(int numRowBits, int  flatIndex){
		Throw.If(numRowBits >=  _numColBits, "too many row bits");
		Throw.If(flatIndex >=  1<<(numRowBits +  _numColBits), 
				"angles index is out of bounds");
		return   _radAngles[numRowBits].get(flatIndex);
	}
	/**
	 * 
	 */
	public DoubleArray[] getAngles(){
		return   _radAngles;
	}
}
