import java.io.*;

/**
 * ProbMatQEmb = Probability matrix q-Embedding
 * This class writes English and Picture Files
 * for a quantum-embedding of a probability matrix
 * expressed as a product of multiplexors. 
 * The multiplexors are written in one line style. 
 * The class uses angles calculated by class ProbMatQEmb
 * from the probability matrix.
 */

public class ProbMatQEmbSEO_writer  extends QEmbSEO_writer{
	ProbMatQEmb   _probQEmb;
	/**
	 * 
	 */	
	public ProbMatQEmbSEO_writer(
	        String 	filePrefix, 
	        int numBits, 
	        ProbMatQEmb  probQEmb)
	{ 	
	 	
    	super(filePrefix, numBits); 
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of bits must be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
				"angles have wrong number of column bits");
		_probQEmb = probQEmb;

    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public ProbMatQEmbSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb, 
		ProbMatQEmb  probQEmb)
    {
    	super(englishOut, pictureOut, emb);
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of bits must be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
				"angles have wrong number of column bits");
		_probQEmb = probQEmb;


    	if(doWrite){write();}
    }
     	
	/**
	 * 
	 */	
	public void write(){
		writeA(_probQEmb.getAngles());
		writeB(_probQEmb.getAngles());
	}	
	/**
	 * 
	 */	
	public void writeHermitian(){
		writeB_Hermitian(_probQEmb.getAngles());
		writeA_Hermitian(_probQEmb.getAngles());
	}	
	/**
	 * 
	 */	    
	public static void main(String[] args){
		
	   String filePrefix = "test";
	   int numStateBits = 2;   
	   
	   int numBits =  2*numStateBits;
	   int numStates = 1<<numStateBits;
	   double[][] 	probMat = 
		   ProbMatrices.randomProbMatrix(numStates, numStates);

/*    
	   double[][] 	probMat = 
	   	   {{ .1, .4, .90, .00},
		    { .4, .1, .01, .10},
	   		{ .1, .1, .00, .90},
	   		{ .4, .4, .09, .00}};
 
	   double[][] 	probMat = 
   	   {{ .25, .25, .25, .25},
	    { .25, .25, .25, .25},
   		{ .25, .25, .25, .25},
   		{ .25, .25, .25, .25}};
 */	  		


	   new ProbMatQEmbSEO_writer(
			   filePrefix, numBits, new ProbMatQEmb(numStateBits, probMat));

     
       
       String 	filePrefixIn = filePrefix;
	   String 	filePrefixOut = filePrefix + "_exact";
	   int numGroundedBits = 3;//ignored
	   //int compiMode = MultiplexorExpander.ORACULAR_APPROX;
	   int compiMode = MultiplexorExpander.EXACT_SEO;
	   int bitPrecision = 3;//ignored
 	   
	   //SEO_reader can't read multiplexors so 
	   //first use MultiplexorExpander
	   //to get exact SEO expansion,
	   //and then apply SE_reader to that
	   new MultiplexorExpanderMain(
			filePrefixIn,
			filePrefixOut,
			compiMode,
			numGroundedBits,
			bitPrecision);
	  	
	   SEO_readerMu	reader = new SEO_readerMu(
	   	    filePrefix + "_exact",
	   	    numBits, 
	   	    false); // useSparse
	   
	   //verify that 
	   //<0|          |x>
	   //    U_2^H U_1      = M_hyb(y|x)
	   //<y|          |0>
	   int row, col;
	   int urow, ucol;
	   for(row=0; row<numStates; row++){
		   for(col=0; col<numStates; col++){
			   System.out.print(Math.sqrt(
				probMat[col][row]*probMat[row][col]
			   ) + "\t");
		   }
		   System.out.print("\n");
	   }
	   System.out.print("\n");
	   	
	   for(row=0; row<numStates; row++){
		   urow = row<<numStateBits;
		   for(col=0; col<numStates; col++){
			   ucol = col;
			   System.out.print(reader.getEntry(urow, ucol) + "\t");
		   }
		   System.out.print("\n");
	   }
   	
	}

}
