/**
 * QEmb = quantum embedding
 * This abstract class is extended by ProbMatQEmb
 * and other similar classes.
 */
public abstract class QEmb {
	protected	int		_numColBits;

	/**
	 * 
	 */
	protected QEmb(){}
	/**
	 * 
	 */
	protected DoubleArray[] buildAngleArraysFromProbMat(double[][] probMat){
		
		int numCols  = 1<<_numColBits;
		double[][][]  reducedProbMats = new double[_numColBits+1][][];
		reducedProbMats[_numColBits]  = probMat;
		int  numRows, j, col, row;
		for(j=_numColBits-1; j >=0; j--){//j is numRowBits
			numRows = 1<<j;
			reducedProbMats[j]  =  new  double[numRows][numCols];
			for(col=0; col<numCols; col++){ 
				for(row=0; row<numRows; row++){
					reducedProbMats[j][row][col] =   
						reducedProbMats[j+1][row][col] + 
						reducedProbMats[j+1][row + numRows][col];
				}//row loop
			}//col loop			
		}//j loop
		int flatIndex;
		double  denom, prob, x, y;
		DoubleArray[] angles =  new  DoubleArray [_numColBits];
		for(j=_numColBits-1; j >=0; j--){//j is numRowBits
			numRows = 1<<j;
			angles[j] = new DoubleArray();
			angles[j].reserveMemory(1<<(j+_numColBits));
			for(col=0; col<numCols; col++){ 
				for(row=0; row<numRows; row++){
					flatIndex = (row<<_numColBits) + col;
					denom = reducedProbMats[j][row][col];
					if(denom  > 1e-9){
						prob = reducedProbMats[j+1][row][col]/denom;
						x =  Math.sqrt(prob);
						y =  Math.sqrt(1-prob);
						angles[j].set(flatIndex,  -Math.atan2(y,x));
					}else{
						angles[j].set(flatIndex, 0.0);
					}
				}//row loop
			}//col loop
		}//j loop
		
		return angles;
	}

	/**
	 * 
	 */
	public int getNumColBits(){
		return 	_numColBits;
	}

}
