import java.io.PrintWriter;

/**
 * QEmb = quantum embedding
 * This abstract class is extended by ProbMatQEmbSEO_writer
 * and other similar classes.
 */

public abstract class QEmbSEO_writer  extends SEO_writer{
	/**
	 *
	 */
	public QEmbSEO_writer(
	   String filePrefix,
	   int numBits)
	{
        super(filePrefix, numBits);
	}
	/**
	 * 
	 */
	public QEmbSEO_writer(
   		PrintWriter  englishOut,
   		PrintWriter  pictureOut,
   		CktEmbedder  emb)
	{
	   super(englishOut, pictureOut, emb);
	}
	/**
	 * 
	 */	
	public abstract void write();
	/**
	 * 
	 */	
	public abstract void writeHermitian();
	/**
	 * 
	 */		
	public RedPair pruneControls(
			CktEmbedder  	emb,
			DoubleArray		angArray,
			int				numRowBits,
			int				mType)
	{
		//override this if you want to 
		//remove unnecessary controls before writing
		return new RedPair(emb, angArray);
	}
	/**
	 * 
	 */		
	public void writeA(DoubleArray[] angles){
	
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below
				
		//first half of the multiplexors
		for(int j=0; j<halfNumBits; j++){
			int numBitsBef = j + halfNumBits + 1;//for emb below
			int[]  bitMap =  Globals.identityPermutation(numBitsBef);
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = j;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					1);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		true,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray);			
		}
	}	
	
	/**
	 * 
	 */		
	public void writeB(DoubleArray[] angles){
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below

		//second half of the multiplexors
		for(int j=halfNumBits; j<_numBits; j++){
			int numBitsBef = _numBits + halfNumBits- j;//for emb below
			int[]  bitMap =  new int[numBitsBef];
			int k=0;
			while(k<halfNumBits){
				bitMap[k] = k + halfNumBits;
				k++;
			}
			while(k<numBitsBef){
				bitMap[k] = k - halfNumBits;
				k++;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = _numBits -j -1;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					2);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		false,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray).writeHermitian();			
		}		
	}
 	
	/**
	 * 
	 */	
	public void writeB_Hermitian(DoubleArray[] angles)
	{
	
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below

		//must invert this:
		////second half of the multiplexors
		//for(int j=halfNumBits; j<_numBits; j++){

		//first half of the multiplexors
		for(int j=_numBits-1; j>=halfNumBits; j--){
			int numBitsBef = _numBits + halfNumBits- j;
			int[]  bitMap =  new int[numBitsBef];
			int k=0;
			while(k<halfNumBits){
				bitMap[k] = k + halfNumBits;
				k++;
			}
			while(k<numBitsBef){
				bitMap[k] = k - halfNumBits;
				k++;
			}
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = _numBits -j -1;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					2);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		true,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray);			
		}
	}
	/**
	 * 
	 */	
	public void writeA_Hermitian(DoubleArray[] angles)
	{
	
		int halfNumBits  = _numBits/2;
		//numBitsAft = _numBits for emb below

		//must invert this:
		////first half of the multiplexors
		//for(int j=0; j<halfNumBits; j++){

		//second half of the multiplexors
		for(int j=halfNumBits-1; j>=0; j--){
			int numBitsBef = j + halfNumBits + 1;//for emb below
			int[]  bitMap =  Globals.identityPermutation(numBitsBef);
			CktEmbedder	emb  =  new  CktEmbedder(
					numBitsBef,
					_numBits,//numBitsAft
					bitMap,
					Globals.F_vec(_numBits),//isP0Control
					Globals.F_vec(_numBits));//isP1Control
			int numRowBits = j;
			DoubleArray	 angArray = angles[numRowBits];
			RedPair rp = pruneControls(
					emb,
					angArray,
					numRowBits,
					1);//mType
			new  MultiplexorOneLineSEO_writer(
		    		_englishOut,
		    		_pictureOut,
		    		false,//doWrite
		    		rp._emb.getCompositeMapOfYouFollowedBy(_emb),  
		    		rp._angArray).writeHermitian();			
		}
					
	}
	/**
	 * 
	 */
	protected class RedPair{//Red = reduced
		public 	CktEmbedder 	_emb;
		public	DoubleArray		_angArray;
		
		public RedPair(CktEmbedder emb, DoubleArray angArray){
			_emb = emb;
			_angArray = angArray;
		}
	}

}
