/**
 * This class writes a Suzuki approximant of order 2k. 
 */
public abstract class SuzukiWriter {

	private static double[] _a;
	private static double[] _aProd;


	/**
	 *
	 */
	public SuzukiWriter(){
	
	    _a = new double[10];     
	    _aProd = new double[10];
	    _a[0] = 1;
	    _aProd[0] = 1;
	    for(int k=1; k<10; k++){
	    	_a[k] = 1.0/(4 - Math.pow(4, 1.0/(2*k+1)));
	    	_aProd[k] = _a[k]*_aProd[k-1];
	    }
		
	}
	/**
	 *
	 */
    public void writeSuzuki2(
    	double t, 
    	boolean withEndCaps)
    {
    	if(withEndCaps){
    		writeExpAtimes(t/2);
    	}
		
    	writeExpBtimes(t);
		
		if(withEndCaps){
			writeExpAtimes(t/2);
    	}
	}
	/**
	 *
	 */
    public void writeSuzuki2k(
    	int k,//suzukiHalfOrder 
    	double t, 
    	boolean withEndCaps)
    {
    	if(k==1){
    		writeSuzuki2(t, withEndCaps);
    	}else{
    		double a = _a[k-1];
    		double aProd = _aProd[k-2];
    		if(withEndCaps){
    			writeExpAtimes(a*aProd*t/2);
        	}
    		writeSuzuki2k(k-1, a*t, false);
    			writeExpAtimes(a*aProd*t);
    		writeSuzuki2k(k-1, a*t, false);
    			writeExpAtimes((1 - 3*a)*aProd*t/2);
    		writeSuzuki2k(k-1, (1 - 4*a)*t, false);
    			writeExpAtimes((1 - 3*a)*aProd*t/2);
	        writeSuzuki2k(k-1, a*t, false);
	        	writeExpAtimes(a*aProd*t);
    		writeSuzuki2k(k-1, a*t, false);
    		if(withEndCaps){
    			writeExpAtimes(a*aProd*t/2);
        	}
    	}
    }
	/**
	 * This method writes exp(At), where
	 * Second Order Suzuki Approximant =
	 * exp(At/2)exp(Bt)exp(At/2)
	 */
	public abstract void writeExpAtimes(double t);
	/**
	 * This method writes exp(Bt), where
	 * Second Order Suzuki Approximant =
	 * exp(At/2)exp(Bt)exp(At/2)
	 */
	public abstract void writeExpBtimes(double t);

}
