import java.io.*;

/**
 * This class writes English and Picture files
 * for the Szegedy quantum walk operator, 
 * expressed as a product of multiplexors. 
 * The multiplexors are written in one line style.
 */
public abstract class SzegedySEO_writer   extends SEO_writer{
	QEmb  _probQEmb;

	/**
	 * 
	 */		
	public SzegedySEO_writer(
	        String 	filePrefix,
	        int numBits,
	        QEmb  probQEmb)
	{ 	
	 	
    	super(filePrefix, numBits);
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of free bits of Szegedy operator must " +
			" be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
			"angles have wrong number of column bits");
		_probQEmb = probQEmb;
		
    	write();

        closeFiles();

	}
	/**
	 * 
	 */	
    public SzegedySEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
		QEmb  probQEmb)
    {
    	super(englishOut, pictureOut, emb);
		Throw.If(_numBits<2 || _numBits%2 != 0, 
			"number of free bits of Szegedy operator must " +
			"be even and >=2");
		Throw.If(probQEmb.getNumColBits()!=_numBits/2, 
			"angles have wrong number of column bits");
		_probQEmb = probQEmb;
    	
    	if(doWrite){write();}
    }
    /**
	 * 
	 */	
    protected abstract void writeQEmb();
    /**
	 * 
	 */	
    protected abstract void writeQEmbHermtian();
    /**
	 * 
	 */	
	public void write(){
		writeReflectionOfTopHalf();		
		writeQEmbHermtian();		
		writeReflectionOfBottomHalf();
		writeQEmb();
	}
	/**
	 * writes Hermitian conjugate
	 */
	public void writeHermitian(){
		writeQEmbHermtian();		
		writeReflectionOfBottomHalf();
		writeQEmb();
		writeReflectionOfTopHalf();	
	}
	/**
	 * 
	 */	
	private void writeReflectionOfBottomHalf(){//  (-1)^( |0><0| (x) I )
		int  numControls = _numBits/2;
		TFControls    controls =  new TFControls(_numBits, numControls);
		int bitPos = _numBits;
		for(int c=0; c<numControls; c++){
			bitPos--;
			controls.setControl(c, bitPos, false);
		}
		writeControlledOneBitGate(
		        numControls-1,//  targetBitPosition
		        controls,
		        new OneBitPhaseFactor(Math.PI));
		
	}
	/**
	 * 
	 */	
	private void writeReflectionOfTopHalf(){//  (-1)^( I (x) |0><0|)
		int  numControls = _numBits/2;
		TFControls    controls =  new TFControls(_numBits, numControls);
		int bitPos = numControls;
		for(int c=0; c<numControls; c++){
			bitPos--;
			controls.setControl(c, bitPos, false);
		}
		writeControlledOneBitGate(
		        numControls,//  targetBitPosition
		        controls,
		        new OneBitPhaseFactor(Math.PI));
		
	}

}
