/**
 * This class allows one to manipulate the bits
 * in a string of bits of predefined length<=MAX_LEN
 */
public class BitVector {
	private int _len;//length
	private int _decRep;//decimal representation
	//in java, int is 32bit signed
	private static int MAX_LEN = 16;
	/**
	 * 
	 */	
	public BitVector(int len, int decRep){
		_len = len;
		_decRep = decRep;
		Throw.If(len>MAX_LEN, "bit vector is too long");
		Throw.If(len<1,"bit vector len must be >=1");
	}
	/**
	 * 
	 */	
	public void copy(BitVector bvec){
		_len = bvec._len;
		_decRep = bvec._decRep;
	}
	/**
	 * 
	 */	
	public void set(int len, int decRep){
		Throw.If(len>MAX_LEN, "bit vector is too long");
		Throw.If(len<1,"bit vector len must be >=1");
		_len = len;
		_decRep = decRep;
	}
	/**
	 * 
	 */	
	public void setLen(int len){
		Throw.If(len>MAX_LEN, "bit vector is too long");
		Throw.If(len<1,"bit vector len must be >=1");
		_len = len;
	}
	/**
	 * 
	 */	
	public void setDecRep(int decRep){
		_decRep = decRep;
	}
	/**
	 * 
	 */	
	public int getLen(){
		return _len;
	}
	/**
	 * 
	 */	
	public int getDecRep(){
		return _decRep;
	}
	/**
	 * 
	 */	
	public boolean bitIsON(int bpos){
		Throw.If(bpos>=_len, "bit position is too large");
		int	mask = (1<<bpos);
		return (_decRep & mask)==mask;
	}
	/**
	 * 
	 */	
	public void setBitON(int bpos)
	{
		Throw.If(bpos>=_len, "bit position is too large");
		_decRep |= (1<<bpos);
	}
	/**
	 * 
	 */	
	public void setBitOFF(int bpos)
	{
		Throw.If(bpos>=_len, "bit position is too large");
		_decRep &= ~(1<<bpos);
	}
	/**
	 * 
	 */	
	public  void setAllBitsON()
	{
		for(int bpos=0; bpos<_len; bpos++){
			_decRep |=(1<<bpos);
		}
	}
	/**
	 * 
	 */	
	public int getNumOfONbits()
	{
		int	count = 0;	
		for(int bpos=0; bpos<_len; bpos++){
			if(bitIsON(bpos))count++;
		}
		return count;
	}
	/**
	 * 
	 */	
	public int findONbitToMyRight(int 	meBit)
	{
		//returns -1 if there is none
		Throw.If(meBit>=_len, "meBit is too large");
		if(meBit==0)return -1;
		
		int 	rightONbit = meBit;
		int		mask = (1<<rightONbit);
		boolean		foundIt = false;  
		do{
			rightONbit--;
			mask >>= 1;
			foundIt = ((_decRep & mask) == mask);
		}while( (rightONbit!=0)  &&  !foundIt );
		return 	foundIt?rightONbit:-1;
	}
	/**
	 * 
	 */	
	public int findONbitToMyLeft(int meBit)
	{
		//returns -1 if there is none
		Throw.If(meBit>=_len, "meBit is too large");
		if(meBit==(_len-1))return -1;
		
		int 	leftONbit = meBit;
		int		mask = (1<<leftONbit);
		boolean		foundIt = false;  
		do{
			leftONbit++;
			mask <<= 1;
			foundIt = ((_decRep & mask) == mask);
		}while( (leftONbit!=(_len-1))  &&  !foundIt );
		return 	foundIt?leftONbit:-1;
	}
	/**
	 * 
	 */	
	public int findLeftmostONbit()
	{
		//returns -1 if there is none
		if(bitIsON(_len-1)){
			return _len-1;
		}else{
			return findONbitToMyRight(_len-1);
		}
	}
	/**
	 * 
	 */	
	public int findRightmostONbit()
	{
		//return -1 if there is none
		if(bitIsON(0)){
			return 0;
		}else{
			return findONbitToMyLeft(0);
		}
	}  
	/**
	 * 
	 */	
	
	//about lazy ordering
	//Refs: look under "gray code"
	//(1)Martin Gardener, "Knotted DoughNuts and Other 
	//Mathematical Entertainments", chapt. 2, "The Binary Gray Code"
	//(2)"Numerical Recipies in C"
	//(3)Many books on Discrete Mathematics for CompSci types
	//(4)On the web, in Eric's Treasure Trove/Math/Gray Codes

	//Normal and lazy sequences both start at 0
	//One has normal (dictionary) order, the other has lazy order.

	//For illustration purposes, suppose N_B = 3
	//The lazy sequence 000, 100, 110, 010, 011, 111, 101, 001
	//is easily obtained from
	//the "standard" lazy sequence 000, 001, 011, 010, 110, 111, 101, 100
	//by "reflecting" each sequence term. 
	//We will use the second sequence because
	//it is more common in the literature.
	
	/**
	 * 
	 */	
	public static int	getLazyFromNormal(
	int		bitLen,
	int		normal)
	{	
		Throw.If(bitLen>MAX_LEN, "bit vector length is too big");
		Throw.If(normal>= (1<<bitLen), "normal being converted to lazy is too big");
		int	 lazy = normal;
		if(bitLen>1){
			for(int m = bitLen-2; m >= 0; m--){
				//Look at bpos = m+1, if it's ON, then flip bpos=m.
				//Remember that ^ is same as a mod2 sum.
				lazy ^= ( ((normal >> m+1) & 1) << m );	
			}
		}
		
		return lazy;
	}
	/**
	 * 
	 */	
	public void lazyAdvance(int newNormal)
	{

		//This method takes bit vector "lazy" (which corresponds to bit vector "normal"),  and 
		//changes it to the next lazy bit vector, "new_lazy" (which corresponds to "new_normal").
		Throw.If(newNormal<1, "new normal must be >=1");
		_decRep ^= ((newNormal) & ~(newNormal-1));
		//example:  000, 001, 011, 010, 110, 111, 101, 100
		//_decRep = 011, normal = 2 = 010 initially
		//new_normal = 3 = 011
		//(newNormal & ~normal) = 011 & 101 = 001 = mask for the bit that changed
		//_decRep = 011 ^ 001 = 010 finally

	}
	/**
	 * 
	 */	    
	public static void main(String[] args){
		int len = 3;
		int maxNormal = (1<<len) -1;
		BitVector curBitVec =  new BitVector(len, 0);
		
		for(int normal=0; normal<=maxNormal; normal++){
			System.out.println(getLazyFromNormal(len, normal) + ", " + 
					curBitVec.getDecRep());
			curBitVec.lazyAdvance(normal+1);
		}
		System.out.println("");
//		int lazy;
		curBitVec.set(len, 0);
		for(int normal=0; normal<=maxNormal; normal++){
			System.out.print(curBitVec.getDecRep() + "=");
			for(int beta=len-1; beta>=0; beta--){
				System.out.print(curBitVec.bitIsON(beta)==true?1:0);
			}
			System.out.println("");
			curBitVec.lazyAdvance(normal+1);
		}
	}
	
	
}