/**
 * This class enumerates n choose r, i.e., it enumerates 
 * combinations (subsets) of r elements from the set 
 * {0,1,..,n-1} of n elements.
 * This code came from post by 
 * Sourabh Bhat (heySourabh@gmail.com)
 * posted at
 * http://stackoverflow.com/questions/127704/algorithm-to-return-all-combinations-of-k-elements-from-n
 *
 */
// author: Sourabh Bhat (heySourabh@gmail.com)

public class Combinations
{
	/**
	 * 
	 */	
    public static int[][] getCombinations(int combLen, int outOf)
    {
        int possibilities = get_nCr(outOf, combLen);
        int[][] combinations = new int[possibilities][combLen];
        int arrayPointer = 0;

        int[] counter = new int[combLen];

        for (int i = 0; i < combLen; i++)
        {
            counter[i] = i;
        }
        breakLoop: while (true)
        {
            // Initializing part
            for (int i = 1; i < combLen; i++)
            {
                if (counter[i] >= outOf - (combLen - 1 - i))
                    counter[i] = counter[i - 1] + 1;
            }

            // Testing part
            for (int i = 0; i < combLen; i++)
            {
                if (counter[i] < outOf)
                {
                    continue;
                } else
                {
                    break breakLoop;
                }
            }

            // Innermost part
            combinations[arrayPointer] = counter.clone();
            arrayPointer++;

            // Incrementing part
            counter[combLen - 1]++;
            for (int i = combLen - 1; i >= 1; i--)
            {
                if (counter[i] >= outOf - (combLen - 1 - i))
                    counter[i - 1]++;
            }
        }

        return combinations;
    }
	/**
	 * 
	 */	
    public static int get_nCr(int n, int r)
    {
        if(r > n)
        {
            throw new ArithmeticException("r is greater then n");
        }
        long numerator = 1;
        long denominator = 1;
        for (int i = n; i >= r + 1; i--)
        {
            numerator *= i;
        }
        for (int i = 2; i <= n - r; i++)
        {
            denominator *= i;
        }

        return (int) (numerator / denominator);
    }
	/**
	 * 
	 */	    
    public static Boolean isCanonicalName(int[] comb, int outOf){
    	int combLen = comb.length;
    	if(comb[0]<0){
    		return false;
    	}
    	for(int j=0; j<combLen-1; j++){
    		if(!(comb[j]<comb[j+1])){
    			return false;
    		}
    	}
    	if(comb[combLen-1]>=outOf){
    		return false;
    	}
    	return true;
    }
	/**
	 * 
	 */	
    public static void main(String[] args)
    {

        int combLen = 3;
        int outOf = 5;
        int[][] combinations = getCombinations(combLen, outOf);
        for (int i = 0; i < combinations.length; i++)
        {
            for (int j = 0; j < combinations[i].length; j++)
            {
                System.out.print(combinations[i][j] + " ");
            }
            System.out.println();
        }
    }
}