/**
 * This abstract class stores a list of controls.
 * It's  extended by TFControls and MultiControls.
 */
public abstract class Controls <T>{
	protected int _numBits;//number of bits in circuit
	
	protected int _numControls;
	
	protected int[]  _bitPositions;//array's length = numControls

	//note: _types is 
	//Boolean[] in TFControls
	//Integer[] in MultiControls
	protected T[]  _types;//array's length = numControls
	
	/**
	 * 
	 */	
	public Controls(){
		_numBits = 0;
		_numControls = 0;
		_bitPositions = null;
		_types = null;
	}
	/**
	 * 
	 */	
	public Controls(int numBits, int numControls){
		_numBits = numBits;
		_numControls = numControls;
		_bitPositions = new int[numControls];
		_types = null;

	}

	/**
	 * 
	 */
	public void copy(Controls<T>  controls, int extraNumBits){
		_numBits = controls._numBits + extraNumBits;
		_numControls = controls._numControls;
		_bitPositions = new int[_numControls];
		//_types = (T[])new Object[_numControls];
		for(int j=0; j<_numControls; j++){
			_bitPositions[j] = controls._bitPositions[j];
			//_types[j] = controls._types[j];
		}
	}	

	/**
	 * 
	 */	
	public Controls(Controls<T> controls, int extraNumBits){
		copy(controls, extraNumBits);
	}

	/**
	 * 
	 */
	public void setBitPos(int c, int bitPos){
		Throw.If(c<0 || c>=_numControls, 
			"control array index is out of bounds");

		Throw.If(bitPos<0 || bitPos>=_numBits, 
			"control bit position is out of bounds");
	
		_bitPositions[c] = bitPos;
		
	}
	/**
	 * 
	 */
	public void setType(int c, T type){
		Throw.If(c<0 || c>=_numControls, 
			"control array index is out of bounds");
		_types[c] = type;
		
	}
	/**
	 * 
	 */
	public void setControl(int c, int bitPos, T type){
		setBitPos(c, bitPos);
		setType(c, type);
	}

	/**
	 * 
	 */
	public int  getNumControls(){
		return _numControls;
	}	
	
	/**
	 * 
	 */
	public int getBitPos(int c){
		return _bitPositions[c];
	}
	/**
	 * 
	 */
	public T  getType(int c){
		return _types[c];
	}
	/**
	 * 
	 */	
	public boolean  isControl(int bitPos){
		for(int j=0; j<_numControls; j++){
			if(bitPos==_bitPositions[j]){
				return true;
			}		
		}
		return false;		
	}
	/**
	 * 
	 */	
	public void checkBitPosAreDecreasing(){
		if(_numControls>0){
	        for(int c=0; c<_numControls-1; c++){
	            Throw.If(_bitPositions[c] <= _bitPositions[c+1],
	              "controls bitPositions must be in " +
	              "strictly decreasing order");
	        }
		}	
	}
	/**
	 *
	 */
    public int getWorkSpaceBitPos(int targetBitPos){
    	//find a bit that is neither the target bit nor one of the controls
    	//Make it as big as possible
    	Throw.If(_numBits - _numControls - 1 <=0, 
    		"must have one more qubit to use as working space");
		int workSpaceBitPos = _numBits -1;
		int	trolBitPos = getBitPos(0);
		int c = 0;
		int j;
		for(j=_numBits-1; j>=0; j--){
			if(j>trolBitPos && j!= targetBitPos){
				workSpaceBitPos = j;
				break;
			}else{  
				if(j==trolBitPos){
					c++;
					if(c<_numControls-1){
						trolBitPos = getBitPos(c);
					}else{
						trolBitPos = -10;
					}
				}
			}   				
		}
		return workSpaceBitPos;
    }

}
