import java.io.*;
import java.util.Scanner;
import java.util.Stack;

/**
 * This abstract class expands one line of Picture and English Files
 */
public abstract class ElemOpExpander{
	protected String  _filePrefixIn  ="";
    protected String  _filePrefixOut  ="";
    protected Scanner  _englishIn = null;
    protected Scanner  _pictureIn = null;
    protected PrintWriter  _englishOut = null;
    protected PrintWriter  _pictureOut = null;
    
    protected boolean _knowNumBitsIn = false;
    protected int	_numBitsIn;
    protected int	_numGitsIn;
    protected GitsMonitor _gm;
     
    
 	protected int _lineNumAdvance = 0;
	protected java.util.Stack<Integer>  _loopNumStack = new Stack<Integer>();
   
    /**
     * 
     */
	public ElemOpExpander(
			String filePrefixIn, 
			String filePrefixOut,
			int numGitsIn)
	{
		_filePrefixIn = filePrefixIn;
		_filePrefixOut = filePrefixOut;
		_numGitsIn = numGitsIn;
		
		Throw.If(filePrefixIn.equals(filePrefixOut),
			"file prefix for in and out files must be different");
				
	}
	/**
	 * 
	 */   
    protected void openFiles() throws FileNotFoundException{
		_englishIn = new Scanner(new File(_filePrefixIn + "_eng.txt"));
		_pictureIn = new Scanner(new File(_filePrefixIn + "_pic.txt"));
		_englishOut = new PrintWriter(
				new File(_filePrefixOut + "_eng.txt"));
		_pictureOut = new PrintWriter(
				new File(_filePrefixOut + "_pic.txt"));
	}
	/**
	 * 
	 */   
    protected void closeFiles(){
    	_englishIn.close();
		_pictureIn.close();
		_englishOut.close();
		_pictureOut.close();
    }
	/**
	 * 
	 */
	protected void readWriteLOOP(){
    	int loopNum = _englishIn.nextInt();
    	_englishIn.next();//skip "REPS:"
    	int reps = _englishIn.nextInt();
    	_pictureIn.nextLine();
    	
    	loopNum += _lineNumAdvance;
    	_loopNumStack.push(loopNum);
    	_englishOut.println("LOOP\t" + loopNum +"\tREPS:\t" + reps);
    	_pictureOut.println("LOOP\t" + loopNum +"\tREPS:\t" + reps);
	}
	/**
	 * 
	 */
    protected void readWriteNEXT(){
    	_englishIn.nextInt();//ignore old loop number
    	_pictureIn.nextLine();
    	
    	int loopNum = _loopNumStack.pop();
    	_englishOut.println("NEXT\t" + loopNum);
    	_pictureOut.println("NEXT\t" + loopNum);
    }
    /**
	 * 
	 */
    protected String readMeasurePictureIn(){
    	String pictLine = _pictureIn.nextLine();
		if(!_knowNumBitsIn){					
			_numBitsIn = pictLine.trim().length()/4 + 1;//eg @---+---Ry
			_knowNumBitsIn = true;
			Throw.If(_numBitsIn<_numGitsIn,
				"number of bits in is smaller than number of gits in");
			Throw.If(getNumGitsOut()<_numGitsIn,
				"number of gits out is smaller than number of gits in");
			Throw.If(getNumBitsOut()<_numBitsIn,
				"number of bits out is smaller than number of bits in");
			_gm = new GitsMonitor(getNumBitsOut(), getNumGitsOut());
		}
		return pictLine;
    	
    }
    /**
	 * 
	 */
    protected void echoEngAndPicLines(String engLine){
		_englishOut.println(engLine);
		
		//this won't work right if
		//getNumGitsOut() depends on numBitsIn and numBitsIn
		//not measured yet
		int extraBits = getNumGitsOut() - _numGitsIn;
		
		for(int j=0; j<extraBits; j++){
			_pictureOut.print("|   ");
		}
		_pictureOut.println(readMeasurePictureIn());
    }
	/**
	 *
	 */
    public int getNumBitsIn(){
    	return _numBitsIn;
    }
	/**
	 *
	 */
    public int getNumGitsIn(){
    	return _numGitsIn;
    }	
    /**
	 *
	 */
    public  int getNumBitsOut(){
    	return _numBitsIn - _numGitsIn + getNumGitsOut();
    }
	/**
	 * getNumGitsOut() should not depend on _numBitsIn
	 */
    public abstract int getNumGitsOut();
    /**
	 *
	 */
    public  int getNumNonGits(){
    	return _numBitsIn - _numGitsIn;
    }

    
 }
