import java.io.*;



/**
 * This class writes English and Picture files
 * for the V matrices, which act as 
 * unitary matrix labellers, in the paper
 * ``Quantum Circuit for Calculating Symmetrized Functions
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */


public class EntanglingMatSEO_writer  extends SEO_writer{
	private Boolean _mType;
	private Double[] _gateAngRads;//length = numBits

	/**
	 * 
	 */	
	public EntanglingMatSEO_writer(
	        String 	filePrefix,
	        int numBits,
	        Boolean mType)
	{ 	 	
    	super(filePrefix, numBits);
		_mType = mType;
		setGateAngRads();
		
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public EntanglingMatSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        Boolean mType)
    {
    	super(englishOut, pictureOut, emb);
    	_mType = mType;
    	setGateAngRads();
    	
    	if(doWrite){write();}
}
	/**
	 * 
	 */
	public void setGateAngRads(){
		int j;
		double c,s;
		_gateAngRads = new Double[_numBits];
		int mOffset = (_mType==true?1:0);//m=1,0 has mOffset=1,0
		for(j=0; j<_numBits; j++){
			s=Math.sqrt(1./(_numBits + mOffset -j));
			c=Math.sqrt(1-Math.pow(s,2));
			_gateAngRads[j]= Math.atan2(s, c);
		}		
	}    
	/**
	 * 
	 */
	public void write(){
		int j, c;
		for(j=0;j<_numBits; j++){
			if(j==0){writeOneBitGate(0,
					new OneBitRot(-_gateAngRads[0],OneBitRot.Y_AXIS)); 
			}else{
				TFControls controls = new TFControls(_numBits, j);
				for(c=0; c<j; c++){
					controls.setControl(c, c /*bitPos*/, false);
				}
				writeControlledOneBitGate(
					j, //targetBitPosition, 
					controls, 
					new OneBitRot(-_gateAngRads[j],OneBitRot.Y_AXIS));
			}
		}		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		int j, c;
		for(j=_numBits-1;j>=0; j--){
			if(j==0){writeOneBitGate(0,
					new OneBitRot(_gateAngRads[0],OneBitRot.Y_AXIS)); 
			}else{
				TFControls controls = new TFControls(_numBits, j);
				for(c=0; c<j; c++){
					controls.setControl(c, c /*bitPos*/, false);
				}
				writeControlledOneBitGate(
					j, //targetBitPosition, 
					controls, 
					new OneBitRot(_gateAngRads[j],OneBitRot.Y_AXIS));
			}
		}		
	}
	/**
	 * 
	 */
    public static void main(String[] args){
    	
 		int numBitsBef = 4;
 		int numBitsAft = 4;
    	CktEmbedder  emb = new CktEmbedder(numBitsBef, numBitsAft,
    		 Globals.identityPermutation(numBitsBef),
    		 Globals.F_vec(numBitsAft),
    		 Globals.F_vec(numBitsAft));
    	
    	String filePrefix = "vmat";
    	Boolean mType=false;
    	
		EntanglingMatSEO_writer  w = new EntanglingMatSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			emb,
    			mType);
    	w.writeHermitian();
    	w.closeFiles();
    }

}

