/**
* This class monitors "gits"
* git = grounded bit 
* grounded bits occupy bits from 
* _minGit to _numBits-1.
* Their state is expected to be |0>
* if they are "ready". They
* may only be a target of sigma_x.
* They are ready iff they have been a target of
* a sigma_x an even number of times.
*/
public class GitsMonitor 
{
	private  int _numBits = 0;//use numBits<0 to disable all commands
	private  int _minGit = 0;
	private boolean[] _gitIsReady = null;
	/**
	 *
	 */   	
	public GitsMonitor(int numBits, int numGits){
		_numBits = numBits;
		if(numBits<0){ return; }
		Throw.If(numGits > numBits,
				"number of gits must be <= number of bits");
		_minGit = numBits - numGits;
		_gitIsReady = Globals.T_vec(numGits);

	}
	/**
	 *
	 */   	
	public int getNumBits(){
		return _numBits;

	}		
	/**
	 *
	 */   	
	public int getNumGits(){
		return _numBits - _minGit;

	}		
	/**
	 *
	 */   	
	public boolean isGit(int bitPos){
		if(_numBits<0){ return false; }
 		return bitPos >= _minGit;
	}
	/**
	 *
	 */   	
	public void checkIsNotGit(int bitPos){
		if(_numBits<0){ return; }
		Throw.If(bitPos>=_minGit,
			"this bit position is NOT supposed to be in grounded sector");
	}
	/**
	 *
	 */   	
	public void checkIsGit(int bitPos){
		if(_numBits<0){ return; }
		Throw.If(bitPos<_minGit,
			"this bit position is supposed to be in grounded sector");

	}
	/**
	 *
	 */   	
	public void noteGBreakingOpAt(int bitPos){
		//ground breaking operation
		if(_numBits<0){ return; }
		checkIsNotGit(bitPos);
	}
	/**
	 *
	 */   	
	public void noteXopAt(int bitPos){
		if(_numBits<0){ return; }
		if(isGit(bitPos)){
			_gitIsReady[bitPos - _minGit] = !_gitIsReady[bitPos - _minGit];
		}
	}
	/**
	 *
	 */   	
	public void note(String lineName, String endOfEngLine){
		int tarBitPos, tarBitPos1, tarBitPos2;
		String[] tokens, subTokens;
		if(lineName.equals("SWAP")){
			tokens = endOfEngLine.trim().split("[\\s]+");
			tarBitPos1 = Integer.parseInt(tokens[0]);
			tarBitPos2 = Integer.parseInt(tokens[1]);
			noteGBreakingOpAt(tarBitPos1);
			noteGBreakingOpAt(tarBitPos2);
		}else{
			tokens = ("x" + endOfEngLine).split("AT");
			//prefix "x" just to be safe
			if(tokens[1]!=null){
				subTokens = tokens[1].trim().split("[\\s]+");
				tarBitPos = Integer.parseInt(subTokens[0]);
		    	if(lineName.equals("SIGX"))
		    	{
		    		noteXopAt(tarBitPos);
		    	}else if(lineName.equals("SIGY")||
		    			lineName.equals("SIGZ")||
		    			lineName.equals("HAD2")||
		    			lineName.equals("ROTX")||
		    			lineName.equals("ROTY")||
		    			lineName.equals("ROTZ")||
		    			lineName.equals("ROTN"))
		    	{
		    		noteGBreakingOpAt(tarBitPos);
		    	}
			}
			
		}
	}
	/**
	 *
	 */   	
	public boolean gitIsReady(int bitPos){
		if(_numBits<0){ return false; }
		checkIsGit(bitPos);
		return _gitIsReady[bitPos - _minGit];
	}

}
