/**
 * This class stores commonly used constants and 
 * functions of global character.
 */
public class Globals {
    
	public final static String VERSION = "Ver. 1.6";
    //public final static double PI = 4*Math.atan(1.0); use Math.PI
    public final static double TOLERANCE = 1e-11;
    public final static Complex COMPLEX0 = new Complex(0.0);
    public final static Complex COMPLEX1 = new Complex(1.0);
    //Pauli matrices and two dim identity
    public final static Complex[][] SIGX_M = 
        {{COMPLEX0, COMPLEX1},
        {COMPLEX1, COMPLEX0}};
    public final static Complex[][] SIGY_M = 
        {{COMPLEX0, new Complex(0.0, -1.0)},
        {new Complex(0.0, 1.0), COMPLEX0}};
    public final static Complex[][] SIGZ_M = 
        {{COMPLEX1, COMPLEX0},
        {COMPLEX0, new Complex(-1.0, 0.0)}};
    public final static Complex[][] ID2_M = 
        {{COMPLEX1, COMPLEX0},
        {COMPLEX0, COMPLEX1}};
    //one bit Hadamard
    public final static Complex[][] HAD2_M = 
        {{new Complex(1/Math.sqrt(2), 0.0), new Complex(1/Math.sqrt(2), 0.0)},
        {new Complex(1/Math.sqrt(2), 0.0), new Complex(-1/Math.sqrt(2), 0.0)}};
    
    public static double log2(double x){
        return Math.log(x)/Math.log(2);      
    }
    public static double sinc(double x){
    	//sinc(x) 	= sin(x)/x
    	//			=\sum_{k=0}^{\infty} x^{2k}(-1)^k/(2k+1)!

    	double y;
    	if(Math.abs(x)<1e-3){
    		double xSq = x*x;
    		
    		double kSummand = 1;
    		y = 1;    		    		
    		int k = 1;
    		do{
    			kSummand *= -xSq/((2*k+1)*2*k);
    			y += kSummand;
    			k++;
    		}while(Math.abs(kSummand)>1e-15);
    		
    	}else{
    		y = Math.sin(x)/x;
    	}
    	
    	return y;
    	
    }
	/**
	 * 
	 */
	public static boolean[] T_vec(int length){
		boolean[]  vec = new boolean[length];
		for(int j=0; j<length; j++){
			vec[j] = true;
		}
		return  vec;
	}
	/**
	 * 
	 */	
	public static boolean[] F_vec(int length){
		boolean[]  vec = new boolean[length];
		for(int j=0; j<length; j++){
			vec[j] = false;
		}
		return  vec;
	}
	/**
	 * IdentityMap = IdentityPermutation
	 */	
	public static int[] identityPermutation(int length){
		int[]  vec = new int[length];
		for(int j=0; j<length; j++){
			vec[j] = j;
		}
		return  vec;
	}

	/**
	 * 
	 */	
	public static double[] negVec(double[] vec){
		int len = vec.length;
		double[] negVec = new double[len];
		for(int j=0; j<len; j++){
			negVec[j] = -vec[j];
		}
		return negVec;
	}
	/**
	 * 
	 */	
	public static double between0and1(double x){
/*		
		double newX = x;
		while(newX<0){
			newX += 1;
		}
		while(newX>= 1){
			newX -= 1;
		}
		return newX;
*/
		double  newX;
		Throw.If(Math.abs(x)>1e5, "use smaller numbers");
		if(x>=0){
			newX = x - (long)x;
			//eg. x= 2.3
			//2.3 - (long)2.3 = .3
		}else{
			newX = 1 + x + (long)(-x);
			//eg. x= -2.3
			//1 - 2.3 + (long)(2.3)= .7 
		}
		return newX;

	}
	/**
	 * 
	 */	
	public static double limitedRadians(double x){
		double factor = 2*Math.PI;
		return factor*between0and1(x/factor);
	}
	/**
	 * 
	 */	
	public static double[] limitedRadians(double[] vec){
		int len = vec.length;
		double factor = 2*Math.PI;
		double[] newVec = new double[len];
		for(int j=0; j<len; j++){
			newVec[j]= factor*between0and1(vec[j]/factor);
		}
		                        
		return newVec;
	}
	/**
	 * 
	 */	
	public static boolean isZero(double x){
		return Math.abs(x) < 1e-8;
	}
	/**
	 * 
	 */	
	public static boolean  isZero(double[] vec){
		boolean ans = true;
		int len = vec.length;
		for(int j=0; j<len; j++){
			if(!isZero(vec[j])){
				ans = false;
				break;
			}
		}
		return ans;
	}
	/**
	 * 
	 */	
	public static boolean[]  boolArrayFromInt(int index, int arraySize){
		Throw.If(index>= 1<<arraySize, "Could not convert integer " +
				"to a boolean array, because specified array size " +
				"is too small");
				
		boolean[] array = new boolean[arraySize];
		int balance = index;
		int binInt;
		for(int j=0; j<arraySize; j++){
			binInt = balance%2;
			array[j] = binInt==1?true:false;
			balance  = (balance- binInt)/2;
		}
		return array;		
		
	}
	/**
	 * 
	 */	
	public static int  intFromBoolArray(boolean[] array){
		int arraySize = array.length;
		int sum = 0;
		int coef = 1;
		for(int j=0; j<arraySize; j++){
			sum += coef*(array[j]==true?1:0);
			coef *= 2;
		}
		return sum;	
	}
}

