/**
 * This class multiplies a vector of doubles by
 * the Hadamard Matrix
 */	
public class HadamardTransform {
	/**
	 * 
	 */	
	public static  double[]		of(
		int 			numBits,
		double[]		inVec)
	{
		Throw.If(numBits<1, 
			"numBits for Hadamard Transform must be >=1");
		int		len = (1 << numBits);
		Throw.If(inVec.length!=len, 
			"inVec for Hadamard Transform has wrong length");
		int		halfLen = (1 << (numBits-1));
		
		double[]	prevVec  = new double[len];
		double[]	outVec 	 = new double[len];
		int beta, j, i;
		double x, y;
		double root2 = Math.sqrt(2);
		for(i=0; i < len; i++){
			outVec[i] = inVec[i];
		}
		for(beta=0; beta < numBits; beta++){
			for(i=0; i < len; i++){
				prevVec[i] = outVec[i];
			}
			j = 0;
			for(i=0; i < halfLen; i++){
				x = prevVec[j];
				y = prevVec[j + 1];	
				outVec[i]= (x + y)/root2;
				outVec[halfLen + i] = (x - y)/root2;
				j +=2;
			}			
		}
		return  outVec;
	}
	/**
	 * 
	 */	    
	public static void main(String[] args){
		int  numBits = 3;
		int  len = 1<<numBits;
		double[] inVec = new double[len];
		double[] outVec = null;

		
		for(int j=0; j<len; j++){
			inVec[j] = Math.random() - .5;
			System.out.print(inVec[j] +"\t");
		}
		
		System.out.println("");
		outVec = of(numBits, inVec);
		for(int j=0; j<len; j++){
			System.out.print(outVec[j] +"\t");
		}
		
		System.out.println("");
		outVec = of(numBits, outVec);
		for(int j=0; j<len; j++){
			System.out.print(outVec[j] +"\t");
		}
		
	}
}
