import java.util.ArrayList;

/**
 * This class stores all the information of an English
 * file as an array. The array index corresponds to
 * the line-count number in the English file 
 * (assuming first line of English file is zero). 
 */
public class LineList extends  ArrayList<Line> {
	
	/**
	 *
	 */
	public void addLOOP(
		int lineNum, 
		int reps)//repetitions
	{
		Line	line = new Line();
		line._lineType = LineEnum.LOOP;
		Throw.If( lineNum != size(),
			"line number must equal current size of array");
		line._ints = new int[2];
		line._ints[0] = reps;//reps = loop repetitions
		line._ints[1] = 0;//timesVisited
		add(line);		
		
    }
	/**
	 *
	 */
	public void addNEXT(
		int loopNum)
	{
		Line	line = new Line();
		line._lineType = LineEnum.NEXT;
		line._ints = new int[1];
		line._ints[0] = loopNum;
		add(line);		
		
    }
	/**
	 *
	 */
	public void addDummyGate()
	{
		Line	line = new Line();
		line._lineType = LineEnum.SWAP;//anything, other than NEXT or LOOP, will do here
		add(line);
	}
	/**
	 *
	 */
	public void addSWAP(
		int bit1, 
		int bit2, 
		TFControls controls)
	{
		Line	line = new Line();
		line._lineType = LineEnum.SWAP;
		line._ints = new int[2];
		line._ints[0] = bit1;
		line._ints[1] = bit2;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public void addPHAS(
		double angleDeg, 
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		line._lineType = LineEnum.PHAS;
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._doubles = new double[1];
		line._doubles[0] = angleDeg;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public void addP_PH(
		int projectionBit, 
		double angleDeg,
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		if(projectionBit==0){			
			line._lineType = LineEnum.P0PH;			
		}else if(projectionBit==1){			
			line._lineType = LineEnum.P1PH;			
		}else{
			Throw.If(true, "unsupported phase shift gate");	
		}
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._doubles = new double[1];
		line._doubles[0] = angleDeg;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public void addSIG(
		int direction,//1=X, 2=Y, 3=Z
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		switch(direction){
			case 1:
				line._lineType = LineEnum.SIGX;
				break;
			case 2:
				line._lineType = LineEnum.SIGY;
				break;
			case 3:
				line._lineType = LineEnum.SIGZ;
				break;
			default:
				Throw.If(true, "unsupported SIG gate");				
		}
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public void addHAD2(
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		line._lineType = LineEnum.HAD2;
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._controls = new TFControls(controls, 0);
		add(line);

	}
	/**
	 *
	 */
	public void addROT(
		int direction,//1=X, 2=Y, 3=Z
		double angleDeg,
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		switch(direction){
			case 1:
				line._lineType = LineEnum.ROTX;
				break;
			case 2:
				line._lineType = LineEnum.ROTY;
				break;
			case 3:
				line._lineType = LineEnum.ROTZ;
				break;
			default:
				Throw.If(true, "unsupported ROT gate");				
		}
		line._doubles = new double[1];
		line._doubles[0] = angleDeg;
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public void addROTN(
		double angleX_deg,
		double angleY_deg,
		double angleZ_deg,
		int targetBitPosition, 
		TFControls controls)
	{
		Line	line = new Line();
		line._lineType = LineEnum.ROTN;
		line._doubles = new double[3];
		line._doubles[0] = angleX_deg;
		line._doubles[1] = angleY_deg;
		line._doubles[2] = angleZ_deg;		
		line._ints = new int[1];
		line._ints[0] = targetBitPosition;
		line._controls = new TFControls(controls, 0);
		add(line);
	}
	/**
	 *
	 */
	public int multiplyYourself(UnitaryMat  matrix){
		//return lenOfSEO
		int lenOfSEO = 0;
		int currentLine = 0;
		while(currentLine < size()){
			Line l = get(currentLine);
			switch(l._lineType){
				case 	LOOP:
					//increment timesVisited
					l._ints[1]++;
					break;
				case 	NEXT:
					int loopNum = l._ints[0];
					if(get(loopNum)._ints[0]/*reps*/
					      != 
					   get(loopNum)._ints[1]/*timesVisited*/
					){
						currentLine = loopNum;
						//increment timesVisited
						get(loopNum)._ints[1]++;
					}else{
						//set timesVisited to zero
						get(loopNum)._ints[1] = 0;
					}
					break;
				case 	SWAP:
					lenOfSEO++;
					matrix.premultiplyByControlledBitSwap(
						l._ints[0],//bit1
						l._ints[1],//bit2
						l._controls);
					break;
				case 	PHAS:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitPhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	P0PH:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitP0PhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	P1PH:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitP1PhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	SIGX:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigX());
					break;
				case 	SIGY:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigY());
					break;
				case 	SIGZ:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigZ());
					break;
				case 	HAD2:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitHadamard());
					break;
				case 	ROTX:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.X_AXIS));
					break;
				case 	ROTY:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.Y_AXIS));
					break;
				case 	ROTZ:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.Z_AXIS));
					break;
				case 	ROTN:
					lenOfSEO++;
					matrix.premultiplyByControlledGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							l._doubles[1]*Math.PI/180, 
							l._doubles[2]*Math.PI/180));
					break;
				default:
					throw new RuntimeException(
					"unsupported line type");
			}//end cases
			currentLine++;
		}//end while
		return lenOfSEO;
	}
	/**
	 *
	 */
	public int writeYourself(SEO_writer  wr){
		//return lenOfSEO
		int lenOfSEO = 0;
		int currentLine = 0;
		while(currentLine < size()){
			Line l = get(currentLine);
			switch(l._lineType){
				case 	LOOP:
					//increment timesVisited
					l._ints[1]++;
					break;
				case 	NEXT:
					int loopNum = l._ints[0];
					if(get(loopNum)._ints[0]/*reps*/
					      != 
					   get(loopNum)._ints[1]/*timesVisited*/
					){
						currentLine = loopNum;
						//increment timesVisited
						get(loopNum)._ints[1]++;
					}else{
						//set timesVisited to zero
						get(loopNum)._ints[1] = 0;
					}
					break;
				case 	SWAP:
					lenOfSEO++;
					wr.writeControlledBitSwap(
						l._ints[0],//bit1
						l._ints[1],//bit2
						l._controls);
					break;
				case 	PHAS:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitPhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	P0PH:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitP0PhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	P1PH:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitP1PhaseFactor(
							l._doubles[0]*Math.PI/180));
					break;
				case 	SIGX:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigX());
					break;
				case 	SIGY:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigY());
					break;
				case 	SIGZ:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new SigZ());
					break;
				case 	HAD2:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitHadamard());
					break;
				case 	ROTX:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.X_AXIS));
					break;
				case 	ROTY:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.Y_AXIS));
					break;
				case 	ROTZ:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							OneBitRot.Z_AXIS));
					break;
				case 	ROTN:
					lenOfSEO++;
					wr.writeControlledOneBitGate(
						l._ints[0],//targetBitPosition
						l._controls,
						new OneBitRot(
							l._doubles[0]*Math.PI/180, 
							l._doubles[1]*Math.PI/180, 
							l._doubles[2]*Math.PI/180));
					break;
				default:
					throw new RuntimeException(
					"unsupported line type");
			}//end cases
			currentLine++;
		}//end while
		return lenOfSEO;
	}

}
