/**
 * This class stores a list of MultiControls,
 * which might include multiplexor controls, 
 * P0=nbar controls and P1=n controls
 */
public class MultiControls extends  Controls<Integer>{
	//negative types are reserved
	public static int P0_TYPE=-1, P1_TYPE=-2, NONE_TYPE=-3;
	
	/**
	 * 
	 */	
	public MultiControls(){
		super();
	}
	/**
	 * 
	 */	
	public MultiControls(int numBits, int numControls){
		super(numBits, numControls);
		_types  = new  Integer[numControls];
	}
	/**
	 * 
	 */	
	public MultiControls(MultiControls controls, int extraNumBits){
		super(controls, extraNumBits);
		_types = new Integer[_numControls];
		for(int j=0; j<_numControls; j++){
			_types[j] = controls._types[j];
		}
	}
	/**
	 * 
	 */	
	public int  getNumNonNegControls(){
		int num=0;
		for(int j=0;  j<_numControls; j++){
			if(_types[j] >= 0){ num++; }
		}
		return num;
	}	

	/**
	 * 
	 */	
	public MultiControls refreshedControls(CktEmbedder  emb){
		MultiControls  aftControls = new MultiControls();
		if(emb.isIdentity()){
			aftControls   =  this;
		}else{
			Throw.If(_numBits != emb.getNumBitsBef(),
					"controls cannot be reset with CktEmbedder");
			
//			int numBitsBef = _numBits;
			int numBitsAft = emb.getNumBitsAft();
			
			int newNumControls = _numControls + emb.getNumNewControls();
			int[] newBitPositions = new int[newNumControls];
			Integer[] newTypes = new Integer[newNumControls];
			
			int[] typesAft = new int[numBitsAft];

			for(int aft=0; aft < numBitsAft; aft++){
				typesAft[aft] = NONE_TYPE;//start off as none
				if(emb.isNewBit(aft)){
					if(emb.isP0Control(aft)){
						typesAft[aft] = P0_TYPE;
					}else if(emb.isP1Control(aft)){
						typesAft[aft] = P1_TYPE;
					}
				}
			}
			for(int j=0; j < _numControls; j++){
					typesAft[emb.aft(_bitPositions[j])] = _types[j];			
			}
			
			//numBitsAft > 0 because numBitsAft>=numBitsBef>0
			int c = 0;
			for(int aft = numBitsAft-1; aft>=0; aft--){;
				if(typesAft[aft]  != NONE_TYPE){					
					newBitPositions[c] = aft;
					newTypes[c] = typesAft[aft];
					c++;		
				}
			}
			
			aftControls._numBits = numBitsAft;
			aftControls._numControls = newNumControls;
			aftControls._bitPositions = newBitPositions;
			aftControls._types = newTypes;
		}
		return  aftControls;
	}

}
