import java.util.Comparator;

/**
 * This class stores an alphabetic list of names.
 * The list remembers the order in which it was filled.
 * lexNum = lexicographic number (i.e., alphabetic order) of name
 * fillNum = fill number of name
 */
public class NamesList{
	private String[] _sortedNames;
	private int[] _fillNumOfLexNum;
	private int[] _lexNumOfFillNum;
	/**
	 * 
	 */	
	private class NameIntPair{
		private String  _name;
		private int  _int;
		
		public NameIntPair(String name, int j){
			_name = name;
			_int = j;		
		}	
	}
	/**
	 * 
	 */	
	private class NameIntPairComparator implements Comparator<NameIntPair>{
		public int compare(NameIntPair p1, NameIntPair p2){
			return (p1._name).compareTo(p2._name);
		}
	}
	/**
	 * 
	 */	
	public NamesList(String[] names){
		
		int len = names.length;
		int j;
		NameIntPair[]  pairs = new NameIntPair[len];
		for(j=0; j<len; j++){
			pairs[j] = new NameIntPair(names[j],j);
		}		
		java.util.Arrays.sort(pairs, new NameIntPairComparator());
		_sortedNames = new String[len];
		_fillNumOfLexNum = new int[len];
		_lexNumOfFillNum = new int[len];

		for(j=0; j<len; j++){
			_sortedNames[j] = pairs[j]._name;
			_fillNumOfLexNum[j] = pairs[j]._int;
		}		
		
		for(j=0; j<len; j++){
			_lexNumOfFillNum[_fillNumOfLexNum[j]] = j;
		}
	
		
		//check that they are all different
		for(j=0; j<len-1; j++){
			if(_sortedNames[j].equals(_sortedNames[j+1])){
				Throw.If(true, 
					"The following name appears more than once in a NamesList:\n" + 
					"\"" + _sortedNames[j] + "\"");
			}
		}
		
	}
	/**
	 * 
	 */
	public int getNumNames(){
		return _sortedNames.length;
	}
	/**
	 * 
	 */	
	public String getName(int fillNum){
		return _sortedNames[_lexNumOfFillNum[fillNum]];
	
	}
	/**
	 * 
	 */
	public int getFillNum(String name){
		int lexNum = 
			java.util.Arrays.binarySearch(_sortedNames, name);
		Throw.If(lexNum<0, "NamesList can't find the name:\n" +
			"\"" + name + "\"");
		return _fillNumOfLexNum[lexNum];
	}
	/**
	 * 
	 */
	public int getFillNum(int lexNum){
		return _fillNumOfLexNum[lexNum];
	}
	/**
	 * 
	 */
	public int getLexNum(int fillNum){
		return _lexNumOfFillNum[fillNum];
	}
}