/**
 * This class creates instances of various
 * important probability matrices.
 */	
public class ProbMatrices {
	/**
	 * 
	 */	
	public static double[][] randomProbMatrix(int numRows, int numCols){
		double[][]  probMatrix = new  double[numRows][numCols];
		   int row, col;
		   double   rowSum, x;
		   for(col=0; col<numCols; col++){
			   rowSum =  0;
			   for(row=0; row<numRows; row++){
				   x = Math.random();
				   probMatrix[row][col]  = x;
				   rowSum += x;
			   }		   
			   for(row=0; row<numRows; row++){
				   probMatrix[row][col]  /= rowSum;
			   }		   
		   }
		   checkColsAddToOne(probMatrix);
		   return  probMatrix;
	       
	}
	
	/**
	 * 
	 */
	public static double[][] metropolisProbMatrix(
			MinimizationProblem p,
			double beta)
	{
		int numStates = p.getNumStates();
		double[][]  probMatrix = new  double[numStates][numStates];
		
		int row, col;
		double rowSum;
		double upBdNeig = p.getUpBdNeig();
		for(col=0; col<numStates; col++){
			rowSum = 0;
			for(row=0; row<numStates; row++){
				if(row!=col){
					if(p.areNeighbors(row,col)){
						probMatrix[row][col] = 
							Math.min(
								1, 
								Math.exp(beta*(p.energy(col)-p.energy(row)))
							)/upBdNeig;
						//probMatrix[row][col] is Prob(row|col). 
						//For minimization problem,
						//this prob is always 1 if going downhill (E(col)>E(row))
						//but less than one if going uphill.
					}else{//not neighbors
						probMatrix[row][col] = 0;				
					}
				}			
			}
			probMatrix[col][col] = 1 - rowSum;
		}
		checkColsAddToOne(probMatrix);
		return probMatrix;
	}
	/**
	 * 
	 */	
	public static void checkColsAddToOne(double[][] probMat){
		int numRows = probMat.length;
		int numCols = probMat[0].length;
		double sum;
		for(int col=0; col<numCols; col++){
			sum = 0.0;
			for(int row=0; row<numRows; row++){
				sum += probMat[row][col];
			}
			Throw.If(Math.abs(sum-1)>10E-8,
				"Probabilities of a column of a " +
				"probMat don't sum to one");
		}
	}
}
