import java.io.PrintWriter;


/**
 * PsiEvOp = Psi Evolution Operator
 * This class writes English and Picture files
 * for the evolution operator U which
 * when acting on |0>_\alpha produces |psi>_\alpha
 * where alpha is composed of _numQudits qudits
 * and _bitsPerDit is the number of qubits per qudit.
 * example of labelling convention:
 * _numQudits=2 _bitsPerDit =3
 * qubit  qudit subqubit
 * 0		0		0
 * 1		0		1	
 * 2		0		2
 * 3		1		0
 * 4		1		1
 * 5		1		2
 */

abstract public class PsiEvOpSEO_writer  extends SEO_writer{
	private int _numQudits;// the number of qudits in alpha
	private int _bitsPerDit;// the number of qubits in each qudit

	/**
	 * 
	 */	
	public PsiEvOpSEO_writer(
	        String 	filePrefix,
	        int numQudits,
	        int bitsPerDit)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numQudits = numQudits;
		_bitsPerDit = bitsPerDit;
		_numBits = _numQudits*_bitsPerDit;
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public PsiEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numQudits,
        int bitsPerDit)
    {
    	super(englishOut, pictureOut, emb);
		_numQudits = numQudits;
		_bitsPerDit = bitsPerDit;
		_numBits = _numQudits*_bitsPerDit;
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	abstract public void write();
	/**
	 * writes Hermitian Conjugate
	 */
	abstract public void writeHermitian();

}

