import java.io.*;

/*
 * This class implements the identity
 * exp(i ax*sigx + ay*sigy + az*sigz)=//down =left to right
 * exp(i*azi*sigx)//0
 * exp(i*lat*sigz)//1 
 * exp(i mag(a) sigx)//2
 * exp(-i*lat*sigz)//3
 * exp(-i*azi*sigx)//4
 * 
 *where 
 *lat = (.5)atan2(sqrt(az^2+ay^2), ax)
 *azi = (.5)atan2(az, ay)
 *
 */

public class ROTN_expansion {
	/*
	 * input is ax,ay,az
	 * output is a vector with components (azi, lat, mag(a), -lat, -azi)
	 */
	public static double[] getAngles(double ax, double ay, double az)
	{
		double ax2 = Math.pow(ax, 2);
		double ay2 = Math.pow(ay, 2);
		double az2 = Math.pow(az, 2);
		double lat = .5*Math.atan2(Math.sqrt(az2+ay2), ax);
		double azi = .5*Math.atan2(az, ay);
		double mag_a = Math.sqrt(ax2 + ay2 + az2);
		
		double[] anglesOut = {azi, lat, mag_a, -lat, -azi};
		return anglesOut;
	}
	/**
	 * 
	 */    
	public static void main(String[] args){
		
		double ax = .3;
		double ay = 2.1;
		double az = -5.6;

		
		double[] anglesOut = ROTN_expansion.getAngles(ax, ay, az);
		
		SEO_writer wr;	
		wr = new SEO_writer("tempo", 2);
    	wr.writeOneBitGate(0, new OneBitRot(anglesOut[0], OneBitRot.X_AXIS));
    	wr.writeOneBitGate(0, new OneBitRot(anglesOut[1], OneBitRot.Z_AXIS));
    	wr.writeOneBitGate(0, new OneBitRot(anglesOut[2], OneBitRot.X_AXIS));
    	wr.writeOneBitGate(0, new OneBitRot(anglesOut[3], OneBitRot.Z_AXIS));
    	wr.writeOneBitGate(0, new OneBitRot(anglesOut[4], OneBitRot.X_AXIS));
    	wr.closeFiles();
    	
    	System.out.println(
    		new SEO_readerMu(
    			"tempo",
    			2, //numBits
    			false) // useSparse
    	);
	   
    	wr = new SEO_writer("tempo", 2);
    	wr.writeOneBitGate(0, new OneBitRot(ax, ay, az));
    	wr.closeFiles();
    	
    	System.out.println(
    		new SEO_readerMu(
    			"tempo",
    			2, //numBits
    			false) // useSparse
    	);


 
	}

}
