import java.io.*;
import java.util.Scanner;

/**
 * A SEO is a Sequence of Elementary Operations.
 * This class reads a SEO from an English file
 * and stores it in a LineList
 */
 
public class SEO_reader {

    protected String      _filePrefix = "";
    protected int         _numBits;
    protected Scanner  	 _englishIn = null;
    protected LineList 	_lineList;
 

	/**
	 *
	 */
    public SEO_reader(
        String filePrefix, 
        int numBits) 
	{
		_filePrefix = filePrefix;
		_numBits = numBits;
		try{
			File  file = new File(filePrefix + "_eng.txt");
			_englishIn = new Scanner(file);
					
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
		_lineList = new LineList();
		 
	}
	/**
	 *
	 */
    public SEO_reader(
        Scanner englishIn, 
        int numBits) 
	{
    	_englishIn = englishIn;
    	_numBits = numBits;
    	_lineList = new LineList();
	}	
    
	/**
	 *
	 */    
    protected void closeFile(){
        if(_englishIn != null){
            _englishIn.close();
        }
    }
	/**
	 * 
	 */
    protected void readStoreAll(){
		String lineName = "";
		while(_englishIn.hasNext()){
			lineName = _englishIn.next();
			readStore(lineName);
		}       	                    
    }    	               
	/**
	 * 
	 */
    public	LineList getLineList(){
    	return _lineList;
    	
    }
	/**
	 * 
	 */
    public	void readStore(String lineName){
		if(lineName.equals("LOOP")){
			readStoreLOOP();
		}else if(lineName.equals("NEXT")){
			readStoreNEXT();
		}else if(lineName.equals("SWAP")){
			readStoreSWAP();
		}else if(lineName.equals("PHAS")){
			readStorePHAS();
		}else if(lineName.equals("P0PH")){
			readStoreP_PhaseFactor(0);
		}else if(lineName.equals("P1PH")){
			readStoreP_PhaseFactor(1);
		}else if(lineName.equals("SIGX")){
			readStoreSIG(1);
		}else if(lineName.equals("SIGY")){
			readStoreSIG(2);
		}else if(lineName.equals("SIGZ")){
			readStoreSIG(3);
		}else if(lineName.equals("HAD2")){
			readStoreHAD2();
		}else if(lineName.equals("ROTX")){
			readStoreROT(1);
		}else if(lineName.equals("ROTY")){
			readStoreROT(2);   
		}else if(lineName.equals("ROTZ")){
			readStoreROT(3);   
		}else if(lineName.equals("ROTN")){
			readStoreROTN();  
		}else{
			throw new RuntimeException(
				"reading an unsupported LineEnum");                
		}

    	
    }
	/**
	 * 
	 */
    private void readStoreLOOP(){
    	int lineNum = _englishIn.nextInt();
    	_englishIn.next();//skip "REPS:"
    	int reps = _englishIn.nextInt();
 		_lineList.addLOOP(lineNum, reps);
    }
	/**
	 * 
	 */
    private void readStoreNEXT(){
    	int loopNum = _englishIn.nextInt();
 		_lineList.addNEXT(loopNum);
    }

	/**
	 * 
	 */
    private void readStoreSWAP(){
    	int bit1 = _englishIn.nextInt();
    	int bit2 = _englishIn.nextInt();
    	TFControls controls = readTFControls(_numBits, _englishIn);
		_lineList.addSWAP(
    			bit1,
    			bit2,
    			controls);                   	
    }
	/**
	* 
	*/   	
    private void readStorePHAS(){
    	double angleDeg = _englishIn.nextDouble();
    	_englishIn.next();//skip "AT"
    	int targetBitPosition = _englishIn.nextInt();
    	TFControls controls = readTFControls(_numBits, _englishIn);
    	_lineList.addPHAS(
    			angleDeg,
    			targetBitPosition,
    			controls);                   	
    }
    
	/**
	 *
	 */
    private void readStoreP_PhaseFactor(int projectionBit){
	   double angleDeg = _englishIn.nextDouble();
	   _englishIn.next();//skip "AT"
       int targetBitPosition = _englishIn.nextInt();
       TFControls controls = readTFControls(_numBits, _englishIn);
       //OneBitGate g = null;
       switch(projectionBit){
			case 0:
				//g = new OneBitP0PhaseFactor(angleDeg*Math.PI/180);
				break;
			case 1:
				//g = new OneBitP1PhaseFactor(angleDeg*Math.PI/180);
				break;
			default:
				Throw.If(true, "unsupported phase shift gate");
       }

       _lineList.addP_PH(
       		projectionBit,
       		angleDeg,
           	targetBitPosition,
           	controls);                   	
   }

	/**
	 *
	 */
    private void readStoreSIG(int direction){
    	_englishIn.next();//skip "AT"
        int targetBitPosition = _englishIn.nextInt();
        TFControls controls = readTFControls(_numBits, _englishIn);
        //OneBitGate g = null;
        switch(direction){
        	case 1:
        		//g = new SigX();
        		break;
        	case 2:
        		//g = new SigY();
        		break;
        	case 3:
        		//g = new SigZ();
        		break;
        	default:
        		Throw.If(true, "unsupported gate direction");
        }
        _lineList.addSIG(
            	direction,
            	targetBitPosition,
            	controls);                   	
    }
	/**
	 * 
	 */    
    private void readStoreHAD2(){
    	_englishIn.next();//skip "AT"
    	int targetBitPosition = _englishIn.nextInt();
    	TFControls controls = readTFControls(_numBits, _englishIn);
    	_lineList.addHAD2(
    			targetBitPosition,
    			controls);                   	
    }

	/**
	 *
	 */
    private void readStoreROT(int direction){
    	double angleDeg = _englishIn.nextDouble();
        _englishIn.next();//skip "AT"
        int targetBitPosition = _englishIn.nextInt();
        TFControls controls = readTFControls(_numBits, _englishIn);
        _lineList.addROT(
        		direction,
        		angleDeg,
            	targetBitPosition,
            	controls);                   	
    }
	 /**
	  * 
	  */
    private void readStoreROTN(){
    	double angleX_deg = _englishIn.nextDouble();
    	double angleY_deg = _englishIn.nextDouble();
    	double angleZ_deg = _englishIn.nextDouble();
    	_englishIn.next();//skip "AT"
    	int targetBitPosition = _englishIn.nextInt();
    	TFControls controls = readTFControls(_numBits, _englishIn);
    	_lineList.addROTN(
    			angleX_deg,
    			angleY_deg,
    			angleZ_deg,
    			targetBitPosition,
    			controls);                   	   

    }
	/**
	 * 
	 */
    public static TFControls readTFControls(
    		int    		numBits,
    		Scanner 	englishIn)
    {
    	//safe to use when no "IF"
    	//when no "IF", will return controls with _numControls=0
    	
    	TFControls controls = new TFControls();//created with _numControls=0
    	int numControls = 0;
        //go to next line and return what was skipped.
    	//if no "IF", will just return "\n"
        String  str = englishIn.nextLine();
        //tokenize string with one or more white space characters as delimiter
        String[] tokens = (str.trim()).split("[\\s]+");
        if(tokens.length > 0){
            numControls = (tokens.length - 1);//first token is the word "IF"
            controls = new TFControls(numBits, numControls);
            for(int j=0; j<numControls; j++){
                //Control bit positions will be in decreasing order,
                //because that is how the SEO_writer wrote them.
            	int len = tokens[j+1].length();//j=0  token is "IF"
            	String subString = tokens[j+1].substring(0, len-1);//doesn't include T/F
                controls.setBitPos(j, 
                		Integer.parseInt(subString));
                //last character of token is T or F
                char ch = tokens[j+1].charAt(len-1);
                if(ch=='T'){
                	controls.setType(j, true);
                }else if(ch=='F'){
                	controls.setType(j, false);
                }else{
                	Throw.If(true, "T or F expected for control type");
                }
            }
        }
        return controls;
    }
    /**
	 * 
	 */
    public static  MultiControls readMultiControls(
    		int numBits,
    		Scanner   englishIn)
    {
    	
    	MultiControls controls = new MultiControls();
        int numControls = 0;
      //go to up to "BY" and return what was skipped
        String  str = englishIn.findInLine(".*BY");
        //tokenize string with one or more white space characters as delimiter
        String[] tokens = (str.trim()).split("[\\s]+");
        if(tokens.length > 1){
        	//first token is the word "IF"
        	//last token is "BY"
            numControls = (tokens.length - 2);
            controls = new MultiControls(numBits, numControls);
            for(int j=0; j<numControls; j++){
                //Control bit positions will be in decreasing order,
                //because that is how the SEO_writer wrote them.
            	int indexOfParenthesis = tokens[j+1].indexOf('(');//j=0 token is "IF"
            	int len = tokens[j+1].length();
            	if(indexOfParenthesis == -1){//no parenthesis was found      		
                	String subString = tokens[j+1].substring(0, len-1);//doesn't include T/F
                    controls.setBitPos(j, 
                    		Integer.parseInt(subString));
                    //last character of token is T or F
                    char ch = tokens[j+1].charAt(len-1);
                    if(ch=='T'){
                    	controls.setType(j, MultiControls.P1_TYPE);
                    }else if(ch=='F'){
                    	controls.setType(j, MultiControls.P0_TYPE);
                    }else{
                    	Throw.If(true, "T or F expected for control type");
                    }
            	}else{
            		String bitPosStr = tokens[j+1].substring(0, indexOfParenthesis);
            		String typeStr = tokens[j+1].substring(indexOfParenthesis+1, len);
            		controls.setControl(j, 
                    	Integer.parseInt(bitPosStr), Integer.parseInt(typeStr));
            	}
           }
        }
        return controls;
    }
    /**
	 * 
	 */
    public static  double[] readMP_Y_angles(
    	Scanner   englishIn)
    {
    	int numAngles = 0;
    	double[] radAngles = new double[0];
        //go to next line and return what was skipped
        String  str = englishIn.nextLine();
        //tokenize string with one or more white space characters as delimiter
        String[] tokens = (str.trim()).split("[\\s]+");
        if(tokens.length > 0){
            numAngles = tokens.length;
            radAngles = new double[numAngles];
            for(int j=0; j<numAngles; j++){
            	//see SEO_writer.writeControlledMultiplexorGate() for conversion
            	radAngles[j] =  Double.parseDouble(tokens[j])*Math.PI/180;
            }
        }
        return radAngles;   	
    
    }

}
