import java.io.*;

/**
 * A SEO is a Sequence of Elementary Operations.
 * This class writes an English and Picture file.
 */
public class SEO_writer {
	protected static int _lineCounter = 0;
    protected String  _filePrefix  ="";
    protected PrintWriter  _englishOut = null;
    protected PrintWriter  _pictureOut = null;
    protected CktEmbedder	_emb = null;
    protected int	_numBits;//_numBits = numBitsBef
    protected GitsMonitor  _gm = new GitsMonitor(-1,-1);
    	//GitsMonitor(-1,-1) produces a dummy _gm.
    	//replace _gm by non-trivial object if want to 
    	//truly monitor gits.
 
    
	/**
	 *
	 */
    public SEO_writer(
	   String filePrefix,
	   int numBits)
    {
         _filePrefix = filePrefix;
        Throw.If(numBits<2, "number of bits must be >= 2");
        _emb = new CktEmbedder(numBits);
        _numBits = numBits;

        
		_englishOut = openEngFile(filePrefix);
		_pictureOut = openPicFile(filePrefix);

    }

    /**
	 * 
	 */
    public SEO_writer(
    		PrintWriter  englishOut,
    		PrintWriter  pictureOut,
    		CktEmbedder  emb)
    {
    	
    	_englishOut = englishOut;
    	_pictureOut = pictureOut;
    	_emb  =  emb;
    	_numBits = emb.getNumBitsBef();
    	Throw.If(emb.getNumBitsAft()<2, "NumBitsAft must be >= 2");   

    }
	/**
	 * 
	 */
    public void setGitsMonitor(GitsMonitor  gm){
    	_gm = gm;
    }

	/**
	 * 
	 */    
    protected static PrintWriter openEngFile(String filePrefix){
    	PrintWriter englishOut=null;
		try{			
			//English File
			File  engFile = new File(filePrefix + "_eng.txt");
			englishOut = new PrintWriter(engFile);
		   
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}  
		return englishOut;
} 
	/**
	 * 
	 */    
    protected static PrintWriter openPicFile(String filePrefix){
    	
    	PrintWriter pictureOut=null;
		try{			
			//Picture File
			File  picFile = new File(filePrefix + "_pic.txt");
			pictureOut = new PrintWriter(picFile);
		   
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
		return pictureOut;
    	
    }

	/**
	 * 
	 */    
    protected void closeFiles(){
    	_lineCounter = 0;
       
	   if(_englishOut != null){
			_englishOut.close();
			_englishOut = null;
	   }
	   
	   if(_pictureOut != null){
			_pictureOut.close();
			_pictureOut = null;
	   }

    }
	/**
	 * 
	 */
    public static int getLineCount(){
    	return _lineCounter;
    }
	/**
	 * 
	 */	
	public int getNumBits(){
		return _numBits;
	}


	/**
	 *
	 */
    public void writeLOOP(int lineNum, int reps){
    	_lineCounter++;	
		Throw.If(_englishOut==null, "writing to a closed file");
		Throw.If(_pictureOut==null, "writing to a closed file");
		
		_englishOut.println("LOOP\t" + lineNum + "\tREPS:\t" + reps);
		_pictureOut.println("LOOP\t" + lineNum + "\tREPS:\t" + reps);

    }
	/**
	 *
	 */
    public void writeNEXT(int loopNum){
        _lineCounter++;
		Throw.If(_englishOut==null, "writing to a closed file");
		Throw.If(_pictureOut==null, "writing to a closed file");
		
		_englishOut.println("NEXT\t" + loopNum);
	    _pictureOut.println("NEXT\t" + loopNum);

    }


	/**
	 *
	 */   
   public void writeControlledBitSwap(		
   	int bit1, 
   	int bit2,
   	TFControls  trols
   ){	
	   	_gm.noteGBreakingOpAt(_emb.aft(bit1)); 
	   	_gm.noteGBreakingOpAt(_emb.aft(bit2));
	   	
    	//preamble, same for all 3 controlled methods
    	_lineCounter++;
		Throw.If(_englishOut==null, "writing to a closed file");
		Throw.If(_pictureOut==null, "writing to a closed file");
		
		int numBitsBef = _emb.getNumBitsBef();
		int numBitsAft = _emb.getNumBitsAft();
		//int aftTargetBitPos = _emb.aft(targetBitPosition);

        TFControls 	aftControls = trols.refreshedControls(_emb);
        aftControls.checkBitPosAreDecreasing();

        //number of controls may be zero
        int numControls = aftControls.getNumControls();
        //end of premamble
		
		Throw.If(bit1 == bit2, "swapped bits must be different");
		Throw.IfOut(bit1, numBitsBef);
		Throw.IfOut(bit2, numBitsBef);
		int big = Math.max(_emb.aft(bit1), _emb.aft(bit2));
		int small = Math.min(_emb.aft(bit1), _emb.aft(bit2));
		
		//English file
		_englishOut.print("SWAP\t" + big + "\t" + small);
		_englishOut.print(numControls!=0?"\tIF\t":"\n");
		
		//list bit-positions in decreasing order
		int c;
		for(c=0; c<numControls; c++){
			_englishOut.print(
				aftControls.getBitPos(c)  +
				(aftControls.getType(c) == true?"T":"F") + 
				(c==(numControls-1)?"\n":"\t")
				);
		}
		   
		
		//picture file
		int biggest = big;
		int smallest = small;
		if(numControls!=0){
			biggest = Math.max(
					aftControls.getBitPos(0),
					big);
			smallest = Math.min(
					aftControls.getBitPos(numControls- 1),
					small);
		}
		
		int j;//a bit position
		for(j=numBitsAft -1; j>biggest; j--){
			_pictureOut.print("|   ");               
		}
		
		int cInit = 0;
		for(j=biggest ; j>=smallest ; j--){
			boolean     isBig = (j==big);
			boolean     isSmall = (j==small);
			boolean     isControl = false;
			boolean     controlType = false;
			String      tres = (j==smallest?"   ":"---");
//			String      dos = (j==smallest?"  ":"--");
			
			//cInit starts at last value
			for(c=cInit;c<numControls;c++){
				if(j==aftControls.getBitPos(c)){
					isControl = true;
					controlType = aftControls.getType(c);
					cInit++;
					break;
				}
			}
			
			if(isControl){
				_pictureOut.print((controlType?"@":"O") + tres);
			}else{//is not control
				if(isBig){
					_pictureOut.print("<" + tres);
				}else if(isSmall){
					_pictureOut.print(">" + tres);
				}else{
					_pictureOut.print("+" + tres);
				}
			}
		}
		
		for(j=(smallest-1); j>=0; j--){
			_pictureOut.print("|   ");               
		}
		
		_pictureOut.print("\n");
		
   }
	/**
	 * ROTX  ang  
	 *    represents exp( i*(sigx)*(ang*pi/180) )
	 */
    public void writeControlledOneBitGate(
        int   targetBitPosition,
        TFControls   trols,
        OneBitGate oneBitGate
    ){
    	if(oneBitGate instanceof SigX){
    		_gm.noteXopAt(_emb.aft(targetBitPosition));
    	}else if(	oneBitGate instanceof SigY ||
    				oneBitGate instanceof SigZ ||
    				oneBitGate instanceof OneBitHadamard ||
    				oneBitGate instanceof OneBitRot)
    	{
    		_gm.noteGBreakingOpAt(_emb.aft(targetBitPosition));
    	}
    	
    	        
    	//preamble, same for all 3 controlled methods
    	_lineCounter++;
		Throw.If(_englishOut==null, "writing to a closed file");
		Throw.If(_pictureOut==null, "writing to a closed file");
		
//		int numBitsBef = _emb.getNumBitsBef();
		int numBitsAft = _emb.getNumBitsAft();
		int aftTargetBitPos = _emb.aft(targetBitPosition);

		TFControls 	aftControls = trols.refreshedControls(_emb);
		aftControls.checkBitPosAreDecreasing();

        //number of controls may be zero
        int numControls = aftControls.getNumControls();
        //end of preamble

        int c;
        for(c=0; c<numControls; c++){
            Throw.If(aftControls.getBitPos(c) == aftTargetBitPos,
              "target bit cannot be a control bit");
        }
        		
        //English file
		if(oneBitGate instanceof OneBitPhaseFactor){
			_englishOut.print("PHAS\t" + 
			((OneBitPhaseFactor)oneBitGate).getAngle()*180/Math.PI);
		}else if(oneBitGate instanceof OneBitP0PhaseFactor){
			_englishOut.print("P0PH\t" + 
			((OneBitP0PhaseFactor)oneBitGate).getAngle()*180/Math.PI);
		}else if(oneBitGate instanceof OneBitP1PhaseFactor){
			_englishOut.print("P1PH\t" + 
			((OneBitP1PhaseFactor)oneBitGate).getAngle()*180/Math.PI);
		}else if(oneBitGate instanceof SigX){
			_englishOut.print("SIGX");
		}else if(oneBitGate instanceof SigY){
			_englishOut.print("SIGY");
		}else if(oneBitGate instanceof SigZ){
			_englishOut.print("SIGZ");
		}else if(oneBitGate instanceof OneBitHadamard){
			_englishOut.print("HAD2");
		}else if(oneBitGate instanceof OneBitRot){
			switch(((OneBitRot)oneBitGate).getAxis()){
				case OneBitRot.X_AXIS:
					_englishOut.print("ROTX\t" +
					((OneBitRot)oneBitGate).getAngleX()*180/Math.PI
					);
					break;
				case OneBitRot.Y_AXIS:
					_englishOut.print("ROTY\t" +
						((OneBitRot)oneBitGate).getAngleY()*180/Math.PI
						);
					break;
				case OneBitRot.Z_AXIS:
					_englishOut.print("ROTZ\t" +
					((OneBitRot)oneBitGate).getAngleZ()*180/Math.PI
					);
					break;
				case OneBitRot.NOT_XYZ:
					 _englishOut.print("ROTN\t" +
					((OneBitRot)oneBitGate).getAngleX()*180/Math.PI
					 + "\t" +
					((OneBitRot)oneBitGate).getAngleY()*180/Math.PI
					 + "\t" +
					((OneBitRot)oneBitGate).getAngleZ()*180/Math.PI
					);
					break;                              
			}

		}else{
			throw new RuntimeException(
			"writing an unsupported" +
			(numControls==0? " OneBitGate": " ControlledGate"));
		}//end of instanceof
		
		_englishOut.print("\tAT\t" + aftTargetBitPos + 
			(numControls!=0?"\tIF\t":"\n")
			);
			
		//list bit-positions in decreasing order
		for(c=0; c<numControls; c++){
			_englishOut.print(
				aftControls.getBitPos(c)  +
				(aftControls.getType(c) == true?"T":"F") + 
				(c==(numControls-1)?"\n":"\t")
				);
		}


        //picture file
		int biggest = aftTargetBitPos;
		int smallest = aftTargetBitPos;
		if(numControls!=0){
			biggest = Math.max(aftControls.getBitPos(0),
			aftTargetBitPos);
			smallest = Math.min(aftControls.getBitPos(numControls- 1),
			aftTargetBitPos);
		}
		
		
		int j;//a bit position
		for(j=numBitsAft-1; j>biggest; j--){
			_pictureOut.print("|   ");               
		}
		
		int cInit = 0;
		for(j=biggest ; j>=smallest ; j--){
			boolean     isTarget = (j==aftTargetBitPos);
			boolean     isControl = false;
			boolean     controlType = false;
			String      tres = (j==smallest?"   ":"---");
			String      dos = (j==smallest?"  ":"--");
			
			//cInit starts at last value
			for(c=cInit;c<numControls;c++){
				if(j==aftControls.getBitPos(c)){
					isControl = true;
					controlType = aftControls.getType(c);
					cInit++;
					break;
				}
			}

				
			if(isControl){
				_pictureOut.print((controlType?"@":"O") + tres);
			}else{//is not control
				if(!isTarget){//is not control nor target
					_pictureOut.print("+" + tres);
				}else{//is target
					if(oneBitGate instanceof OneBitPhaseFactor){
						_pictureOut.print("Ph" + dos);
					}else if(oneBitGate instanceof OneBitP0PhaseFactor){
						_pictureOut.print("OP" + dos);
					}else if(oneBitGate instanceof OneBitP1PhaseFactor){
						_pictureOut.print("@P" + dos);
					}else if(oneBitGate instanceof SigX){
						_pictureOut.print("X" + tres);
					}else if(oneBitGate instanceof SigY){
						_pictureOut.print("Y" + tres);
					}else if(oneBitGate instanceof SigZ){
						_pictureOut.print("Z" + tres);
					}else if(oneBitGate instanceof OneBitHadamard){
						_pictureOut.print("H" + tres);
					}else if(oneBitGate instanceof OneBitRot){
						switch(((OneBitRot)oneBitGate).getAxis()){
							case OneBitRot.X_AXIS:
							  _pictureOut.print("Rx" + dos);
							  break;
							case OneBitRot.Y_AXIS:
							  _pictureOut.print("Ry" + dos);
							  break;
							case OneBitRot.Z_AXIS:
							  _pictureOut.print("Rz" + dos);
							  break;
							case OneBitRot.NOT_XYZ:
							  _pictureOut.print("R" + tres);
							  break;                                  
						}
					}else{
						throw new RuntimeException(
						"writing an unsupported" +
						(numControls==0? " OneBitGate": " ControlledGate"));
					}//endif
				}//endif
			}//endif
		}//end of j loop
		
		for(j=(smallest-1); j>=0; j--){
			_pictureOut.print("|   ");               
		}
		_pictureOut.print("\n");
           
   }

	/**
	 * MP_Y  ang  
	 *    represents exp( i*(sigy)*(ang*pi/180) )
	 */
    public void writeControlledMultiplexorGate(
        int   	targetBitPosition,
        MultiControls   trols,
        boolean 	withMinus,//if yes, use negative of radAngles
        DoubleArray  radAngles //all internal angles in radians
    ){ 
    	_gm.noteGBreakingOpAt(_emb.aft(targetBitPosition));
    	
    	//preamble, same for all 3 controlled methods
    	_lineCounter++;
		Throw.If(_englishOut==null, "writing to a closed file");
		Throw.If(_pictureOut==null, "writing to a closed file");
		
//		int numBitsBef = _emb.getNumBitsBef();
		int numBitsAft = _emb.getNumBitsAft();
		int aftTargetBitPos = _emb.aft(targetBitPosition);

        MultiControls 	aftControls = trols.refreshedControls(_emb);
        aftControls.checkBitPosAreDecreasing();

        //number of controls may be zero
        int numControls = aftControls.getNumControls();
        //end of preamble
        
        int c; 
        for(c=0; c<numControls; c++){
            Throw.If(aftControls.getBitPos(c) == aftTargetBitPos,
              "target bit cannot be a control bit");
        }
        
        int numNonNegControls  =  aftControls.getNumNonNegControls();
        Throw.If(numNonNegControls==0, "multiplexor with no half-moon controls");
        int numAngles  =  radAngles.length();
        Throw.If(numAngles!=(1<<numNonNegControls), 
        		"wrong number of multiplexor angles");       
       
		
        //English file
		_englishOut.print("MP_Y\tAT\t" + 
			aftTargetBitPos + 
			"\tIF\t"
			);
		//list bit-positions in decreasing order
		for(c=0; c<numControls; c++){
			String typeStr = "";
			if(aftControls.getType(c)>=0){
				typeStr = "(" + aftControls.getType(c);
			}else if(aftControls.getType(c)==MultiControls.P0_TYPE){
				typeStr = "F";
			}else if(aftControls.getType(c)==MultiControls.P1_TYPE){
				typeStr = "T";
			}
			_englishOut.print(
				aftControls.getBitPos(c)  +  typeStr + "\t");
		}
		_englishOut.print("\tBY\t");//use BY to indicate end of controls
		for(int j=0; j<numAngles; j++){
			_englishOut.print((withMinus?-1:1)*radAngles.get(j)*180/Math.PI  +
					(j==(numAngles-1)?"\n":"\t"));
		}


        //picture file
		int biggest = aftTargetBitPos;
		int smallest = aftTargetBitPos;
		if(numControls!=0){
			biggest = Math.max(aftControls.getBitPos(0),
			aftTargetBitPos);
			smallest = Math.min(aftControls.getBitPos(numControls- 1),
			aftTargetBitPos);
		}
		
		int j;//a bit position
		for(j=numBitsAft-1; j>biggest; j--){
			_pictureOut.print("|   ");               
		}
		
		int  cInit = 0;
		for(j=biggest ; j>=smallest ; j--){
			boolean     isTarget = (j==aftTargetBitPos);
			boolean     isControl = false;
			int     	controlType = 0;
			String      tres = (j==smallest?"   ":"---");
			String      dos = (j==smallest?"  ":"--");
			String      uno = (j==smallest?" ":"-");
			
			//cInit starts at last value
			for(c=cInit;c<numControls;c++){
				if(j==aftControls.getBitPos(c)){
					isControl = true;
					controlType = aftControls.getType(c);
					cInit++;
					break;
				}
			}
				
			if(isControl){
				if(controlType>=0){
					if(controlType<10){
						_pictureOut.print("(" + controlType + dos);
					}else if(controlType<100){
						_pictureOut.print("(" + controlType + uno);
					}else{
						_pictureOut.print("(" + tres);
					}
				}else if(controlType==MultiControls.P0_TYPE){
					_pictureOut.print("O" + tres);
				}else if(controlType==MultiControls.P1_TYPE){
					_pictureOut.print("@" + tres);
				}
			}else{//is not control
				if(!isTarget){//is not control nor target
					_pictureOut.print("+" + tres);
				}else{//is target
					_pictureOut.print("Ry" + dos);
				}
			}
		}
		
		for(j=(smallest-1); j>=0; j--){
			_pictureOut.print("|   ");               
		}
		_pictureOut.print("\n");
           
    }
	/**
	 *
	 */
	  public void writeBitSwap(int bit1, int bit2)
	  {
		  writeControlledBitSwap(bit1, bit2, 
			   new TFControls(_numBits, 0));	   	   
	  }

       

   /**
	*
	*/   
   public void writeOneBitGate(
      	int   targetBitPosition,
       	OneBitGate oneBitGate 
   ){
       writeControlledOneBitGate(
       targetBitPosition,
       new TFControls(_numBits, 0),
           oneBitGate);
       }
   /**
	* PHAS angle
	*		represents e(i*angle*PI/180)
	*/   
   public void writeGlobalPhaseFactor(double angle){
       writeControlledOneBitGate(
       0, // targetBitPosition, anyone will do
       new TFControls(_numBits, 0),
       new OneBitPhaseFactor(angle));
   }
	/**
	 * MP_Y  ang  
	 *    represents exp( i*(sigy)*(ang*pi/180) )
	 */
    public void writeMultiplexorGate(
        int   	targetBitPosition,
        MultiControls   controls,
        boolean		withMinus,//if yes, use negative of radAngles
        DoubleArray  radAngles  //all internal angles in radians
    ){    
    	Throw.If(controls.getNumControls() != controls.getNumNonNegControls(),
    		"some of the controls of this multiplexor are not half-moons");
    	writeControlledMultiplexorGate(targetBitPosition, controls, 
    			withMinus, radAngles);
    }
   
}
