import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

/**
 * This class defines the GUI for the main and 
 * only window (Control Panel) of QOperAv.
 */
public class QOperAvApplet extends JApplet{
	private JLabel 		_L_image;
	private JTextField	_TF_filePrefix;
	private JComboBox 	_CB_numAtomBits;
	private JComboBox 	_CB_numProbeBits;
	private JTextField	_TF_gamma;
	private JTextField	_TF_dtOver2pi;
	private JButton		_B_computeSEO;
	private JTextField	_TF_lenOfSEO;

	/**
	 * 
	 */
	public void init(){
		java.net.URL iconURL = getClass().getResource("qoper-av-logo.gif");
		_L_image = new JLabel(new ImageIcon(iconURL));
		_TF_filePrefix = new JTextField();
		_CB_numAtomBits = new JComboBox(new String[]
		    {"2", "3", "4", "5", "6", "7", "8"});
		_CB_numProbeBits = new JComboBox(new String[]
		    {"2", "3", "4", "5", "6", "7", "8"});
		_TF_gamma = new JTextField();
		_TF_dtOver2pi = new JTextField();
		_B_computeSEO = new JButton("Write Files");
		_TF_lenOfSEO = new JTextField();
		_TF_lenOfSEO.setEditable(false);
		
		JPanel panHeader = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panHeader.add(_L_image, BorderLayout.CENTER);
		panHeader.add(new JLabel(Globals.VERSION), BorderLayout.SOUTH);
		
		JPanel panIn = new JPanel(new GridLayout(5, 2));//rows, cols
		panIn.add(new JLabel("File Prefix"));
			panIn.add(_TF_filePrefix);
		panIn.add(new JLabel("Number of Atom Qubits"));
			panIn.add(_CB_numAtomBits);
		panIn.add(new JLabel("Number of Probe Qubits"));
			panIn.add(_CB_numProbeBits);
		panIn.add(new JLabel("Gamma"));
			panIn.add(_TF_gamma);
		panIn.add(new JLabel("Delta t/(2*PI)"));
			panIn.add(_TF_dtOver2pi);
		panIn.setBorder(new TitledBorder("Inputs"));
		
		JPanel panWriteFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panWriteFlow.add(_B_computeSEO);

		JPanel panInPlus = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panInPlus.add(panHeader, BorderLayout.NORTH);
		panInPlus.add(panIn, BorderLayout.CENTER);
		panInPlus.add(panWriteFlow, BorderLayout.SOUTH);
		
		JPanel panOut = new JPanel(new GridLayout(1, 2));//rows, cols
		panOut.add(new JLabel("Number of Elem. Ops."));
			panOut.add(_TF_lenOfSEO);
		panOut.setBorder(new TitledBorder("Outputs"));
		
		setLayout(new BorderLayout(2, 2));//hgap, vgap
		add(panInPlus, BorderLayout.CENTER);
		add(panOut, BorderLayout.SOUTH);
		
		_B_computeSEO.addActionListener(new ButtonListener());
	}
	
	/**
	 *
	 */
	private class ButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {

			String  filePrefix;
			int numAtomBits, numProbeBits; 
			double gamma, dtOver2pi;

			
			try{
				if(	_TF_filePrefix.getText().length()==0) 
				{throw new Exception();}
				
				filePrefix = _TF_filePrefix.getText();
				
				numAtomBits = _CB_numAtomBits.getSelectedIndex() + 2;
				numProbeBits = _CB_numProbeBits.getSelectedIndex() + 2;
				
				gamma = Double.parseDouble(_TF_gamma.getText());
				dtOver2pi = Double.parseDouble(_TF_dtOver2pi.getText());
						
				QOperAvMain 	m = new QOperAvMain(filePrefix, 
						numAtomBits, numProbeBits, gamma, dtOver2pi);
				_TF_lenOfSEO.setText(String.format("%d", m.getLenOfSEO()));
			}
			catch(Exception ex){
				_TF_lenOfSEO.setText("");
				JOptionPane.showMessageDialog(null, 
						"QOperAv got BAD INPUT:\n" + ex.toString());
				return;
			}

		}
		
	}
	/**
	 *
	 */
	public static void main(String[] args) {
	    
	    JFrame frame = new JFrame("QOperAv");

	    
	    QOperAvApplet applet = new QOperAvApplet();

	    frame.add(applet, BorderLayout.CENTER);

	    applet.init();

	    //frame.setLocationRelativeTo(null); // Center the frame
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	    
	    frame.pack();//frame.setSize(400, 250);
	    frame.setVisible(true);
	  }


}

