import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

/**
 * This class defines the GUI for the main and 
 * only window (Control Panel) of QuSAnn
 */
public class SimAnnApplet extends JApplet{
	private JLabel 		_L_image;
	private JTextField	_TF_filePrefix;
	private JComboBox 	_CB_numStateBits;
	private JComboBox 	_CB_numProbeBits;
	private JComboBox 	_CB_numPESteps;
	private JComboBox 	_CB_groverDepth;
	private JTextField 	_TF_upBdNeig;
	private JComboBox 	_CB_numBetas;
	private JTextField	_TF_deltaBeta;
	private JButton		_B_computeSEO;
	private JTextField  _TF_numBits;
	private JTextField	_TF_lenOfSEO;
//	private JTextField	_TF_error;
	/**
	 * 
	 */
	public void init(){
		java.net.URL iconURL = getClass().getResource("qsann-logo.gif");
		_L_image = new JLabel(new ImageIcon(iconURL));
		_TF_filePrefix = new JTextField();
		_CB_numStateBits = new JComboBox(new String[]
		     {"1", "2", "3", "4"});
		_CB_numProbeBits  = new JComboBox(new String[]
		     {"1", "2", "3", "4"});
		_CB_numPESteps = new JComboBox(new String[]
		     {"1", "2", "3", "4"});
		_CB_groverDepth = new JComboBox(new String[]
		     {"1", "2", "3", "4"});
		_TF_upBdNeig = new JTextField();
		_CB_numBetas = new JComboBox(new String[]
		     {"2", "3", "4", "5"});
		_TF_deltaBeta = new JTextField();
		_B_computeSEO = new JButton("Write Files");
		_TF_numBits = new JTextField();
			_TF_numBits.setEditable(false);
		_TF_lenOfSEO = new JTextField();
			_TF_lenOfSEO.setEditable(false);
//		_TF_error = new JTextField();
//			_TF_error.setEditable(false);
		
		JPanel panHeader = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panHeader.add(_L_image, BorderLayout.CENTER);
		panHeader.add(new JLabel(Globals.VERSION), BorderLayout.SOUTH);
		
		JPanel panIn = new JPanel(new GridLayout(8, 2));//rows, cols
		panIn.add(new JLabel("File Prefix"));
			panIn.add(_TF_filePrefix);
		panIn.add(new JLabel("Number Of State Bits"));
			panIn.add(_CB_numStateBits);
		panIn.add(new JLabel("Number Of Probe Bits (for each PE step)"));
			panIn.add(_CB_numProbeBits);
		panIn.add(new JLabel("Number Of Phase Estimation (PE) Steps"));
			panIn.add(_CB_numPESteps);
		panIn.add(new JLabel("Grover Depth"));
			panIn.add(_CB_groverDepth);
		panIn.add(new JLabel("Upper Bound on Number of Neighbors"));
			panIn.add(_TF_upBdNeig);
		panIn.add(new JLabel("Number of Betas"));
			panIn.add(_CB_numBetas);
		panIn.add(new JLabel("Delta Beta Per Unit Time"));
			panIn.add(_TF_deltaBeta);
		panIn.setBorder(new TitledBorder("Inputs"));
		
		JPanel panWriteFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panWriteFlow.add(_B_computeSEO);

		JPanel panInPlus = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panInPlus.add(panHeader, BorderLayout.NORTH);
		panInPlus.add(panIn, BorderLayout.CENTER);
		panInPlus.add(panWriteFlow, BorderLayout.SOUTH);
		
		JPanel panOut = new JPanel(new GridLayout(2, 2));//rows, cols
		panOut.add(new JLabel("Number of Qubits"));
			panOut.add(_TF_numBits);
		panOut.add(new JLabel("Number of Elem. Ops."));
			panOut.add(_TF_lenOfSEO);
//		panOut.add(new JLabel("Error"));
//			panOut.add(_TF_error);
		panOut.setBorder(new TitledBorder("Outputs"));
		
		setLayout(new BorderLayout(2, 2));//hgap, vgap
		add(panInPlus, BorderLayout.CENTER);
		add(panOut, BorderLayout.SOUTH);
		
		_B_computeSEO.addActionListener(new ButtonListener());
	}
	
	/**
	 *
	 */
	private class ButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {

			String  filePrefix;
			int numStateBits;
			int numProbeBits;
			int numPESteps;	
			int groverDepth;
			double upBdNeig;		
						
			int numBetas;
			double deltaBeta;
				
			try{
				if(	_TF_filePrefix.getText().length()==0 ||
					_TF_upBdNeig.getText().length()==0 ||
					_TF_deltaBeta.getText().length()==0){
					throw new Exception();}
				
				filePrefix = _TF_filePrefix.getText();

				numStateBits = _CB_numStateBits.getSelectedIndex() + 1;
				numProbeBits = _CB_numProbeBits.getSelectedIndex() + 1;
				numPESteps = _CB_numPESteps.getSelectedIndex() + 1;
				groverDepth = _CB_groverDepth.getSelectedIndex() + 1;
				
				upBdNeig =  Double.parseDouble(_TF_upBdNeig.getText());
				
				numBetas =  _CB_numBetas.getSelectedIndex() + 2;
				deltaBeta = Double.parseDouble(_TF_deltaBeta.getText());
								
				SimAnnMain  m  = new SimAnnMain(
		    			filePrefix,
		    			numStateBits,
		    			numProbeBits,
		    			numPESteps,
		    			groverDepth,
		    			upBdNeig,
		    			numBetas,
		    			deltaBeta);
				int numBits = 2*numStateBits + numProbeBits*numPESteps;
				_TF_numBits.setText(String.format("%d", numBits));
				_TF_lenOfSEO.setText(String.format("%d", m.getLenOfSEO()));
//				_TF_error.setText(String.format("%e", m.getError()));
			}
			catch(Exception ex){
				_TF_numBits.setText("");
				_TF_lenOfSEO.setText("");
	//			_TF_error.setText("");
				JOptionPane.showMessageDialog(null, 
						"QuSAnn got BAD INPUT:\n" + ex.toString());
				return;
			}
		}

	}
	/**
	 *
	 */
	public static void main(String[] args) {
	    
	    JFrame frame = new JFrame("QuSAnn");

	    
	    SimAnnApplet applet = new SimAnnApplet();

	    frame.add(applet, BorderLayout.CENTER);

	    applet.init();

	    //frame.setLocationRelativeTo(null); // Center the frame
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	    
	    frame.pack();//frame.setSize(400, 250);
	    frame.setVisible(true);
	  }
}


