import java.io.*;


/**
 * This class writes English and Picture files
 * for a SINGLE phase estimation step
 * of the quantum simulated annealing algorithm.
 */
public class SimAnnPhaseEstSEO_writer extends PhaseEstSEO_writer{
	private ProbMatQEmb   _probQEmb;
	
	/**
	 *
	 */
	public SimAnnPhaseEstSEO_writer(
	    String 	filePrefix,
	    int numBits,
		int numProbeBits,
		ProbMatQEmb  probQEmb)
	{
		super(filePrefix, numBits);
		_numAtomBits = _numBits - numProbeBits;
		_numProbeBits = numProbeBits;
		_ending = PhaseEstEnding.HADS;
		_probQEmb = probQEmb;
		
	    write();
	    
	    closeFiles();       
	}
	/**
	 * 
	 */
    public SimAnnPhaseEstSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		boolean doWrite,
		CktEmbedder emb,
		int numProbeBits,
		ProbMatQEmb  probQEmb)
    {
    	super(englishOut, pictureOut, emb);

		_numAtomBits = _numBits - numProbeBits;
		_numProbeBits = numProbeBits;
		_ending = PhaseEstEnding.HADS;
		_probQEmb = probQEmb;
 	
    	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */	 
    public void  writeAtom(){
		new ProbMatSzegedySEO_writer(
			_englishOut,
			_pictureOut,
			false,// doWrite
			_emb,
	   	    _probQEmb
	        ).write();
	}
    /**
	 * 
	 */	 
    public void  writeAtomHermitian(){
		new ProbMatSzegedySEO_writer(
			_englishOut,
			_pictureOut,
			false,// doWrite
			_emb,
	   	    _probQEmb
	        ).writeHermitian();
	}
    /**
	 * 
	 */
	public static void main(String[] args){
		String  filePrefix = "pEst";
		int numStateBits = 1;
		int numAtomBits = 2*numStateBits;
		int numProbeBits = 3;
		int numBits = numAtomBits + numProbeBits;
		int numStates = 1<<numStateBits;
		SimAnnPhaseEstSEO_writer w =new SimAnnPhaseEstSEO_writer(
	        openEngFile(filePrefix),
	        openPicFile(filePrefix),
	        true,//doWrite
	        new CktEmbedder(numBits),//emb
			numProbeBits,
			new ProbMatQEmb(numStateBits, 
					ProbMatrices.randomProbMatrix(numStates, numStates)));
		w.writeHermitian();
		w.closeFiles();

	}

}


