/**
 * This class writes English and Picture files
 * for the full algorithm of Wocjan and Abeyesinghe
 * (see ArXiv:0804.4259)
 * for doing quantum simulated annealing.
 */

public class SimAnnSEO_writer  extends SEO_writer{
	private int  _numAtomBits;//Atom is general unitary transformation
	private int  _numProbeBits;//from Atom^1 to Atom^pow(2,numProbeBits-1)
	private int  _numPESteps;//Phase Estimation steps in stairs
	private int  _groverDepth;
	private MinimizationProblem 	_p;
	private double[] 	_betas;//beta(inverse temperature). Index is time

	
	ProbMatQEmb  _probQEmbBef;//before
	ProbMatQEmb  _probQEmbAft;//after


	/**
	 * 
	 */	
	public SimAnnSEO_writer(
	        String 	filePrefix,
	    	int 	numStateBits,
	    	int 	numProbeBits,
	    	int  	numPESteps,
	    	int		groverDepth,
	    	MinimizationProblem 	p,
	    	double[] 	betas)
	{ 	
	 	
    	super(filePrefix, 2/*dummy _numBits*/);
		_numAtomBits = 2*numStateBits;
		_numProbeBits = numProbeBits;
		_numPESteps = numPESteps;
		//must reassign these variables because super constructor 
		//used  numBits=2
		_numBits = _numAtomBits + numProbeBits*numPESteps;
        _emb = new CktEmbedder(_numBits);
		
		
		_groverDepth = groverDepth;
		_p = p;
		_betas = betas;


		write();
		
        closeFiles();

	}
	/**
	 * 
	 */	
	private void write(){
		int betaLen = _betas.length;
		int numColBits = _numAtomBits/2;
		GroWriter1 gw = new GroWriter1();
		_probQEmbAft = new ProbMatQEmb(numColBits,
				ProbMatrices.metropolisProbMatrix(_p, _betas[0]));
		int time=0;
		while(time<betaLen-1){
			_probQEmbBef = _probQEmbAft;
			_probQEmbAft = new ProbMatQEmb(numColBits, 
					ProbMatrices.metropolisProbMatrix(_p, _betas[time+1]));
			gw.write(_groverDepth);
			time++;
		}
	}
		
	/**
	 * 
	 */	
	
	
    private class GroWriter1 extends GroverPiOver3Writer{
	    /**
	     * 
	     */
    	public void writeMat1(){
    		SimAnnStairsSEO_writer 	w = new SimAnnStairsSEO_writer(
		        _englishOut,
		        _pictureOut,
		        false,//doWrite
		        _emb,
				_numProbeBits,
				_probQEmbBef,
				_numPESteps);
    		w.write();
    		writeControlledPhase(false);
    		w.writeHermitian();
    	}
    	/**
	     * 
	     */                        
    	public void writeMat1Hermitian(){
    		SimAnnStairsSEO_writer 	w = new SimAnnStairsSEO_writer(
    		        _englishOut,
    		        _pictureOut,
    		        false,//doWrite
    		        _emb,
    				_numProbeBits,
    				_probQEmbBef,
    				_numPESteps);
        		w.write();
        		writeControlledPhase(true);
        		w.writeHermitian();
        	}
    	/**
	     * 
	     */                      
    	public void writeMat2(){
    		SimAnnStairsSEO_writer 	w = new SimAnnStairsSEO_writer(
		        _englishOut,
		        _pictureOut,
		        false,//doWrite
		        _emb,
				_numProbeBits,
				_probQEmbAft,
				_numPESteps);
    		w.write();
    		writeControlledPhase(false);
    		w.writeHermitian();
    		
    	}
    	/**
	     * 
	     */                      
    	public void writeMat2Hermitian(){
    		SimAnnStairsSEO_writer 	w = new SimAnnStairsSEO_writer(
    		        _englishOut,
    		        _pictureOut,
    		        false,//doWrite
    		        _emb,
    				_numProbeBits,
    				_probQEmbAft,
    				_numPESteps);
        		w.write();
        		writeControlledPhase(true);
        		w.writeHermitian();  		
    	}
    	/**
    	 * 
    	 */	
    	private void writeControlledPhase(boolean  wantHermitian){
    		//  (omega)^( I_atomBits (x) |0><0|_probeBits)
    		int  numNonPhaseControls = _numPESteps*_numProbeBits-1;
    		//one bit has controlled phase
    		TFControls    controls =  
    			new TFControls(_numBits, numNonPhaseControls);
    		int bitPos = numNonPhaseControls;
    		for(int c=0; c<numNonPhaseControls; c++){
    			bitPos--;
    			controls.setControl(c, bitPos, false);
    		}
    		writeControlledOneBitGate(
    			numNonPhaseControls,//targetBitPosition
		        controls,
		        new OneBitP0PhaseFactor(wantHermitian?-PI_OVER3:PI_OVER3));
    	}
    }
}

