import java.io.*;

/**
 * This class writes English and Picture files
 * for MULTIPLE phase estimation steps (bunch of steps = stairs) 
 * of the quantum simulated annealing algorithm.
 */
public class SimAnnStairsSEO_writer extends PhaseEstStairsSEO_writer{
	/**
	 *
	 */
	public SimAnnStairsSEO_writer(
	    String 	filePrefix,
	    int numBits,
		int numProbeBits,
		ProbMatQEmb  probQEmb,
		int numPESteps)
	{
		super(	filePrefix,
				numBits,
				numProbeBits,
				(ProbMatQEmb) probQEmb,
				numPESteps);
 
	}
	/**
	 * 
	 */
    public SimAnnStairsSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		boolean doWrite,
		CktEmbedder emb,
		int numProbeBits,
		ProbMatQEmb  probQEmb,
		int numPESteps)
    {
    	super(	englishOut,
    			pictureOut,
    			doWrite,
    			emb,
    			numProbeBits,
    			(ProbMatQEmb) probQEmb,
    			numPESteps);

    }
	/**
	  * 
	  */
	protected void writePhaseEst(CktEmbedder emb){
		new SimAnnPhaseEstSEO_writer(
			_englishOut,
	        _pictureOut,
	        false,//doWrite
	        emb.getCompositeMapOfYouFollowedBy(_emb),
			_numProbeBits,
			(ProbMatQEmb)_probQEmb).write();
	}
	/**
	 * 
	 */
	protected void writePhaseEstHermitian(CktEmbedder emb){
		new SimAnnPhaseEstSEO_writer(
			_englishOut,
	        _pictureOut,
	        false,//doWrite
	        emb.getCompositeMapOfYouFollowedBy(_emb),
			_numProbeBits,
			(ProbMatQEmb)_probQEmb).writeHermitian();
	}

    /**
	 * 
	 */
	public static void main(String[] args){
		String  filePrefix = "stairs";
		int numStateBits = 1;
		int numAtomBits = 2*numStateBits;
		int numProbeBits = 3;
		int numPESteps = 3;
		int numBits = numAtomBits + numProbeBits*numPESteps;
		int numStates = 1<<numStateBits;
		SimAnnStairsSEO_writer w = new SimAnnStairsSEO_writer(
	        openEngFile(filePrefix),
	        openPicFile(filePrefix),
	        true,//doWrite
	        new CktEmbedder(numBits),//emb
			numProbeBits,
			new ProbMatQEmb(numStateBits, 
					ProbMatrices.randomProbMatrix(numStates, numStates)),
			numPESteps);
		w.writeHermitian();
		w.closeFiles();

	}

}


