import java.io.*;

/**
 * This class's main() method is the applet sans gui.
 * Fou = Fourier
 */
public class FouMain {
	private int		_lenOfSEO;
	private double 	_error;

	/**
	 *
	 */
	public FouMain(
		String 	filePrefix,
		int numBits)
	{
		String filePrefixPlus = filePrefix + "_qfou";
		int numRows = 1<<numBits;

    	new FouSEO_writer(
    		filePrefixPlus, 
            numBits);

    	
    	Complex[][] dftMat = new Complex[numRows][numRows];    	
    	int row, col;
    	double angle;
    	for(row=0; row<numRows; row++){
    		for(col=0; col<numRows; col++){
    			angle = (2*Math.PI*row*col)/numRows;
    			dftMat[row][col] = 
    				new Complex(Math.cos(angle)/Math.sqrt(numRows), 
    						Math.sin(angle)/Math.sqrt(numRows));
    			//System.out.println("row=" + row + " ,col=" + col +
    			//		", z = " + dftMat[row][col].toString());
    		}
    	}
    	
    	SEO_readerMu	reader = new SEO_readerMu(
    	    filePrefixPlus,
    	    numBits, 
    	    false); // useSparse
    	_lenOfSEO = reader.getLenOfSEO();
    	//System.out.println(reader);
    	
    	_error = 0.0;
    	for(row=0; row<numRows; row++){
    		for(col=0; col<numRows; col++){
    			_error += Complex.abs(Complex.minus(
    					dftMat[row][col], 
    					reader.getEntry(row,col)
    					));
    		}
    	}
    	
    	PrintWriter   logOut = null;
    	try{
	        File  logFile = new File(filePrefixPlus + "_log.txt");
	        logOut = new PrintWriter(logFile);
    	}
    	catch(FileNotFoundException ex){
    		System.out.println("\n" + ex.toString());
        }
    	logOut.println("Inputs:");
        logOut.println("File Prefix = " + filePrefix);
        logOut.println("Number of Bits = " + numBits);
        
        logOut.println("");
        logOut.println("Outputs:");
        logOut.println("Number of Elem. Ops. = " + _lenOfSEO);
        logOut.printf("Error = %e\n", _error);
        logOut.close();
   	
	}
	/**
	 *
	 */
	public int getLenOfSEO() {
		return _lenOfSEO;
	}
	/**
	 *
	 */
	public double getError(){
		return _error;
	}
	/**
	 *
	 */
    public static void main(String[] args){
    	
		int numBits = 4;
    	String 	filePrefix = "test";
    	
    	FouMain   m =  new FouMain(filePrefix, numBits);
    	
    	
/*    	
    	System.out.println("expiMat");
    	System.out.println(expiMat.toString());

    	System.out.println("SEO product");
    	System.out.println(reader);
*/
    	System.out.println("error is  " + m._error);
   	

    }
}
