import java.io.*;

/**
 * This class writes English and Picture files
 * for the exact compilation, discovered by Coppersmith,
 * of the Discrete Fourier Transform.
 * We follow the conventions of  
 * quant-ph 0407215, "QC Paulinesia" by R.R. Tucci.
 *
 */


public class FouSEO_writer  extends SEO_writer{


/**
 * 
 */	
	public FouSEO_writer(
	        String 	filePrefix,
	        int numBits)
	{ 	
	 	
    	super(filePrefix, numBits);
		
    		write();

        closeFiles();

	}
/**
 * 
 */
    public FouSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb)
    {
    	super(englishOut, pictureOut, emb);
    	
    	if(doWrite){write();}
    }
    
/**
 * 
 */
	public void write(){
		int j, k;
		
		//permutation R
		for(j = _numBits - 1; j>=1; j--){
			for(k = j-1; k>=0; k--){
				writeBitSwap(j,k);
			}
		}


		TFControls controls = new TFControls(_numBits, 1);		
		for(k=0; k<_numBits; k++){
			writeOneBitGate(k, new OneBitHadamard());
			controls.setControl(0, k, true);
			for(j=k+1; j<_numBits; j++){
				//note j>k so abs(j-k) = j-k
		        writeControlledOneBitGate(
					j, //targetBitPosition, 
					controls, 
					new OneBitP1PhaseFactor(
					Math.PI/(1<<(j-k))));
			}
		}
				
		
	}
/**
 * writes Hermitian Conjugate
 */
	public void writeHermitian(){
	
		int j, k;
		
		//must invert this
		//for(k=0; k<_numBits; k++){
		//for(j=k+1; j<_numBits; j++){
		TFControls controls = new TFControls(_numBits, 1);
		for(k=_numBits-1; k>=0; k--){
			controls.setControl(0, k, true);
			for(j=_numBits-1; j>=k+1; j--){
				//note j>k so abs(j-k) = j-k
		        writeControlledOneBitGate(
					j, //targetBitPosition, 
					controls, 
					new OneBitP1PhaseFactor(
					-Math.PI/(1<<(j-k))));
			}
			writeOneBitGate(k, new OneBitHadamard());
		}

		//must invert this:
		//permutation R
		//for(j = _numBits - 1; j>=1; j--){
		//for(k = j-1; k>=0; k--){

		for(j = 1; j<_numBits; j++){
			for(k = 0; k<j; k++){
				writeBitSwap(j,k);
			}
		}

	}
/**
 * 
 */
    public static void main(String[] args){
    	
 		int numBitsBef = 4;
 		int numBitsAft = 6;
    	CktEmbedder  emb = new CktEmbedder(numBitsBef, numBitsAft,
    		 Globals.identityPermutation(numBitsBef),
    		 Globals.F_vec(numBitsAft),
    		 Globals.F_vec(numBitsAft));
    	
    	String filePrefix = "test";
    	
		FouSEO_writer  w = new FouSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			emb);
    	w.writeHermitian();
    	w.closeFiles();
    }

}
