import java.io.*;

/**
 * This class's main() method is the applet sans gui.
 */
public class GlueMain {
	private int		_lenOfSEO;
	private double 	_error;

	/**
	 *
	 */
	public GlueMain(
		String 	filePrefix,
	    int numBits,  
		double 	strength,
		IntPair 	pair)
	{
		String filePrefixPlus = filePrefix + "_qglue";
		int numRows = 1<<numBits;
		
    	new GlueSEO_writer(
    		filePrefixPlus,
            numBits, 
        	strength,
        	pair);

    	
    	GlueAdjMat 	incMat = new GlueAdjMat(numBits, pair);
    	double[][] realSymMat = new double[numRows][numRows];    	
    	int row, col;
    	for(row=0; row<numRows; row++){
    		for(col=0; col<numRows; col++){
    			realSymMat[row][col] = strength*incMat.getEntry(row,col);
    		}
    	}
    	ExpiRealSym  expiMat = new ExpiRealSym(realSymMat);
    	
    	SEO_readerMu	reader = new SEO_readerMu(
    	    filePrefixPlus,
    	    numBits, 
    	    false); // useSparse
    	_lenOfSEO = reader.getLenOfSEO();
    	
    	_error = 0.0;
    	for(row=0; row<numRows; row++){
    		for(col=0; col<numRows; col++){
    			_error += Complex.abs(Complex.minus(
    					expiMat.getEntry(row,col), 
    					reader.getEntry(row,col)
    					));
    		}
    	}
    	
    	PrintWriter   logOut = null;
    	try{
	        File  logFile = new File(filePrefixPlus + "_log.txt");
	        logOut = new PrintWriter(logFile);
    	}
    	catch(FileNotFoundException ex){
    		System.out.println("\n" + ex.toString());
        }
    	logOut.println("Inputs:");
        logOut.println("File Prefix = " + filePrefix);
        logOut.println("Number of Bits = " + numBits);
        logOut.println("Coupling Constant = " + strength);
        logOut.println("Pair = " + pair.toString());
        
        logOut.println("");
        logOut.println("Outputs:");
        logOut.println("Number of Elem. Ops. = " + _lenOfSEO);
        logOut.printf("Error = %e\n", _error);
        logOut.close();
   	
	}
	/**
	 *
	 */
	public int getLenOfSEO() {
		return _lenOfSEO;
	}
	/**
	 *
	 */
	public double getError(){
		return _error;
	}
	/**
	 *
	 */
    public static void main(String[] args){
    	
		int numBits = 4;
    	double 	strength = .6;//1/Math.sqrt(2);
		IntPair pair = new IntPair( 3, 12);
    	String 	filePrefix = "test";
    	
    	GlueMain m = new GlueMain(filePrefix, numBits,
    			strength, pair);
    	
    	
/*    	
    	System.out.println("expiMat");
    	System.out.println(expiMat.toString());

    	System.out.println("SEO product");
    	System.out.println(reader);
*/
    	System.out.println("error is  " + m._error);
   	

    }
}
