import java.io.*;

/**
 * This class's main() method is the applet sans gui.
 */
public class OracleMain {
	private int		_lenOfSEO;
	private double 	_error;

	/**
	 *
	 */
	public OracleMain(
        String 	filePrefix,
        int numBits,  
        double strength,
		OracleBands bands )
	{
		String filePrefixPlus = filePrefix + "_qorac";
		
    	new OracleSEO_writer(
    		filePrefixPlus,
            numBits, 
        	strength,
        	bands);

    	
    	OracleAdjMat 	incMat = new OracleAdjMat(numBits, bands);
    	int dim = 1<<numBits;
    	double[][] realSymMat = new double[dim][dim];    	
    	int row, col;
    	for(row=0; row<dim; row++){
    		for(col=0; col<dim; col++){
    			realSymMat[row][col] = strength*incMat.getEntry(row,col);
    		}
    	}
    	ExpiRealSym  expiMat = new ExpiRealSym(realSymMat);
    	
    	SEO_readerMu	reader = new SEO_readerMu(
    	    filePrefixPlus,
    	    numBits, 
    	    false); // useSparse
    	_lenOfSEO = reader.getLenOfSEO();
    	
    	_error = 0.0;
    	for(row=0; row<dim; row++){
    		for(col=0; col<dim; col++){
    			_error += Complex.abs(Complex.minus(
    					expiMat.getEntry(row,col), 
    					reader.getEntry(row,col)
    					));
    		}
    	}
    	
    	PrintWriter   logOut = null;
    	try{
	        File  logFile = new File(filePrefixPlus + "_log.txt");
	        logOut = new PrintWriter(logFile);
    	}
    	catch(FileNotFoundException ex){
    		System.out.println("\n" + ex.toString());
        }
    	logOut.println("Inputs:");
        logOut.println("File Prefix = " + filePrefix);
        logOut.println("Number of Bits = " + numBits);
        logOut.println("Coupling Constant = " + strength);
        logOut.println(bands.toString());

        logOut.println("");
        logOut.println("Outputs:");
        logOut.println("Number of Elem. Ops. = " + _lenOfSEO);
        logOut.printf("Error = %e\n", _error);
        logOut.close();
   	
	}
	/**
	 *
	 */
	public int getLenOfSEO() {
		return _lenOfSEO;
	}
	/**
	 *
	 */
	public double getError(){
		return _error;
	}
	/**
	 *
	 */
    public static void main(String[] args){
    	
		int numBits = 4;
    	double 	strength = .6;//1/Math.sqrt(2);
		OracleBands bands = new OracleBands( 3,
				new int[][] {{0,0}, {3,4}, {6,7}});
    	String 	filePrefix = "test";
    	
    	OracleMain m = new OracleMain(filePrefix, numBits,
    			strength, bands );
    	
    	
/*    	
    	System.out.println("expiMat");
    	System.out.println(expiMat.toString());

    	System.out.println("SEO product");
    	System.out.println(reader);
*/
    	System.out.println("error is  " + m._error);
   	

    }
}
