import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.TitledBorder;

/**
 * This class defines the GUI for the main and 
 * only window (Control Panel) of QuanTree.
 */
public class TreeApplet extends JApplet{
	private JLabel 		_L_image;
	private JTextField	_TF_filePrefix;
	private JComboBox 	_CB_numBits;
	private JTextField	_TF_strength;
	private JComboBox 	_CB_numTrots;
	private JButton		_B_computeSEO;
	private JTextField	_TF_lenOfSEO;
	private JTextField	_TF_error;
	/**
	 *
	 */
	public void init(){
		java.net.URL iconURL = getClass().getResource("qtree-logo.gif");
		_L_image = new JLabel(new ImageIcon(iconURL));
		_TF_filePrefix = new JTextField();
		_CB_numBits = new JComboBox(new String[]
		    {"2", "3", "4", "5", "6", "7", "8"});
		_TF_strength = new JTextField();
		_CB_numTrots = new JComboBox(new String[]
		    {"1", "2", "3", "4", "5"});
		_B_computeSEO = new JButton("Write Files");
		_TF_lenOfSEO = new JTextField();
		_TF_lenOfSEO.setEditable(false);
		_TF_error = new JTextField();
		_TF_error.setEditable(false);
		
		JPanel panHeader = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panHeader.add(_L_image, BorderLayout.CENTER);
		panHeader.add(new JLabel(Globals.VERSION), BorderLayout.SOUTH);
		
		JPanel panIn = new JPanel(new GridLayout(5, 2));//rows, cols
		panIn.add(new JLabel("File Prefix"));
			panIn.add(_TF_filePrefix);
		panIn.add(new JLabel("Number of Qubits"));
			panIn.add(_CB_numBits);
		panIn.add(new JLabel("Coupling Constant"));
			panIn.add(_TF_strength);
		panIn.add(new JLabel("Number of Trots"));
			panIn.add(_CB_numTrots);
		panIn.add(new JLabel("Order of Approx."));
			panIn.add(new JLabel("3"));
		panIn.setBorder(new TitledBorder("Inputs"));
		
		JPanel panWriteFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panWriteFlow.add(_B_computeSEO);

		JPanel panInPlus = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panInPlus.add(panHeader, BorderLayout.NORTH);
		panInPlus.add(panIn, BorderLayout.CENTER);
		panInPlus.add(panWriteFlow, BorderLayout.SOUTH);
		
		JPanel panOut = new JPanel(new GridLayout(2, 2));//rows, cols
		panOut.add(new JLabel("Number of Elem. Ops."));
			panOut.add(_TF_lenOfSEO);
		panOut.add(new JLabel("Error"));
			panOut.add(_TF_error);
		panOut.setBorder(new TitledBorder("Outputs"));
		
		setLayout(new BorderLayout(2, 2));//hgap, vgap
		add(panInPlus, BorderLayout.CENTER);
		add(panOut, BorderLayout.SOUTH);
		
		_B_computeSEO.addActionListener(new ButtonListener());
	}
	/**
	 *
	 */
	private class ButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {

			String  filePrefix;
			int numBits; 
			double strength;
			int numTrots;
			
			try{
				if(	_TF_filePrefix.getText().length()==0 ||
					_TF_strength.getText().length()==0)
				{ throw new Exception();}
				
				filePrefix = _TF_filePrefix.getText();

				numBits = _CB_numBits.getSelectedIndex() + 2;
				
				strength = Double.parseDouble(_TF_strength.getText());
				
				numTrots = _CB_numTrots.getSelectedIndex() + 1;
				
				TreeMain m = new TreeMain(filePrefix, numBits, strength, numTrots);
				_TF_lenOfSEO.setText(String.format("%d", m.getLenOfSEO()));
				_TF_error.setText(String.format("%e", m.getError()));
				
			}
			catch(Exception ex){
				_TF_lenOfSEO.setText("");
				_TF_error.setText("");
				JOptionPane.showMessageDialog(null, 
						"QuanTree got BAD INPUT:\n" + ex.toString());
				return;
			}
			
			

		}
		
	}
	/**
	 *
	 */
	public static void main(String[] args) {
	    
	    JFrame frame = new JFrame("QuanTree");

	    
	    TreeApplet applet = new TreeApplet();

	    frame.add(applet, BorderLayout.CENTER);

	    applet.init();

	    //frame.setLocationRelativeTo(null); // Center the frame
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	    
	    frame.pack();//frame.setSize(400, 250);
	    frame.setVisible(true);
	  }


}
