import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


/**
 * This class defines the GUI for the main and 
 * only window (Control Panel) of Quibbs.
 * 
 * Dec 9, 2013: Eclipse sdk70 bug. 
 * printf() invoked from applet puts files in Quibbs/bin/Asia,
 * whereas if use printf() in application puts files in Quibbs/Asia,
 * assuming one uses Asia as the I/O folder 
 */
public class QuibbsApplet extends JApplet{
	private JTextField	_TF_folder;
	private JButton	_B_folder_info;

	private JButton		_B_readCFiles;
	private JButton		_B_doPrerun;
	private JButton	_B_prerun_info;
	private JList 		_L_startingState;
	private JComboBox	_CB_nodeStates;
	private JButton		_B_randomStartingState;
	private JTextField	_TF_numProbeBits;
	private JTextField	_TF_numPESteps;
	private JTextField	_TF_maxNumGroverSteps;
	private JTextField	_TF_gamTol;
	private JTextField 	_TF_deltaLam;

	
	private JCheckBox	_CB_omitStairs;
	private JButton		_B_computeSEO;
	private JButton		_B_computeSEO_info;
	
	private JTextField	_TF_probStartingState;
	private JTextField	_TF_gam0;
	private JTextField	_TF_finGamma;
	private JTextField	_TF_numGroverSteps;
	private JTextField  _TF_numBits;
	private JTextField	_TF_lenOfSEO;
//	private JTextField	_TF_error;
		
	private CB_Net _net;
	private String _relPath;
	private int[]  _startingNodeStates;
	private double _prob0;
	private double _gam0;
	private SS_ListSelectionListener _L_listener;
	private NodeStatesComboListener _CB_listener;
	
	/**
	 * 
	 */
	public void init(){
		
		_TF_folder = new JTextField();
		
		_B_folder_info = new JButton("?");
		_B_folder_info.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String myInfo= 
						"The \"IO Folder\" is the name of an already existing folder\n"
						+ "which is in same directory as the Applet.\n"
						+ "Example: try the folder \"Asia\", without the apostrophees.\n"
						+ "Note there are no forward slash or backslash at end of this entry.\n"
						+ "If need to include slashes in between "
						+ "folder names to indicate directory\n"
						+ "hierarchy, use slashes / as in Unix, "
						+ "not backslashes \\ as in Windows.";
				JOptionPane.showMessageDialog(null,
						myInfo,"Tips", JOptionPane.INFORMATION_MESSAGE);				
			}
		});

		_B_readCFiles = new JButton("Read Bayesian Net");
		_B_doPrerun = new JButton("Do a Pre-run");
		_B_prerun_info = new JButton("?");
		_B_prerun_info.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String myInfo= 
						  "The \"Do a Pre-run\" button:\n"
						+ "(1) reads files states.txt, parents.txt.\n"
						+ "(2) writes files probsT.txt, probsF.txt, blankets.txt, nits.txt.\n\n"
						+ "The \"Read Bayesian Net\" button:\n"
						+ "(1) reads files states.txt, parents.txt, probs.txt.";
						
				JOptionPane.showMessageDialog(null,
						myInfo,"Tips", JOptionPane.INFORMATION_MESSAGE);				
			}
		});
		
		_L_startingState = new JList();
			_L_startingState.setSelectionMode(
					ListSelectionModel.SINGLE_SELECTION);
			_L_startingState.setVisibleRowCount(3);
		_CB_nodeStates = new JComboBox();
		_B_randomStartingState = new JButton("Random Start");
		_TF_numProbeBits  = new JTextField();
		_TF_numPESteps = new JTextField();
		_TF_maxNumGroverSteps = new JTextField();
		_TF_gamTol = new JTextField();
		_TF_deltaLam = new JTextField();
		
		_CB_omitStairs = new JCheckBox("Omit V Gates (diagnostic)");
		_B_computeSEO = new JButton("Write Q.Circuit Files");
		_B_computeSEO_info = new JButton("?");
		_B_computeSEO_info.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String myInfo= 
						  "The \"Write Q.Circuit Files\" button "
						+ "writes the following quantum circuit files:\n" +
						"quibbs_log.txt, quibbs_eng.txt, quibbs_pic.txt";						
				JOptionPane.showMessageDialog(null,
						myInfo,"Tips", JOptionPane.INFORMATION_MESSAGE);				
			}
		});

		_TF_probStartingState = new JTextField();
			_TF_probStartingState.setEditable(false);
		_TF_gam0 = new JTextField();
			_TF_gam0.setEditable(false);
		_TF_finGamma = new JTextField();
			_TF_finGamma.setEditable(false);
		_TF_numGroverSteps = new JTextField();
			_TF_numGroverSteps.setEditable(false);			
		_TF_numBits = new JTextField();
			_TF_numBits.setEditable(false);
		_TF_lenOfSEO = new JTextField();
			_TF_lenOfSEO.setEditable(false);
//		_TF_error = new JTextField();
//			_TF_error.setEditable(false);
		LineBorder borderStyle = new LineBorder(Color.black, 2);
		//	BevelBorder border = new BevelBorder(BevelBorder.RAISED);
		//	EtchedBorder border = new EtchedBorder();
		
		JPanel panHeader = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		java.net.URL iconURL = getClass().getResource("quibbs-logo.gif");
		JLabel L_image = new JLabel(new ImageIcon(iconURL));
		panHeader.add(L_image, BorderLayout.CENTER);
		panHeader.add(new JLabel(Globals.VERSION), BorderLayout.SOUTH);

		JPanel panPrelimButs = new JPanel(new FlowLayout(FlowLayout.RIGHT, 20, 2));
		panPrelimButs.add(_B_doPrerun);
		panPrelimButs.add(_B_readCFiles);
		panPrelimButs.add(_B_prerun_info);

		JPanel panPrelim = new JPanel(new BorderLayout(2, 2));//hgap, vgap
		panPrelim.add(new JLabel("I/O Folder"), BorderLayout.WEST);
		panPrelim.add(_TF_folder, BorderLayout.CENTER );
		panPrelim.add(_B_folder_info, BorderLayout.EAST );
		panPrelim.add(panPrelimButs, BorderLayout.SOUTH);
						
		JPanel panSSButs = new JPanel(new FlowLayout(FlowLayout.RIGHT, 20, 2));
		panSSButs.add(_CB_nodeStates);
		panSSButs.add(_B_randomStartingState);

		JPanel panSS = 
			new JPanel(new BorderLayout(2, 2));//hgap, vgap	
		JScrollPane  panScroll = new JScrollPane(
				_L_startingState,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		panSS.add(panScroll,  BorderLayout.NORTH);
		panSS.add(panSSButs, BorderLayout.CENTER);
		panSS.setBorder(new TitledBorder("Starting State"));

		JPanel panInGrid = new JPanel(new GridLayout(5, 2));//rows, cols
		panInGrid.add(new JLabel("Number Of Probe Bits (for each PE step)"));
			panInGrid.add(_TF_numProbeBits);
		panInGrid.add(new JLabel("Number Of Phase Estimation (PE) Steps"));
			panInGrid.add(_TF_numPESteps);		
		panInGrid.add(new JLabel("Maximum Number of Grover Steps"));
			panInGrid.add(_TF_maxNumGroverSteps);
		panInGrid.add(new JLabel("Gamma Tolerance (degs)"));
			panInGrid.add(_TF_gamTol);		
		panInGrid.add(new JLabel("Delta Lambda (degs)"));
			panInGrid.add(_TF_deltaLam);

		JPanel panIn = new JPanel(new BorderLayout(2,2));//hgap, vgap;
		panIn.add(panPrelim, BorderLayout.NORTH);
		panIn.add(panSS, BorderLayout.CENTER);
		panIn.add(panInGrid, BorderLayout.SOUTH);
		panIn.setBorder(new TitledBorder(borderStyle,"Inputs"));
				
		
		JPanel panMid = new JPanel(new FlowLayout(FlowLayout.RIGHT, 20, 2));
		panMid.add(_CB_omitStairs);
		panMid.add(_B_computeSEO);
		panMid.add(_B_computeSEO_info);
	
		JPanel panOut = new JPanel(new GridLayout(6, 2));//rows, cols
		panOut.add(new JLabel("Prob. of Starting State"));
			panOut.add(_TF_probStartingState);
		panOut.add(new JLabel("Starting Gamma (degs)"));
			panOut.add(_TF_gam0);
		panOut.add(new JLabel("Final Gamma (degs)"));
			panOut.add(_TF_finGamma);
		panOut.add(new JLabel("Number of Grover Steps "));
			panOut.add(_TF_numGroverSteps);
		panOut.add(new JLabel("Number of Qubits"));
			panOut.add(_TF_numBits);
		panOut.add(new JLabel("Number of Elem. Ops."));
			panOut.add(_TF_lenOfSEO);
//		pan5.add(new JLabel("Error"));
//			pan5.add(_TF_error);
		panOut.setBorder(new TitledBorder(borderStyle, "Outputs"));


		JPanel panMidOut = new JPanel(new BorderLayout(2, 2));//hgap, vgap;
		panMidOut.add(panMid, BorderLayout.NORTH);
		panMidOut.add(panOut, BorderLayout.CENTER);

		setLayout(new BorderLayout(2, 2));//hgap, vgap;
		add(panHeader, BorderLayout.NORTH);
		add(panIn, BorderLayout.CENTER);
		add(panMidOut, BorderLayout.SOUTH);
		
		_B_doPrerun.addActionListener(new DoPrerunButtonListener());
		_L_startingState.addListSelectionListener(
				_L_listener = new SS_ListSelectionListener());
		_B_readCFiles.addActionListener(new ReadCfilesButtonListener());
		_B_randomStartingState.addActionListener(
				new RandomStartButtonListener());
		_CB_nodeStates.addItemListener(
				_CB_listener = new NodeStatesComboListener());
		_B_computeSEO.addActionListener(new WriteQfilesButtonListener());
	}
	/**
	 * 
	 */	
	private void removeListeners(){
		_L_startingState.removeListSelectionListener(_L_listener);
		_CB_nodeStates.removeItemListener(_CB_listener);
	}
	/**
	 * 
	 */	
	private void restoreListeners(){
		_L_startingState.addListSelectionListener(_L_listener =
			new SS_ListSelectionListener());
		_CB_nodeStates.addItemListener(_CB_listener =
			new NodeStatesComboListener());
	}
	/**
	 * 
	 */	
	private void changeNodeStatesComboBox(
			int newFocusNode,
			int newSelectedNodeState)//use -1 if no change of selection
	{
		removeListeners();
		
		_CB_nodeStates.removeAllItems();
		NamesList stateNames = 
			_net.getNode(newFocusNode).getStateNamesList();
		int j, numStates = stateNames.getNumNames();
		for(j=0; j<numStates; j++){
			_CB_nodeStates.addItem(stateNames.getName(j));		
		}
		if(newSelectedNodeState>=0){
			_CB_nodeStates.setSelectedIndex(newSelectedNodeState);
		}

		restoreListeners();
	}
	/**
	 *
	 */
	private void changeSS_List(
			int[] newStartingNodeStates,
			int newSelectedNode)//use -1 if no change of selection
	{
		
		removeListeners();
		_startingNodeStates = newStartingNodeStates;
		_L_startingState.setListData(
				_net.nodeStatesArrayToStrings(_startingNodeStates));
		if(newSelectedNode>=0){
			_L_startingState.setSelectedIndex(newSelectedNode);
		}
		_prob0 = _net.getProb(_startingNodeStates);
		_gam0 = 2*Math.acos(Math.sqrt(_prob0));
		_TF_probStartingState.setText(String.format("%e",_prob0));
		_TF_gam0.setText(String.format("%e", _gam0*180/Math.PI));
		_TF_finGamma.setText("");
		_TF_numGroverSteps.setText("");
		_TF_numBits.setText("");
		_TF_lenOfSEO.setText("");
	
		
		restoreListeners();
	}
	/**
	 * 
	 */	
	private void clearOutputTextFields(){
		_TF_probStartingState.setText("");
		_TF_gam0.setText("");
		_TF_finGamma.setText("");
		_TF_numGroverSteps.setText("");
		_TF_numBits.setText("");
		_TF_lenOfSEO.setText("");	
	}
	/**
	 * 
	 */	
	private class SS_ListSelectionListener implements ListSelectionListener{

		public void valueChanged(ListSelectionEvent eve) {
			int newSelectedNode = _L_startingState.getSelectedIndex();
			Throw.If(newSelectedNode<0, "no node selected");
			int newSelectedNodeState = _startingNodeStates[newSelectedNode];
			changeNodeStatesComboBox(newSelectedNode, newSelectedNodeState);
		}
	}
	/**
	 * 
	 */
	private class DoPrerunButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent eve) {
			try{
				if(	_TF_folder.getText().length()==0){
					throw new Exception();
				}
				
				_relPath = _TF_folder.getText() + "/";
				CB_Net net = new CB_Net(_relPath + "states.txt", 
						_relPath + "parents.txt");
				net.writeProbsFile(
						_relPath + "probsT.txt", 
						true);			//varyInStatesFirst		
			
				net.writeProbsFile(
					_relPath + "probsF.txt", 
					false);			//varyInStatesFirst
				net.writeMBlanketsFile(_relPath + "blankets.txt");
				net.writeNitsFile(_relPath + "nits.txt");
			}
			catch(Exception ex){
				clearOutputTextFields();
				JOptionPane.showMessageDialog(null, 
						"Quibbs got BAD INPUT:\n" + ex.toString());
				return;
			}
		}
	}
	/**
	 * 
	 */
	private class ReadCfilesButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent eve) {
			try{
				if(	_TF_folder.getText().length()==0){
					throw new Exception();
				}
				
				_relPath = _TF_folder.getText() + "/";

				_net = new CB_Net(_relPath + "states.txt", 
					_relPath + "parents.txt",
					_relPath + "probs.txt");
				int j, numNodes = _net.getNumNodes();
				int[] newStartingNodeStates = new int[numNodes];
				for(j=0; j<numNodes; j++){
					newStartingNodeStates[j] = 0;
				}
				int newSelectedNode = 0;
				changeSS_List(newStartingNodeStates, newSelectedNode);
				int newSelectedNodeState = 
					_startingNodeStates[newSelectedNode];
				changeNodeStatesComboBox(newSelectedNode, 
						newSelectedNodeState);
				
			}
			catch(Exception ex){
				clearOutputTextFields();
				JOptionPane.showMessageDialog(null, 
						"Quibbs got BAD INPUT:\n" + ex.toString());
				return;
			}
		}
	}
	/**
	 *
	 */
	private class RandomStartButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {
			try{
				if(_startingNodeStates==null)return;
				int[] newStartingNodeStates = _net.getRandomNodeStatesArray();
				int newSelectedNode = 0;
				changeSS_List(newStartingNodeStates, newSelectedNode);
				changeNodeStatesComboBox(newSelectedNode, 
						_startingNodeStates[newSelectedNode]);
			}
			catch(Exception ex){
				clearOutputTextFields();
				JOptionPane.showMessageDialog(null, 
						"Quibbs got BAD INPUT:\n" + ex.toString());
				return;
			}
		}
	}	
	/**
	 *
	 */
	private class NodeStatesComboListener implements ItemListener{

		public void itemStateChanged(ItemEvent eve) {
			try{
				if(_startingNodeStates==null || 
						_CB_nodeStates.getItemCount()==0)return;
				int selectedState = _CB_nodeStates.getSelectedIndex();
				Throw.If(selectedState<0, "no state selected");
				int selectedNode = _L_startingState.getSelectedIndex();
				Throw.If(selectedNode<0, "no selected node");
				int j, numNodes = _net.getNumNodes();
				int[] newStartingNodeStates = new int[numNodes];
				for(j=0; j<numNodes; j++){
					newStartingNodeStates[j] =_startingNodeStates[j];
				}
				
				newStartingNodeStates[selectedNode] = selectedState;
				changeSS_List(newStartingNodeStates, 
						selectedNode);
			}
			catch(Exception ex){
				clearOutputTextFields();
				JOptionPane.showMessageDialog(null, 
						"Quibbs got BAD INPUT:\n" + ex.toString());
				return;
			}

		}
	}
	/**
	 *
	 */
	private class WriteQfilesButtonListener implements ActionListener{

		public void actionPerformed(ActionEvent eve) {
			
			try{
				if(	_TF_numProbeBits.getText().length()==0 ||
					_TF_numPESteps.getText().length()==0||
					_TF_maxNumGroverSteps.getText().length()==0||
					_TF_gamTol.getText().length()==0 ||
					_TF_deltaLam.getText().length()==0){ 
					throw new Exception();
				}

				int numProbeBits = Integer.parseInt(
						_TF_numProbeBits.getText());
				int numPESteps = Integer.parseInt(
						_TF_numPESteps.getText());
				int maxNumGroverSteps = Integer.parseInt(
						_TF_maxNumGroverSteps.getText());
				double gamTol =  Double.parseDouble(
						_TF_gamTol.getText())*Math.PI/180;
				double deltaLam =  Double.parseDouble(
						_TF_deltaLam.getText())*Math.PI/180;
							
				QuibbsMain  m = new QuibbsMain(
						_net,//don't read net again
					    _relPath,
						_startingNodeStates,
					    numProbeBits,
					    numPESteps,
					    maxNumGroverSteps,
						-1,//calculate true gam0
						gamTol,
					    deltaLam,
						_CB_omitStairs.isSelected());
				_TF_finGamma.setText(String.format("%e", m.getFinGamma()*180/Math.PI));
				_TF_numGroverSteps.setText(String.format("%d", m.getNumGroverSteps()));				
				int numBits = 2*_net.getNumNits() + numProbeBits*numPESteps;
				_TF_numBits.setText(String.format("%d", numBits));
				_TF_lenOfSEO.setText(String.format("%d", m.getLenOfSEO()));
//				_TF_error.setText(String.format("%e", m.getError()));
			}
			catch(Exception ex){
				_TF_numBits.setText("");
				_TF_lenOfSEO.setText("");
	//			_TF_error.setText("");
				JOptionPane.showMessageDialog(null, 
						"Quibbs got BAD INPUT:\n" + ex.toString());
				return;
			}
		}

	}
	/**
	 *
	 */
	public static void main(String[] args) {
	    
	    JFrame frame = new JFrame("Quibbs");

	    
	    QuibbsApplet applet = new QuibbsApplet();

	    frame.add(applet, BorderLayout.CENTER);

	    applet.init();

	    //frame.setLocationRelativeTo(null); // Center the frame
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	    
	    frame.pack();
	    //frame.setSize(400, 250);
	    frame.setVisible(true);
	  }
}


