import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;


/**
 * This class's main() method is the applet sans gui.
 * Quibbs = Quantum Gibbs
 */
public class QuibbsMain {
	private CB_Net	_net;
	private double _finGamma;
	private int _numGroverSteps;
	private int		_lenOfSEO;
//	private double 	_error;

	/**
	 *
	 */
	public QuibbsMain(
			CB_Net	net,
	        String 	relPath,
			int[] 	startingNodeStates,
	    	int 	numProbeBits,
	    	int  	numPESteps,
	    	int 	maxNumGroverSteps,
			double	gam0,
			double 	gamTol,
			double 	deltaLam,
			boolean omitStairs)
	{
    	if(net==null){
    		net = new CB_Net(relPath + "states.txt", 
				relPath + "parents.txt",
				relPath + "probs.txt");
    	}
    	_net = net;

		QuibbsSEO_writer  wr = new QuibbsSEO_writer(
			_net,
			relPath,
			startingNodeStates,
			numProbeBits,
			numPESteps,
			maxNumGroverSteps,
			gam0,
			gamTol,
			deltaLam,
			omitStairs);
        _finGamma = wr.getFinGamma();
        _numGroverSteps = wr.getNumGroverSteps();
 		
		String filePrefixPlus = relPath + "quibbs";
    	
    	SEO_countOnlyReader	reader = new SEO_countOnlyReader(
    			filePrefixPlus);
    	_lenOfSEO = reader.getLenOfSEO();
    	//System.out.println(reader);
    	
    	
    	PrintWriter   logOut = null;
    	try{
	        File  logFile = new File(filePrefixPlus + "_log.txt");
	        logOut = new PrintWriter(logFile);
    	}
    	catch(FileNotFoundException ex){
            ex.printStackTrace();
        }
    	logOut.println("Inputs:");

        logOut.println("I/O Folder = " 
        		+ relPath.substring(0,relPath.length()-1));//delete / at end
        logOut.println("Starting State:");
        int j, numNodes = _net.getNumNodes();
        String[] strArray = _net.nodeStatesArrayToStrings(
        		wr.getStartingNodeStates());
        for(j=0; j<numNodes; j++){
        	 logOut.println(strArray[j]);
        }
        logOut.println("Number Of Probe Bits (for each PE step) = " 
        		 + numProbeBits);
        logOut.println("Number Of Phase Estimation (PE) Steps = " + numPESteps);
        logOut.println("Maximum Number of Grover Steps = " + maxNumGroverSteps);
        if(gam0>=0){
        	logOut.println("Enforced (inputted instead of outputted) Starting Gamma (degs) = " 
        			+ gam0*180/Math.PI);
        }
        logOut.println("Gamma Tolerance (degs) = " + gamTol*180/Math.PI);
        logOut.println("Delta Lambda (degs) = " + deltaLam*180/Math.PI);
       
        logOut.println("");
        logOut.println("Outputs:");
		double prob0 = _net.getProb(wr.getStartingNodeStates());
        logOut.println("Prob. of Starting State = " + prob0);
        logOut.println("Starting Gamma (degs) = " + 
        		2*Math.acos(Math.sqrt(prob0))*180/Math.PI);
        logOut.println("Final Gamma (degs) = " + _finGamma*180/Math.PI);
        logOut.println("Number of Grover Steps = " + _numGroverSteps);
        int numBits = 2*_net.getNumNits() + numProbeBits*numPESteps;
        logOut.println("Number of Qubits = " + numBits);
        logOut.println("Number of Elem. Ops. = " + _lenOfSEO);
 //       logOut.printf("Error = %e\n", _error);
        logOut.close();
   	
	}
	
	/**
	 *
	 */
	public double getFinGamma(){
		return _finGamma;
	}
	/**
	 *
	 */
	public int getNumGroverSteps() {
		return _numGroverSteps;
	}
	/**
	 *
	 */
	public int getLenOfSEO() {
		return _lenOfSEO;
	}
	/**
	 *
	 */
//	public double getError(){
//		return _error;
//	}
	/**
	 *
	 */
    public static void main(String[] args){
   
    	//startingNodeStates rules:
    	//if startingNodeStates==null, will reset to random
		//if startingNodeStates=={0}, will reset all components to zero
    	//Exception will be thrown for illegal startingNodeStates
    	
		//String 	relPath = "2nodes/"; int[] startingNodeStates = {2,1};
		//String 	relPath = "3nodes/"; 
		//String 	relPath = "4nodeFullyConnected/";
		String 	relPath = "Asia/";
		
		//int[] startingNodeStates=null;

		//sometimes random startingNodeStates has zero probability
		//here is one choice for Asia net that has nonzero prob
		int[] startingNodeStates={1,1,0,1,0,1,1,0};

		
		int numProbeBits = 3;
		int numPESteps = 3;
    	int maxNumGroverSteps = 10;
//		double gam0Degs = 173.15;
    	double gam0Degs = -5;//neg means not imposed as input
		double gamTolDegs = .05;
		double deltaLamDegs = 150.0;

			
		new QuibbsMain(
			null, //net
		    relPath,
			startingNodeStates,
		    numProbeBits,
		    numPESteps,
		    maxNumGroverSteps,
			gam0Degs*Math.PI/180,
			gamTolDegs*Math.PI/180,
		    deltaLamDegs*Math.PI/180,
			false);//omitStairs=true used for debugging only
 
    }
}
