import java.util.*;
import java.io.*;

/**
 * This class performs a quick and dirty translation
 * of a Netica network report (a txt file)
 * into Quibbs input files.
 * inputs: net-report.txt, key.txt
 * ouput files: states.txt, parents.txt and probs.txt
 */

public class Netica2Quibbs {
	
	public static void translate(String keyFileName, String reportFileName){

		int nd, numNodes=0;
		String[] nodeName = null;
		int[] numParents = null;
		int[] numStates = null;

		try{
			Scanner input = new Scanner(new File(keyFileName));
			numNodes = Integer.parseInt(input.next());
			nodeName = new String[numNodes];
			numParents = new int[numNodes];
			numStates = new int[numNodes];
			for(nd=0; nd<numNodes; nd++){
				nodeName[nd] = input.next();
				numStates[nd] = Integer.parseInt(input.next());	
				numParents[nd] = Integer.parseInt(input.next());						
			}
			input.close();	
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}
		
		int pa, numPar, st, numSta;
		String[] stateNames;
		String[] parentNames;
		String[] parStateNames;
		double[] probsForGivenParStates;

		try{
			Scanner input = new Scanner(new File(reportFileName));
			PrintWriter statesOut = new PrintWriter(new File("states.txt"));
			PrintWriter parentsOut = new PrintWriter(new File("parents.txt"));
			PrintWriter probsOut = new PrintWriter(new File("probs.txt"));
			for(nd=0; nd<numNodes; nd++){
				input.next();//skip node name and colon
				numPar = numParents[nd];
				numSta = numStates[nd];
				parentNames = new String[numPar];
				stateNames = new String[numSta];
				parStateNames = new String[numPar];
				probsForGivenParStates = new double[numSta];
				
				
				for(st=0; st<numSta; st++){
					stateNames[st] = input.next();
				}
				statesOut.println("#\t" + nodeName[nd]);
				for(st=0; st<numSta;st++){
					statesOut.println(stateNames[st]);
				}
				
				for(pa=0; pa<numPar; pa++){
					parentNames[pa] = input.next();
				}
				parentsOut.println("#\t" + nodeName[nd]);
				for(pa=0; pa<numPar;pa++){
					parentsOut.println(parentNames[pa]);
				}
				
				probsOut.println("#\t" + nodeName[nd]);
				while(input.hasNext()&&!input.hasNext( 
						java.util.regex.Pattern.compile(".*:"))){ 
					for(st=0; st<numSta; st++){
						probsForGivenParStates[st] = 
							Double.parseDouble(input.next());
					}
					for(pa=0; pa<numPar;pa++){
						parStateNames[pa] = input.next();
					}
					for(st=0; st<numSta; st++){
						probsOut.print(stateNames[st] + "\t");
						for(pa=0; pa<numPar;pa++){
							probsOut.print(parStateNames[pa] + "\t");
						}
						probsOut.println(probsForGivenParStates[st]);
					}
				}				
			}
			input.close();
			parentsOut.close();
			statesOut.close();
			probsOut.close();
		}
		catch(FileNotFoundException ex){
    		System.out.println("\n" + ex.toString());
        }
	}
	/**
	 * 
	 */
	public static void main(String[] args){
		//first line of key.txt is number of nodes
		//Thereafter, for each node, a line with: 
		//(1)node name (2) num of states (3)num of parents, 
		//separated by white space.
		String keyFileName = "key-neta.txt";
		String reportFileName = "probs-neta.txt";
		
		translate(keyFileName, reportFileName);
	}

}
