import java.io.*;
import java.util.Scanner;

/**
 * 	This class translates 
 *  Picture and English Files outputted by Quibbs
 *  to a julich-english file that uses
 *  simple operations understood by the Julich QC simulator
 */

public class Quibbs2Julich {
    private Scanner  _englishIn = null;
    private PrintWriter  _englishOut = null;
    private int _numBitsOut;
    private double _globalPhaseRads = 0;//global phase in radians
    int			_lenOfSEO = 0;
    boolean 	_testing=false;

    /**
     * 
     */
	public Quibbs2Julich(
			String filePrefix,
			int muxorCompiMode,
			int bitPrecision)
	{
				
		ElemOpExpander curExpander; //cur = current
		
		curExpander = new MultiplexorExpander(
				filePrefix,
				"1noMuxor",//filePrefixOut
				0,//numGitsIn
				muxorCompiMode,//compiMode
				//MultiplexorExpander.ORACULAR_APPROX,//compiMode
				bitPrecision);
		
		curExpander = new MultiCRotExpander(
				"1noMuxor",//filePrefixIn
				"2noMCRot",//filePrefixOut,
				curExpander.getNumGitsOut());//numGitsIn
		
		curExpander = new MultiCNotExpander(
				"2noMCRot",//filePrefixIn
				"3noMCNot",//filePrefixOut
				curExpander.getNumGitsOut(),//numGitsIn
				MultiCNotExpander.TOFFOLIS);//compiMode
		
		curExpander = new LoopExpander(
				"3noMCNot",//filePrefixIn
				"4noLoop");//filePrefixOut
		
		_numBitsOut = curExpander.getNumBitsOut();
		
		translate("4noLoop", filePrefix);
	}
    /**
     * 
     */
	public Quibbs2Julich(int numBitsOut)
	{
		//for testing translate() only
		_testing = true;
		_numBitsOut = numBitsOut;
		translate("transTest", "transTest");
	}
    /**
     * 
     */
	public int getNumBitsOut(){
		return _numBitsOut;
	}
    /**
     * 
     */
	public int getLenOfSEO(){
		return _lenOfSEO;
	}
    /**
     * 
     */
	public double getGlobalPhaseRads(){
		return _globalPhaseRads;
	}

	/**
     * 
     */
	private void translate(String filePrefixIn, String filePrefixOut)
	{
		try{
			_englishIn = new Scanner(new File(filePrefixIn + "_eng.txt"));
			_englishOut = new PrintWriter(
					new File(filePrefixOut + "_jul.txt"));

			int tarBitPos;
			String lineName = "";
			while(_englishIn.hasNext()){
				lineName = _englishIn.next();
				if(lineName.equals("SWAP")){
					readWriteJulSWAP();
				}else if(lineName.equals("PHAS")){
					//pure, unconditioned phase
					double angleDeg = _englishIn.nextDouble();
					_englishIn.nextLine();//skip rest
					_globalPhaseRads += angleDeg*Math.PI/180;
					_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
					//no writing
				}else if(lineName.equals("P0PH")){
					readWriteJulP_PH(0);					
				}else if(lineName.equals("P1PH")){
					readWriteJulP_PH(1);
				}else if(lineName.equals("SIGX")){
					readWriteJulSIGX();
				}else if(lineName.equals("SIGY")){
					_englishIn.next();//skip "AT"
					tarBitPos = _englishIn.nextInt();
					//writing
					writeJulSIGY(tarBitPos);
				}else if(lineName.equals("SIGZ")){
					_englishIn.next();//skip "AT"
					tarBitPos = _englishIn.nextInt();
					//writing
					writeJulSIGZ(tarBitPos);
				}else if(lineName.equals("HAD2")){
					readWriteHAD2();				
				}else if(lineName.equals("ROTX")){
					readWriteJulROT(1);					
				}else if(lineName.equals("ROTY")){
					readWriteJulROT(2);
				}else if(lineName.equals("ROTZ")){
					readWriteJulROT(3);
				}else if(lineName.equals("ROTN")){
					readWriteJulROTN();
			 
				}else{
					Throw.If(true, "unsupported line name");
				}
				if(_testing==true){_englishOut.println("");}
					
			}
			_englishIn.close();
			_englishOut.close();
		}
		catch(FileNotFoundException ex){
			System.out.println("\n" + ex.toString());
		}	
	}
	/**
     * 
     */
	private void compensateForP0Trols(TFControls trols){
		int c, numTrols = trols.getNumControls();
		for(c=0; c<numTrols; c++){
			if(trols.getType(c)==false){
				writeJulSIGX(trols.getBitPos(c));
			}
		}		
	}
	/**
     * 
     */
	private void writeJulP1PH(double angleDeg, int tarBitPos){
		_lenOfSEO++; _englishOut.println(
			"R\t" + 
			tarBitPos + "\t" + 
			angleDeg);
	}
	/**
     * 
     */
	private void writeJulCP1PH(
			double angleDeg, int tarBitPos, TFControls trols)
	{
		compensateForP0Trols(trols);
		_lenOfSEO++; _englishOut.println(
			"U\t" + 
			trols.getBitPos(0) + "\t" +
			tarBitPos + "\t" +
			angleDeg);
		compensateForP0Trols(trols);
	}
	/**
     * 
     */
	private void writeJulSIGX(int tarBitPos){
		writeJulROTX(90.0, tarBitPos);
		_globalPhaseRads -= Math.PI/2;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}                                      
	private void writeJulSIGY(int tarBitPos){
		writeJulROTY(90.0, tarBitPos);
		_globalPhaseRads -= Math.PI/2;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}                                      
	private void writeJulSIGZ(int tarBitPos){
		writeJulROTZ(90.0, tarBitPos);
		_globalPhaseRads -= Math.PI/2;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}
	/**
     * 
     */
	private void writeJulROTX(double angleDeg, int tarBitPos){
		_lenOfSEO++; _englishOut.println(
				"X\t" + 
				tarBitPos + "\t" + 
				angleDeg);
	}                                      
	private void writeJulROTY(double angleDeg, int tarBitPos){
		_lenOfSEO++; _englishOut.println(
				"Y\t" + 
				tarBitPos + "\t" + 
				angleDeg);
	}                                      
	private void writeJulROTZ(double angleDeg, int tarBitPos){
		writeJulP1PH(-2*angleDeg, tarBitPos);
		_globalPhaseRads -= 2*angleDeg*Math.PI/180;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}                                      
	/**
     * 
     */
	private void writeJulCROTX(
			double angleDeg, int tarBitPos, TFControls trols)
	{
		compensateForP0Trols(trols);
		_lenOfSEO++; _englishOut.println(
			"V\t" + 
			trols.getBitPos(0) + "\t" +
			tarBitPos + "\t" + 
			-2*angleDeg);
		compensateForP0Trols(trols);
		_globalPhaseRads -= 2*angleDeg*Math.PI/180;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}                                      
	/**
     * 
     */
	private void writeJulCROTY(
			double angleDeg, int tarBitPos, TFControls trols)
	{
		writeJulROTX(-45, tarBitPos);
		writeJulCROTZ(angleDeg, tarBitPos, trols);
		writeJulROTX(45, tarBitPos);
	}                                      
	/**
     * 
     */
	private void writeJulCROTZ(
			double angleDeg, int tarBitPos, TFControls trols)
	{
		writeJulCP1PH(-2*angleDeg, tarBitPos, trols);
		_globalPhaseRads -= 2*angleDeg*Math.PI/180;
		_globalPhaseRads =Globals.limitedRadians(_globalPhaseRads);
	}                                      
	/**
     * 
     */
	private void readWriteJulSWAP(){
		int tarBitPos1 = _englishIn.nextInt();
		int tarBitPos2 = _englishIn.nextInt();
		//writing
		_lenOfSEO++; _englishOut.println(
				"SWAP\t1\t" + 
				tarBitPos1 + "\t" + 
				tarBitPos2);
	}	
	/**
     * 
     */
	private void readWriteJulP_PH(int projectionBit){
		double angleDeg = _englishIn.nextDouble();
		_englishIn.next();//skip "AT"
		int tarBitPos = _englishIn.nextInt();
		TFControls trols = SEO_reader.readTFControls(_numBitsOut, _englishIn);
		int numTrols = trols.getNumControls();
		//writing
		if(projectionBit==0){writeJulSIGX(tarBitPos);}
		if(numTrols==0){			
			writeJulP1PH(angleDeg, tarBitPos);
		}else if(numTrols==1){
			writeJulCP1PH(angleDeg, tarBitPos, trols);
		}else{
			Throw.If(true, "expected <=1 controls");
		}
		if(projectionBit==0){writeJulSIGX(tarBitPos);}
	}
	/**
     * 
     */
	private void readWriteHAD2()
	{
		_englishIn.next();//skip "AT"
		int tarBitPos = _englishIn.nextInt();
		//writing
		_lenOfSEO++; _englishOut.println("H\t" + tarBitPos);
	}
	/**
     * 
     */
	private void readWriteJulSIGX()
	{
		_englishIn.next();//skip "AT"
		int tarBitPos = _englishIn.nextInt();
		TFControls trols = SEO_reader.readTFControls(_numBitsOut, _englishIn);
		int numTrols = trols.getNumControls();
		//writing
		compensateForP0Trols(trols);
		if(numTrols==0){
			writeJulSIGX(tarBitPos);
		}else if(numTrols==1){
			_lenOfSEO++; _englishOut.println(
				"CNOT\t" +
				trols.getBitPos(0) + "\t" +
				tarBitPos);
		}else if(numTrols==2){
			_lenOfSEO++; _englishOut.println(
				"T\t" +
				trols.getBitPos(0) + "\t" +
				trols.getBitPos(1) + "\t" +
				tarBitPos);
		}else{
			Throw.If(true, "expected <=2 controls");
		}
		compensateForP0Trols(trols);
	}
	/**
     * 
     */
	private void readWriteJulROT(int direction){
		double angleDeg = _englishIn.nextDouble();
		_englishIn.next();//skip "AT"
		int tarBitPos = _englishIn.nextInt();
		TFControls trols = SEO_reader.readTFControls(_numBitsOut, _englishIn);
		int numTrols = trols.getNumControls();
		//writing
		if(numTrols==0){
			if(direction==1){
				writeJulROTX(angleDeg, tarBitPos);
			}else if(direction==2){
				writeJulROTY(angleDeg, tarBitPos);
			}else if(direction==3){
				writeJulROTZ(angleDeg, tarBitPos);
			}else{
				Throw.If(true, "bogus direction");
			}
		}else if(numTrols==1){
			if(direction==1){
				writeJulCROTX(angleDeg, tarBitPos, trols);
			}else if(direction==2){
				writeJulCROTY(angleDeg, tarBitPos, trols);
			}else if(direction==3){
				writeJulCROTZ(angleDeg, tarBitPos, trols);
			}else{
				Throw.If(true, "bogus direction");
			}

		}else{
			Throw.If(true, "expected <=1 controls");
		}
	}
	/**
     * 
     */
	private void readWriteJulROTN()
	{	
		double angleX_deg = _englishIn.nextDouble();
    	double angleY_deg = _englishIn.nextDouble();
    	double angleZ_deg = _englishIn.nextDouble();			
    		double angleX_rad = angleX_deg*Math.PI/180;
    		double angleY_rad = angleY_deg*Math.PI/180;
    		double angleZ_rad = angleZ_deg*Math.PI/180;			
    	_englishIn.next();//skip "AT"
    	int tarBitPos = _englishIn.nextInt();
    	TFControls trols = SEO_reader.readTFControls(_numBitsOut, _englishIn);
    	int numTrols = trols.getNumControls();
    	//writing
    	double[] angRadOut = ROTN_expansion.getAngles(
    			angleX_rad, angleY_rad, angleZ_rad);
    	if(numTrols==0){
    		writeJulROTX(angRadOut[0]*180/Math.PI, tarBitPos);
    		writeJulROTZ(angRadOut[1]*180/Math.PI, tarBitPos);
    		writeJulROTX(angRadOut[2]*180/Math.PI, tarBitPos);
    		writeJulROTZ(angRadOut[3]*180/Math.PI, tarBitPos);
    		writeJulROTX(angRadOut[4]*180/Math.PI, tarBitPos);
    	}else if(numTrols==1){
    		writeJulROTX(
    				angRadOut[0]*180/Math.PI, tarBitPos);
    		writeJulROTZ(
    				angRadOut[1]*180/Math.PI, tarBitPos);
    		writeJulCROTX(
    				angRadOut[2]*180/Math.PI, tarBitPos, trols);
    		writeJulROTZ(
    				angRadOut[3]*180/Math.PI, tarBitPos);
    		writeJulROTX(
    				angRadOut[4]*180/Math.PI, tarBitPos);
    	}else{
    		Throw.If(true, "expected <=1 controls");
    	}
	}
	/**
	 *
	 */
    public static void main(String[] args){
    	
    	boolean test = false;
    	int muxorCompiMode =MultiplexorExpander.EXACT_SEO;
    	//int muxorCompiMode =MultiplexorExpander.ORACULAR_APPROX;
    	int bitPrecision = 2;//for oracular approx
    	
    	
     	//String 	netName = "2nodes";
		//String 	netName = "3nodes";
		//String 	netName = "4nodeFullyConnected";
		String 	netName = "Asia";
		//String 	netName = "Tree4Level";
		//String 	netName = "net9-2";

		String 	relPath = "../Quibbs/" + netName + "/";
		String	filePrefix = "quibbs";
   
		Quibbs2Julich translator;
    	if(test){
    		translator = new Quibbs2Julich(6/*numBitsOut*/);
    	}else{
    		translator = new Quibbs2Julich(
    			relPath + filePrefix,
    			muxorCompiMode,
    			bitPrecision);
    	}
    	
    	System.out.println(
    		"Number of Qubits = " + translator.getNumBitsOut());
    	System.out.println(
    		"Length of SEO = " + translator.getLenOfSEO());
    	System.out.println(
    		"Global Phase (degs) = " + 
    		translator.getGlobalPhaseRads()*180/Math.PI);
    	
	   

    }
	
}
