import java.io.PrintWriter;


/**
 * JenStartEvOp = Jennings Start Evolution Operator
 * This class writes English and Picture files
 * for the U such that U|0>=|s>.
 * |s> is the AFGA starting state in the paper
 * ``Quantum Circuit for Discovering from Data the
 * Structure of Classical Bayesian Networks" by R.R. Tucci, arXiv:?
 */
 

public class JenStartEvOpSEO_writer  extends SEO_writer{
	private int _numNodes;//the number of nodes of the graph being discovered
	private int _maxNumPa;//maximum number of parents of the graph, 
						//is <= (numNodes-1)

	/**
	 * 
	 */	
	public JenStartEvOpSEO_writer(
	        String 	filePrefix,
	        int numNodes,
	        int maxNumPa)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numNodes = numNodes;
		_maxNumPa = maxNumPa;
		_numBits = JenMain.calcNumBits(_numNodes, _maxNumPa);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public JenStartEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numNodes,
        int maxNumPa)
    {
    	super(englishOut, pictureOut, emb);
		_numNodes = numNodes;
		_maxNumPa = maxNumPa;
		_numBits = JenMain.calcNumBits(_numNodes, _maxNumPa);
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
    private CktEmbedder getVMatEmb(int combLen){
    	int numBitsBef = _numNodes*Combinations.get_nCr(_numNodes-1, combLen);
    	int numBitsAft = _numBits;
    	int[] bitMap = new int[numBitsBef];
    	int k;
    	int boost = _numNodes;
    	for(k=0; k<combLen; k++){
    		boost += _numNodes*Combinations.get_nCr(_numNodes-1, k);
    	}
    	for(k=0; k<numBitsBef; k++){
    		bitMap[k] = k + boost;
    	}
    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				bitMap,
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
		
	}
	/**
	 * 
	 */
    public void write(){
     	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
     	
     	int combLen;
		for(combLen=0; combLen<=_maxNumPa; combLen++){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getVMatEmb(combLen),        
    			false);//mtype,
		}
		new JenGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numNodes,
	        _maxNumPa);
		for(combLen=_maxNumPa; combLen>=0; combLen--){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getVMatEmb(combLen),        
    			false).writeHermitian();//mtype
		}
		
    	new JenTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numNodes,
	        _maxNumPa);
		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
     	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);

    	new JenTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			false, //doWrite
			simpleEmb,        
			_numNodes,
	        _maxNumPa).writeHermitian();

     	int combLen;
		for(combLen=0; combLen<=_maxNumPa; combLen++){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getVMatEmb(combLen),        
    			false);//mtype,
		}
		new JenGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			false, //doWrite
			simpleEmb,        
			_numNodes,
	        _maxNumPa).writeHermitian();
		for(combLen=_maxNumPa; combLen>=0; combLen--){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getVMatEmb(combLen),        
    			false).writeHermitian();//mtype
		}
		
		
	}

	/**
	 * 
	 */
    public static void main(String[] args){

 
    	
    	int numNodes = 3; 
    	int maxNumPa = 2;  	
        int numBits = JenMain.calcNumBits(numNodes, maxNumPa);
 
    	String filePrefix = "start-ev-op";
    	
    	JenStartEvOpSEO_writer  w = new JenStartEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numNodes,
    			maxNumPa);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

