import java.io.PrintWriter;

/**
 * This class writes English and Picture files
 * for: (1) the controlled gate that targets the omega qubit in |s>
 * (lonely sig_x acting on omega  qubit is also included), 
 * and (2) The cnot that targets the mu_0 qubit.
 * |s> is the AFGA starting state in the paper
 * ``Quantum Circuit for Discovering from Data the
 * Structure of Classical Bayesian Networks" by R.R. Tucci, arXiv:?
 */

public class JenTargetingSEO_writer  extends SEO_writer{
	private int _numNodes;//the number of nodes of the graph being discovered
	private int _maxNumPa;//maximum number of parents of the graph, 
						//is <= (numNodes-1)

	/**
	 * 
	 */	
	public JenTargetingSEO_writer(
	        String 	filePrefix,
	        int numNodes,
	        int maxNumPa)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numNodes = numNodes;
		_maxNumPa = maxNumPa;
		_numBits = JenMain.calcNumBits(_numNodes, _maxNumPa);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public JenTargetingSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numNodes,
        int maxNumPa)
    {
    	super(englishOut, pictureOut, emb);
		_numNodes = numNodes;
		_maxNumPa = maxNumPa;
		_numBits = JenMain.calcNumBits(_numNodes, _maxNumPa);
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	private void writeOmegaGate(){
		
		int numTrols = JenMain.calcNumBits(_numNodes, _maxNumPa) - 3;
		TFControls trols = new TFControls(_numBits, numTrols);
		
		int trolPos = _numBits -4;
		int c = 0;
		while(c<numTrols-_numNodes){
			trols.setControl(c, trolPos, false);
			trolPos--;
			c++;
		}
		while(c<numTrols){
			trols.setControl(c, trolPos, true);
			trolPos--;
			c++;
		}
		writeControlledOneBitGate(
				_numBits - 1, //targetBitPosition, 
				trols, 
				new SigX());
	}
	/**
	 * 
	 */
	public void write(){
		writeOneBitGate(_numBits -1, new SigX());
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOmegaGate();
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		writeOmegaGate();
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOneBitGate(_numBits -1, new SigX());
	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	int numNodes = 3;
    	int maxNumPa = 2;
    	
        int numBits = JenMain.calcNumBits(numNodes, maxNumPa);
  
    	String filePrefix = "targ";
    	
    	JenTargetingSEO_writer  w = new JenTargetingSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    	        numNodes,
    	       maxNumPa);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}


