import java.io.PrintWriter;


/**
 * This class writes English and Picture files
 * for the gamma qubit controlled portion in |s>.
 * (lonely H acting on gamma qubit is also included)
 * |s> is the AFGA starting state in Margi section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MargiGammaControlledSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi
	private int _numMargiBits;// the number of marginal qubits
	/**
	 * 
	 */	
	public MargiGammaControlledSEO_writer(
	        String 	filePrefix,
	        int numPsiBits,
	        int numMargiBits)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numMargiBits = numMargiBits;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MargiGammaControlledSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numPsiBits,
        int numMargiBits)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = numPsiBits;
		_numMargiBits = numMargiBits;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
 	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	public void write(){
		int gammaBit = _numBits-3;
		
		writeOneBitGate(gammaBit, new OneBitHadamard());
		TFControls trols = new TFControls(_numBits, 3);
		trols.setControl(0, gammaBit, true);
		for(int j=0; j<_numMargiBits; j++){
			trols.setControl(1, _numPsiBits + j , false);
			trols.setControl(2, j, true);
			writeControlledOneBitGate(
					_numPsiBits + _numMargiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
			trols.setControl(1, _numPsiBits + j , true);
			trols.setControl(2, j, false);
			writeControlledOneBitGate(
					_numPsiBits + _numMargiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		int gammaBit = _numBits-3;
		
		TFControls trols = new TFControls(_numBits, 3);
		trols.setControl(0, gammaBit, true);
		for(int j=_numMargiBits-1; j>=0; j--){
			trols.setControl(1, _numPsiBits + j , true);
			trols.setControl(2, j, false);
			writeControlledOneBitGate(
					_numPsiBits + _numMargiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
			trols.setControl(1, _numPsiBits + j , false);
			trols.setControl(2, j, true);
			writeControlledOneBitGate(
					_numPsiBits + _numMargiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
		}
		writeOneBitGate(gammaBit, new OneBitHadamard());

	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	int numPsiBits = 7;
    	int numMargiBits = 3;
        int numBits = MargiMain.calcNumBits(numPsiBits, numMargiBits);
  
    	String filePrefix = "gam-con";

    	MargiGammaControlledSEO_writer  w = new MargiGammaControlledSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numPsiBits,
    			numMargiBits);

    	w.writeHermitian();
    	w.closeFiles();
    }
}

