import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;


/**
 * This class's main() method is the applet sans gui.
 * Margi = Marginalizer
 */


public class MargiMain {
	private double _P0;
	private double _finGamma;
	private int _numGroverSteps;
	private int		_lenOfSEO;
//	private double 	_error;

	/**
	 *
	 */
	public MargiMain(
	        String 	filePrefix,
	        int numPsiBits,
	        Boolean[] cVec,
	        double estP1overP0,
	    	int 	maxNumGroverSteps,
			double	gam0,
			double 	gamTol,
			double 	deltaLam)									
	{

		String filePrefixPlus = filePrefix + "_qMargi";

		MargiSEO_writer  wr = new MargiSEO_writer(
				filePrefixPlus,
				numPsiBits,
		        cVec,
		        estP1overP0,
		    	maxNumGroverSteps,
				gam0,
				gamTol,
				deltaLam);
		_P0 = wr.getP0();
        _finGamma = wr.getFinGamma();
        _numGroverSteps = wr.getNumGroverSteps();
		
		
		int numMargiBits = cVec.length;
		int numBits = MargiMain.calcNumBits(numPsiBits, numMargiBits);
		
		
    	
    	SEO_countOnlyReader	reader = new SEO_countOnlyReader(
    			filePrefixPlus);
    	_lenOfSEO = reader.getLenOfSEO();
    	//System.out.println(reader);
    	
    	
    	PrintWriter   logOut = null;
    	try{
	        File  logFile = new File(filePrefixPlus + "_log.txt");
	        logOut = new PrintWriter(logFile);
    	}
    	catch(FileNotFoundException ex){
            ex.printStackTrace();
        }
    	logOut.println("Inputs:");
        logOut.println("File prefix = " + filePrefix);
        logOut.println("Number Of Psi Qubits = " + numPsiBits);
        logOut.println("Number Of Marginal Qubits = " + numMargiBits);
        logOut.println("c vector = ");
        for(int r=0; r<numMargiBits; r++){
        	logOut.println((cVec[r]==true?1:0));
        }        
        logOut.println("Estimate of |z_1|^2 over |z_0|^2 = " + estP1overP0);

        logOut.println("Maximum Number of Grover Steps = " + maxNumGroverSteps);
        if(gam0>=0){
        	logOut.println("Enforced (inputted instead of outputted) Starting Gamma (degs) = " 
        			+ gam0*180/Math.PI);
        }
        logOut.println("Gamma Tolerance (degs) = " + gamTol*180/Math.PI);
        logOut.println("Delta Lambda (degs) = " + deltaLam*180/Math.PI);
       
        logOut.println("");
        logOut.println("Outputs: ");
		double  prob_st = _P0*(1 + estP1overP0);
		Throw.If(estP1overP0<0 || prob_st>1,
			"The estimate of |z_1|^2 / |z_0|^2 must be > 0 and < "
			+ (1/_P0 - 1));
        logOut.println("|z_0|^2 = " + _P0);
        logOut.println("Starting Gamma (degs) = " + 
        		2*Math.acos(Math.sqrt(prob_st))*180/Math.PI);
        logOut.println("Final Gamma (degs) = " + _finGamma*180/Math.PI);
        logOut.println("Number of Grover Steps = " + _numGroverSteps);
        logOut.println("Number of Qubits = " + numBits);
        logOut.println("Number of Elem. Ops. = " + _lenOfSEO);
 //       logOut.printf("Error = %e\n", _error);
        logOut.close();
   	
	}
	/**
	 *
	 */
	public double getP0(){
		return _P0;
	}	
	
	/**
	 *
	 */
	public double getFinGamma(){
		return _finGamma;
	}
	/**
	 *
	 */
	public int getNumGroverSteps() {
		return _numGroverSteps;
	}
	/**
	 *
	 */
	public int getLenOfSEO() {
		return _lenOfSEO;
	}
	/**
	 *
	 */
//		public double getError(){
//			return _error;
//		}
	/**
	 * 
	 */
    public static int calcNumBits(int numPsiBits, int numMargiBits){
		Throw.If(numMargiBits<1 , 
				"must have at least one marginal bit"); 
		
		Throw.If(numPsiBits<=numMargiBits , 
				"must have at least one summed over bit"); 
    	return numPsiBits + 2*numMargiBits
				+ 3;//for gamma, mu_0 and omega  qubits
    	
    }
	/**
	 *
	 */
    public static void main(String[] args){
  
   	
    	String 	filePrefix = "test2";
    	
    	int numPsiBits = 6;
	
     	Boolean[] cVec={true, false, true, false};

    	double estP1overP0 = 1.5;
		
		int maxNumGroverSteps = 50;
//		double gam0Degs = 173.15;
    	double gam0Degs = -5;//neg means not imposed as input
		double gamTolDegs = .5;
		double deltaLamDegs = 170.0;

			
		new MargiMain(
		        filePrefix,
		        numPsiBits,
		        cVec,
		        estP1overP0,
			    maxNumGroverSteps,
				gam0Degs*Math.PI/180,
				gamTolDegs*Math.PI/180,
			    deltaLamDegs*Math.PI/180);

   }
}
