import java.io.PrintWriter;

/**
 * MargiPsiEvOp = Marginalizer Psi Evolution Operator
 * This class is a simple extension of
 * the abstract class  PsiEvOpSEO_writer.
 * It's used to override the abstract methods 
 * write() and writeHermitian(). 
 * The |psi> here refers to the one that appears in the |s>.
 * |s> is the AFGA starting state in Margi section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */



public class MargiPsiEvOpSEO_writer extends PsiEvOpSEO_writer{

	public MargiPsiEvOpSEO_writer(
	        String 	filePrefix,
	        int numQudits)
	{ 	 	
		super(filePrefix, numQudits, 1 /*bitsPerDit*/);
	}
	/**
	 * 
	 */
    public MargiPsiEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numQudits)
    {
    	super(englishOut, pictureOut, 
    			doWrite, emb, numQudits, 1 /*bitsPerDit*/);

    }
	/**
	 * 
	 */
	/**
	 * calculates |<c|psi>|^2
	 */
	//static public double getProbAtC(Boolean[] cVec){}
    //not used
	public void write(){
		//This is just a trivial example.
		//Should replace this method's code with code
		//that applies to situation being considered.
		for(int j=0; j<_numBits; j++){
			writeOneBitGate(j,
					new OneBitRot(-j*Math.PI/18,OneBitRot.X_AXIS)); 
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		//This is just a trivial example.
		//Should replace this method's code with code
		//that applies to situation being considered.
		for(int j=_numBits-1; j>=0; j--){
			writeOneBitGate(j,
					new OneBitRot(j*Math.PI/18,OneBitRot.X_AXIS)); 
		}

	}
	/**
	 * 
	 */
	public static void main(String[] args){
    	
    	int numQudits = 4; 
    	int numBits = numQudits;
 
    	String filePrefix = "psiOp";
    	
    	MargiPsiEvOpSEO_writer  w = new MargiPsiEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numQudits);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

