import java.io.PrintWriter;


/**
 * MargiStartEvOp = Marginalizer Start Evolution Operator
 * This class writes English and Picture files
 * for the U such that U|0>=|s>.
 * |s> is the AFGA starting state in Margi section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */
 

public class MargiStartEvOpSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi
	private int _numMargiBits;// the number of marginal qubits
	private Boolean[] _cVec;//dimension is numPsiBits

	/**
	 * 
	 */	
	public MargiStartEvOpSEO_writer(
	        String 	filePrefix,
	        int numPsiBits,
	        Boolean[] cVec)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numMargiBits = cVec.length;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
		_cVec = new Boolean[_numMargiBits];
		for(int j=0; j<_numMargiBits; j++){
			_cVec[j] = cVec[j];

		}
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MargiStartEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
		int numPsiBits,
        Boolean[] cVec)
    {
    	super(englishOut, pictureOut, emb);
    	_numPsiBits = numPsiBits;
		_numMargiBits = cVec.length;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
		_cVec = new Boolean[_numMargiBits];
		for(int j=0; j<_numMargiBits; j++){
			_cVec[j] = cVec[j];

		}
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
    private CktEmbedder getPsiEmb(){	
    	int numBitsBef = _numPsiBits;
    	int numBitsAft = _numBits;

    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				Globals.identityPermutation(numBitsBef),//bitMap
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
	}
	/**
	 * 
	 */
    public void write(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
 
		new MargiPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getPsiEmb(),        
    			_numPsiBits);
		for(int j=0; j<_numMargiBits; j++){
			writeOneBitGate(_numPsiBits + j, new OneBitHadamard());
		}
		new MargiGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numPsiBits,
			_numMargiBits);
		
    	new MargiTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,
			_numPsiBits,
			_cVec);
		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
    	 		
    	new MargiTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			false, //doWrite
			simpleEmb, 
			_numPsiBits,
			_cVec).writeHermitian();
		new MargiGammaControlledSEO_writer(
				_englishOut,
				_pictureOut,
				false, //doWrite
				simpleEmb,        
				_numPsiBits,
				_numMargiBits).writeHermitian();
		for(int j=_numMargiBits-1; j>=0; j--){
			writeOneBitGate(_numPsiBits + j, new OneBitHadamard());
		}
		new MargiPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getPsiEmb(),        
    			_numPsiBits).writeHermitian();
		
	}

	/**
	 * 
	 */
    public static void main(String[] args){
    	
    	int numPsiBits = 7;

     	Boolean[] cVec={true, true, true, false};

    	
    	int numMargiBits = cVec.length; 
        int numBits = MargiMain.calcNumBits(numPsiBits, numMargiBits);
 
    	String filePrefix = "start-ev-op";
    	
    	MargiStartEvOpSEO_writer  w = new MargiStartEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),
    			numPsiBits,
    			cVec);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

