import java.io.PrintWriter;

/**
 * This class writes English and Picture files
 * for: (1) the controlled gate that targets the omega qubit in |s>
 * (lonely sig_x acting on omega  qubit is also included), 
 * and (2) The cnot that targets the mu_0 qubit.
 * |s> is the AFGA starting state in Margi section of the paper
 * ``Quantum Circuit for Calculating Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MargiTargetingSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi
	private int _numMargiBits;// the number of marginal qubits
	private Boolean[] _cVec;//dimension is numPsiBits

	/**
	 * 
	 */	
	public MargiTargetingSEO_writer(
	        String 	filePrefix,
	        int numPsiBits,
	        Boolean[] cVec)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numMargiBits = cVec.length;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
		_cVec = new Boolean[_numMargiBits];
		for(int j=0; j<_numMargiBits; j++){
			_cVec[j] = cVec[j];
		}
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MargiTargetingSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
		int numPsiBits,
        Boolean[] cVec)
    {
    	super(englishOut, pictureOut, emb);
    	_numPsiBits = numPsiBits;
		_numMargiBits = cVec.length;
		_numBits = MargiMain.calcNumBits(_numPsiBits, _numMargiBits);
		_cVec = new Boolean[_numMargiBits];
		for(int j=0; j<_numMargiBits; j++){
			_cVec[j] = cVec[j];
		}
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	private void writeOmegaGate(){
		
		int numTrols = 2*_numMargiBits;
		TFControls trols = new TFControls(_numBits, numTrols);
		
		int trolPos = _numBits -4;
		int c =0;
		while(c<_numMargiBits){
			trols.setControl(c, trolPos, false);
			trolPos--;
			c++;
		}
		while(c<2*_numMargiBits){
			trols.setControl(c, trolPos, _cVec[trolPos-_numPsiBits]);
			trolPos--;
			c++;
		}
		writeControlledOneBitGate(
				_numBits - 1, //targetBitPosition, 
				trols, 
				new SigX());
	}
	/**
	 * 
	 */
	public void write(){
		writeOneBitGate(_numBits -1, new SigX());
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOmegaGate();
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		writeOmegaGate();
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOneBitGate(_numBits -1, new SigX());
	}

	/**
	 * 
	 */
    public static void main(String[] args){
    	int numPsiBits = 7;
    	Boolean[] cVec={false, true, true};
    	
    	int numMargiBits = cVec.length; 
        int numBits = MargiMain.calcNumBits(numPsiBits, numMargiBits);
  
    	String filePrefix = "targ";
    	
    	MargiTargetingSEO_writer  w = new MargiTargetingSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),
    			numPsiBits,
    			cVec);

    	w.writeHermitian();
    	w.closeFiles();
    	
		for(int j=numMargiBits-1; j>=0; j--){
				System.out.printf("%d",cVec[j]==true?1:0);
		}

    }

}


