import java.io.PrintWriter;


/**
 * This class writes English and Picture files
 * for the gamma qubit controlled portion in |s>.
 * (lonely H acting on gamma qubit is also included)
 * |s> is the AFGA starting state in the paper
 * ``Quantum Circuit for Calculating Mean Values
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MeanGammaControlledSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi

	/**
	 * 
	 */	
	public MeanGammaControlledSEO_writer(
	        String 	filePrefix,
	        int numPsiBits)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MeanGammaControlledSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numPsiBits)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);
 	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	public void write(){
		int gammaBit = _numBits-3;
		
		writeOneBitGate(gammaBit, new OneBitHadamard());
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, gammaBit, true);
		for(int j=0; j<_numPsiBits; j++){
			writeControlledOneBitGate(
					j, //targetBitPosition, 
					trols, 
					new OneBitHadamard());
			writeControlledOneBitGate(
					_numPsiBits + j, //targetBitPosition, 
					trols, 
					new OneBitHadamard());
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		int gammaBit = _numBits-3;
		
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, gammaBit, true);
		for(int j=_numPsiBits-1; j>=0; j--){
			writeControlledOneBitGate(
					_numPsiBits + j, //targetBitPosition, 
					trols, 
					new OneBitHadamard());
			writeControlledOneBitGate(
					j, //targetBitPosition, 
					trols, 
					new OneBitHadamard());
		}
		writeOneBitGate(gammaBit, new OneBitHadamard());

	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	int numPsiBits = 3;
        int numBits = MeanMain.calcNumBits(numPsiBits);
  
    	String filePrefix = "gam-con";

    	MeanGammaControlledSEO_writer  w = new MeanGammaControlledSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numPsiBits);

    	w.writeHermitian();
    	w.closeFiles();
    }
}

