import java.io.PrintWriter;


/**
 * MeanStartEvOp = Mean Start Evolution Operator
 * This class writes English and Picture files
 * for the U such that U|0>=|s>.
 * |s> is the AFGA starting state in the paper
 * ``Quantum Circuit for Calculating Mean Values
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */
 

public class MeanStartEvOpSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi

	/**
	 * 
	 */	
	public MeanStartEvOpSEO_writer(
	        String 	filePrefix,
	        int numPsiBits)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);

		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MeanStartEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numPsiBits)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
    private CktEmbedder getPsiEmb(){	
    	int numBitsBef = _numPsiBits;
    	int numBitsAft = _numBits;

    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				Globals.identityPermutation(numBitsBef),//bitMap
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
	}
	/**
	 * 
	 */
    public void write(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
 
		new MeanPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getPsiEmb(),        
    			_numPsiBits);
		new MeanGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numPsiBits);
		
    	new MeanTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numPsiBits);
		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
    	 		
    	new MeanTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			false, //doWrite
			simpleEmb,        
			_numPsiBits).writeHermitian();
		new MeanGammaControlledSEO_writer(
				_englishOut,
				_pictureOut,
				false, //doWrite
				simpleEmb,        
				_numPsiBits).writeHermitian();
		new MeanPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getPsiEmb(),        
    			_numPsiBits).writeHermitian();
		
	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	
    	int numPsiBits = 3; 
        int numBits = MeanMain.calcNumBits(numPsiBits);
 
    	String filePrefix = "start-ev-op";
    	
    	MeanStartEvOpSEO_writer  w = new MeanStartEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numPsiBits);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

