import java.io.*;

/**
 * This class writes English and Picture files
 * for: (1) the controlled gate that targets the omega qubit in |s>
 * (lonely sig_x acting on omega  qubit is also included), 
 * and (2) The cnot that targets the mu_0 qubit.
 * |s> is the AFGA starting state in the paper
 * ``Quantum Circuit for Calculating Mean Values
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MeanTargetingSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi

	/**
	 * 
	 */	
	public MeanTargetingSEO_writer(
	        String 	filePrefix,
	        int numPsiBits)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MeanTargetingSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numPsiBits)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = numPsiBits;
		_numBits = MeanMain.calcNumBits(_numPsiBits);
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	private void writeOmegaGate(){
		
		int numTrols = 2*_numPsiBits;
		TFControls trols = new TFControls(_numBits, numTrols);
		
		int trolPos = _numBits -4;
		int c =0;
		while(c<_numPsiBits*2){
			trols.setControl(c, trolPos, false);
			trolPos--;
			c++;
		}
		writeControlledOneBitGate(
				_numBits - 1, //targetBitPosition, 
				trols, 
				new SigX());
	}
	/**
	 * 
	 */
	public void write(){
		writeOneBitGate(_numBits -1, new SigX());
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOmegaGate();
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		writeOmegaGate();
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOneBitGate(_numBits -1, new SigX());
	}

	/**
	 * 
	 */
    public static void main(String[] args){

     	
    	int numPsiBits = 3; 
        int numBits = MeanMain.calcNumBits(numPsiBits);
  
    	String filePrefix = "targ";
    	
    	MeanTargetingSEO_writer  w = new MeanTargetingSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numPsiBits);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}




