import java.io.*;


/**
 * This class writes English and Picture files
 * for the gamma qubit controlled portion in |s>.
 * (lonely H acting on gamma qubit is also included)
 * |s> is the AFGA starting state in Mobius section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MobGammaControlledSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi

	/**
	 * 
	 */	
	public MobGammaControlledSEO_writer(
	        String 	filePrefix,
	        int numPsiBits)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = numPsiBits;
		_numBits = MobMain.calcNumBits(_numPsiBits);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MobGammaControlledSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numPsiBits)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = numPsiBits;
		_numBits = MobMain.calcNumBits(_numPsiBits);
 	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	public void write(){
		int gammaBit = _numBits-3;
		
		writeOneBitGate(gammaBit, new OneBitHadamard());
		TFControls trols = new TFControls(_numBits, 3);
		trols.setControl(0, gammaBit, true);
		for(int j=0; j<_numPsiBits; j++){
			trols.setControl(1, _numPsiBits + j , false);
			trols.setControl(2, j, true);
			writeControlledOneBitGate(
					2*_numPsiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		int gammaBit = _numBits-3;
		
		TFControls trols = new TFControls(_numBits, 3);
		trols.setControl(0, gammaBit, true);
		for(int j=_numPsiBits-1; j>=0; j--){
			trols.setControl(1, _numPsiBits + j , false);
			trols.setControl(2, j, true);
			writeControlledOneBitGate(
					2*_numPsiBits + j, //targetBitPosition, 
					trols, 
					new SigX());
		}
		writeOneBitGate(gammaBit, new OneBitHadamard());

	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	int numPsiBits = 4;
        int numBits = MobMain.calcNumBits(numPsiBits);
  
    	String filePrefix = "gam-con";

    	MobGammaControlledSEO_writer  w = new MobGammaControlledSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numPsiBits);

    	w.writeHermitian();
    	w.closeFiles();
    }
}

