import java.io.PrintWriter;


/**
 * MobStartEvOp = Mobius Start Evolution Operator
 * This class writes English and Picture files
 * for the U such that U|0>=|s>.
 * |s> is the AFGA starting state in Mobius section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */
 

public class MobStartEvOpSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi
	private Boolean[] _cVec;//dimension is numPsiBits

	/**
	 * 
	 */	
	public MobStartEvOpSEO_writer(
	        String 	filePrefix,
	        Boolean[] cVec)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = cVec.length;
		_numBits = MobMain.calcNumBits(_numPsiBits);
		_cVec = new Boolean[_numPsiBits];
		for(int j=0; j<_numPsiBits; j++){
			_cVec[j] = cVec[j];

		}
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MobStartEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        Boolean[] cVec)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = cVec.length;
		_numBits = MobMain.calcNumBits(_numPsiBits);
		_cVec = new Boolean[_numPsiBits];
		for(int j=0; j<_numPsiBits; j++){
			_cVec[j] = cVec[j];

		}
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
    private CktEmbedder getPsiEmb(){	
    	int numBitsBef = _numPsiBits;
    	int numBitsAft = _numBits;

    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				Globals.identityPermutation(numBitsBef),//bitMap
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
	}
	/**
	 * 
	 */
    public void write(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
 
		new MobPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getPsiEmb(),        
    			_numPsiBits);
		for(int j=0; j<_numPsiBits; j++){
			writeOneBitGate(_numPsiBits + j, new OneBitHadamard());
		}
		new MobGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numPsiBits);
		
    	new MobTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_cVec);
		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
    	 		
    	new MobTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			false, //doWrite
			simpleEmb,        
			_cVec).writeHermitian();
		new MobGammaControlledSEO_writer(
				_englishOut,
				_pictureOut,
				false, //doWrite
				simpleEmb,        
				_numPsiBits).writeHermitian();
		for(int j=_numPsiBits-1; j>=0; j--){
			writeOneBitGate(_numPsiBits + j, new OneBitHadamard());
		}
		new MobPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getPsiEmb(),        
    			_numPsiBits).writeHermitian();
		
	}

	/**
	 * 
	 */
    public static void main(String[] args){

     	Boolean[] cVec={true, true, true, false};

    	
    	int numPsiBits = cVec.length; 
        int numBits = MobMain.calcNumBits(numPsiBits);
 
    	String filePrefix = "start-ev-op";
    	
    	MobStartEvOpSEO_writer  w = new MobStartEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			cVec);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

