import java.io.*;

/**
 * This class writes English and Picture files
 * for: (1) the controlled gate that targets the omega qubit in |s>
 * (lonely sig_x acting on omega  qubit is also included), 
 * and (2) The cnot that targets the mu_0 qubit.
 * |s> is the AFGA starting state in Mobius section of the paper
 * ``Quantum Circuit for Calculating Mobius-like Transforms
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class MobTargetingSEO_writer  extends SEO_writer{
	private int _numPsiBits;// the number of qubits in psi
	private Boolean[] _cVec;//dimension is numPsiBits

	/**
	 * 
	 */	
	public MobTargetingSEO_writer(
	        String 	filePrefix,
	        Boolean[] cVec)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numPsiBits = cVec.length;
		_numBits = MobMain.calcNumBits(_numPsiBits);
		_cVec = new Boolean[_numPsiBits];
		for(int j=0; j<_numPsiBits; j++){
			_cVec[j] = cVec[j];
		}
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public MobTargetingSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        Boolean[] cVec)
    {
    	super(englishOut, pictureOut, emb);
		_numPsiBits = cVec.length;
		_numBits = MobMain.calcNumBits(_numPsiBits);
		_cVec = new Boolean[_numPsiBits];
		for(int j=0; j<_numPsiBits; j++){
			_cVec[j] = cVec[j];
		}
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	private void writeOmegaGate(){
		
		int numTrols = 2*_numPsiBits;
		TFControls trols = new TFControls(_numBits, numTrols);
		
		int trolPos = _numBits -4;
		int c =0;
		while(c<_numPsiBits){
			trols.setControl(c, trolPos, false);
			trolPos--;
			c++;
		}
		while(c<2*_numPsiBits){
			trols.setControl(c, trolPos, _cVec[trolPos-_numPsiBits]);
			trolPos--;
			c++;
		}
		writeControlledOneBitGate(
				_numBits - 1, //targetBitPosition, 
				trols, 
				new SigX());
	}
	/**
	 * 
	 */
	public void write(){
		writeOneBitGate(_numBits -1, new SigX());
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOmegaGate();
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		writeOmegaGate();
		TFControls trols = new TFControls(_numBits, 1);
		trols.setControl(0, _numBits - 3, true);
		writeControlledOneBitGate(
				_numBits - 2, //targetBitPosition, mu_0
				trols, 
				new SigX());
		writeOneBitGate(_numBits -1, new SigX());
	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	Boolean[] cVec={false, true, true};
    	
    	int numPsiBits = cVec.length; 
        int numBits = MobMain.calcNumBits(numPsiBits);
  
    	String filePrefix = "targ";
    	
    	MobTargetingSEO_writer  w = new MobTargetingSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			cVec);

    	w.writeHermitian();
    	w.closeFiles();
    	
		for(int j=numPsiBits-1; j>=0; j--){
				System.out.printf("%d",cVec[j]==true?1:0);
		}

    }

}


