import java.io.*;


/**
 * This class writes English and Picture files
 * for the gamma qubit controlled portion in |s>.
 * (lonely H acting on gamma qubit is also included)
 * |s> is the AFGA starting state for Method A in the paper
 * ``Quantum Circuit for Calculating Symmetrized Functions
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class SymGammaControlledSEO_writer  extends SEO_writer{
	private int _numQudits;// the number of qudits in alpha
	private int _bitsPerDit;// the number of qubits in each qudit

	/**
	 * 
	 */	
	public SymGammaControlledSEO_writer(
	        String 	filePrefix,
	        int numQudits,
	        int bitsPerDit)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numQudits = numQudits;
		_bitsPerDit = bitsPerDit;
		_numBits = SymMain.calcNumBits(_numQudits, _bitsPerDit);
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public SymGammaControlledSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numQudits,
        int bitsPerDit)
    {
    	super(englishOut, pictureOut, emb);
		_numQudits = numQudits;
		_bitsPerDit = bitsPerDit;
		_numBits = SymMain.calcNumBits(_numQudits, _bitsPerDit);
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
	public void write(){
		int floorDit, roofDit;
		int subBit, bit1, bit2;
		int gammaBit = _numBits-3;
		
		writeOneBitGate(gammaBit, new OneBitHadamard());
		TFControls trols = new TFControls(_numBits, 2);
		trols.setControl(0, gammaBit, true);
		int swapCounter = _numQudits*_bitsPerDit-1;
		for(floorDit=1; floorDit<_numQudits; floorDit++){
			for(roofDit=floorDit-1; roofDit>=0; roofDit--){
				swapCounter++;
				for(subBit=_bitsPerDit-1; subBit>=0; subBit--){
					bit1= subBit + roofDit*_bitsPerDit;
					bit2= subBit + floorDit*_bitsPerDit;
					trols.setControl(1, swapCounter, true);
					writeControlledBitSwap(bit1, bit2, trols);
				}				
			}
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		int floorDit, roofDit;
		int subBit, bit1, bit2;
		int gammaBit = _numBits-3;
		
		TFControls trols = new TFControls(_numBits, 2);
		trols.setControl(0, gammaBit, true);
		int swapCounter = _numBits -3;
		for(floorDit=_numQudits-1; floorDit>=0; floorDit--){
			for(roofDit=0; roofDit<floorDit; roofDit++){
				swapCounter--;
				for(subBit=0; subBit<_bitsPerDit; subBit++){
					bit1= subBit + roofDit*_bitsPerDit;
					bit2= subBit + floorDit*_bitsPerDit;
					trols.setControl(1, swapCounter, true);
					writeControlledBitSwap(bit1, bit2, trols);
				}
			}
		}
		writeOneBitGate(gammaBit, new OneBitHadamard());

	}

	/**
	 * 
	 */
    public static void main(String[] args){

    	int numQudits = 4; int bitsPerDit = 3;
    	//int numQudits = 4; int bitsPerDit = 1;
        int numBits = SymMain.calcNumBits(numQudits, bitsPerDit);
  
    	String filePrefix = "gam-con";

    	SymGammaControlledSEO_writer  w = new SymGammaControlledSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numQudits,
    	        bitsPerDit);

    	w.writeHermitian();
    	w.closeFiles();
    }
}

