import java.io.PrintWriter;

/**
 * SymPsiEvOp = Symmetrizer Psi Evolution Operator
 * This class is a simple extension of
 * the abstract class  PsiEvOpSEO_writer.
 * It's used to override the abstract methods 
 * write() and writeHermitian(). 
 * The |psi> here refers to the one that appears in the |s>.
 * |s> is the AFGA starting state for Method A in the paper
 * ``Quantum Circuit for Calculating Symmetrized Functions
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */



public class SymPsiEvOpSEO_writer extends PsiEvOpSEO_writer{

	public SymPsiEvOpSEO_writer(
	        String 	filePrefix,
	        int numQudits,
	        int bitsPerDit)
	{ 	 	
		super(filePrefix, numQudits, bitsPerDit);
	}
	/**
	 * 
	 */
    public SymPsiEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        int numQudits,
        int bitsPerDit)
    {
    	super(englishOut, pictureOut, 
    			doWrite, emb, numQudits, bitsPerDit);

    }
	/**
	 * 
	 */
	/**
	 * calculates |<c|psi>|^2
	 */
	static public double getProbAtC(Boolean[][] cMat){
		//This is just a trivial example.
		//Should replace this method's code with code
		//that applies to situation being considered.
		
		//Must have \sum_c P(c) = 1	
		int numQudits = cMat.length;
		int bitsPerDit = cMat[0].length;

		return 1.0/(1<<(numQudits*bitsPerDit));
	}
	public void write(){
		//This is just a trivial example.
		//Should replace this method's code with code
		//that applies to situation being considered.
		for(int j=0; j<_numBits; j++){
			writeOneBitGate(j,
					new OneBitRot(-j*Math.PI/18,OneBitRot.X_AXIS)); 
		}
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
		//This is just a trivial example.
		//Should replace this method's code with code
		//that applies to situation being considered.
		for(int j=_numBits-1; j>=0; j--){
			writeOneBitGate(j,
					new OneBitRot(j*Math.PI/18,OneBitRot.X_AXIS)); 
		}

	}
	/**
	 * 
	 */
	public static void main(String[] args){
    	
    	int numQudits = 4; 
    	int bitsPerDit = 3;  
    	int numBits = numQudits*bitsPerDit;
 
    	String filePrefix = "psiOp";
    	
    	SymPsiEvOpSEO_writer  w = new SymPsiEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			numQudits,
    			bitsPerDit);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

