import java.io.PrintWriter;


/**
 * SymStartEvOp = Symmetrizer Start Evolution Operator
 * This class writes English and Picture files
 * for the U such that U|0>=|s>.
 * |s> is the AFGA starting state for Method A in the paper
 * ``Quantum Circuit for Calculating Symmetrized Functions
 * Via Grover-like algorithm" by R.R. Tucci, arXiv:?
 */

public class SymStartEvOpSEO_writer  extends SEO_writer{
	private int _numQudits;// the number of qudits in alpha
	private int _bitsPerDit;// the number of qubits in each qudit
	private Boolean[][] _cMat;//dimension is numQudits X bitsPerDit

	/**
	 * 
	 */	
	public SymStartEvOpSEO_writer(
	        String 	filePrefix,
	        Boolean[][] cMat)
	{ 	 	
		super(filePrefix, 2 /*_numBits, temporary*/);
		_numQudits = cMat.length;
		_bitsPerDit = cMat[0].length;
		_numBits = SymMain.calcNumBits(_numQudits, _bitsPerDit);
		_cMat = new Boolean[_numQudits][_bitsPerDit];
		for(int j=0; j<_numQudits; j++){
			for(int k=0; k<_bitsPerDit; k++){
				_cMat[j][k] = cMat[j][k];
			}
		}
		
		_emb = new CktEmbedder(_numBits);
    	write();

        closeFiles();

	}
	/**
	 * 
	 */
    public SymStartEvOpSEO_writer(
		PrintWriter  englishOut,
		PrintWriter  pictureOut,
		Boolean doWrite,
		CktEmbedder emb,
        Boolean[][] cMat)
    {
    	super(englishOut, pictureOut, emb);
		_numQudits = cMat.length;
		_bitsPerDit = cMat[0].length;
		_numBits = SymMain.calcNumBits(_numQudits, _bitsPerDit);
		_cMat = new Boolean[_numQudits][_bitsPerDit];
		for(int j=0; j<_numQudits; j++){
			for(int k=0; k<_bitsPerDit; k++){
				_cMat[j][k] = cMat[j][k];
			}
		}
  	
    	if(doWrite){write();}
    }
	/**
	 * 
	 */
    private CktEmbedder getPsiEmb(){	
    	int numBitsBef = _numQudits*_bitsPerDit;
    	int numBitsAft = _numBits;

    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				Globals.identityPermutation(numBitsBef),//bitMap
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
	}
	/**
	 * 
	 */
    private CktEmbedder getVMatEmb(int numActiveBits){
    	int numBitsBef = numActiveBits;
    	int numBitsAft = _numBits;
    	int[] bitMap = new int[numBitsBef];
    	int boost = _numQudits*_bitsPerDit
    			+ (numActiveBits*(numActiveBits-1))/2;
    	for(int k=0; k<numBitsBef; k++){
    		bitMap[k] = k + boost;
    	}
    	return new  CktEmbedder(
				numBitsBef,
				numBitsAft,
				bitMap,
				Globals.F_vec(numBitsAft),//isP0Control
				Globals.F_vec(numBitsAft));//isP1Control
		
	}
	/**
	 * 
	 */
    public void write(){
     	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);
 
		int numActBits;

		new SymPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getPsiEmb(),        
    			_numQudits,
    			_bitsPerDit);
		for(numActBits=1; numActBits<_numQudits; numActBits++){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getVMatEmb(numActBits),        
    			true);//mtype,
		}
		new SymGammaControlledSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_numQudits,
	        _bitsPerDit);
		for(numActBits=_numQudits-1; numActBits>=1; numActBits--){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getVMatEmb(numActBits),        
    			true).writeHermitian();//mtype
		}
		
    	new SymTargetingSEO_writer(
			_englishOut,
			_pictureOut,
			true, //doWrite
			simpleEmb,        
			_cMat);
		
	}
	/**
	 * writes Hermitian Conjugate
	 */
	public void writeHermitian(){
    	CktEmbedder  simpleEmb = new CktEmbedder(_numBits);

		int numActBits;
		
    	new SymTargetingSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			simpleEmb,        
    			_cMat).writeHermitian();

		for(numActBits=1; numActBits<_numQudits; numActBits++){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			true, //doWrite
    			getVMatEmb(numActBits),        
    			true);//mtype,
		}
		new SymGammaControlledSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			simpleEmb,        
    			_numQudits,
    	        _bitsPerDit).writeHermitian();
		for(numActBits=_numQudits-1; numActBits>=1; numActBits--){
			new EntanglingMatSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getVMatEmb(numActBits),        
    			true).writeHermitian();//mtype
		}
		
		new SymPsiEvOpSEO_writer(
    			_englishOut,
    			_pictureOut,
    			false, //doWrite
    			getPsiEmb(),        
    			_numQudits,
    			_bitsPerDit).writeHermitian();
	}

	/**
	 * 
	 */
    public static void main(String[] args){

 /*   	
 	Boolean[][] cMat={
    			{true, false, true},
    			{true, true, false},
    			{false, false, false},
    			{false, true, true}			
    	};
  */
     	Boolean[][] cMat={
    			{true},
    			{true},
    			{false},
    			{false}			
    	};

    	
    	int numQudits = cMat.length; 
    	int bitsPerDit = cMat[0].length;  	
        int numBits = SymMain.calcNumBits(numQudits, bitsPerDit);
 
    	String filePrefix = "start-ev-op";
    	
    	SymStartEvOpSEO_writer  w = new SymStartEvOpSEO_writer(
    			openEngFile(filePrefix),
    			openPicFile(filePrefix),
    			true, //doWrite
    			new CktEmbedder(numBits),        
    			cMat);

    	w.writeHermitian();
    	w.closeFiles();
    	
    }
}

